﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0008
*/
DQ.lazyLoad(DQ.subdir + "dq-core.js", "DQ.Window", function () {
    DQ.MapRelation = function (data) {
        for (var nm in data) {
            this[nm] = data[nm];
        }
    }
    DQ.MapRelation.prototype = {
        load: function (pos) {
            if (arguments.length) {
                world.map.loadMap(this.url, pos);
            } else {
                world.map.loadMap(this.url);
            }
            if (this.flag != -1) {
                world.flag[this.flag] = true;
            }
        }
    }

    DQ.MapCatalog = function (options) {
        /// <summary>
        /// </summary>

        options = DQ.options(options, {
            url: null,
            callback: null
        });

        this.onLoaded = options.callback;
        this._url = options.url;
        this._url && this.load(this._url);
    }

    DQ.MapCatalog.prototype = {
        catalog: [],
        onLoaded: function () { },
        load: function (url, callback) {
            var me = this;
            DQ.loadCatalog(url, function (data) {
                eval('var s=' + data);
                me._data = s.catalog;
                me.catalog.length = 0;
                me.version = s.version;
                me.catalog = new Array();
                for (var i = 0; i < me._data.length; i++) {
                    me.catalog[me._data[i].uid] = new DQ.MapRelation(me._data[i]);
                }
                debug.writeln(DQ.format("map loading completed:", url));
                callback && callback(me);
                me.onLoaded && me.onLoaded.apply(me);
            });
        },
        find: function (key, value) {
            if (arguments.length == 0) {
                throw new Error("Invalid arguments.");
            }
            if (arguments.length == 1) {
                value = key;
                key = "name";
            }
            for (var i = 10001; i < this.catalog.length; i++) {
                if (this.catalog[i][key] == value) {
                    return this.catalog[i];
                }
            }

            return null;
        }
    }

    DQ.Location = function (x, y) {
        this.current.x = x;
        this.current.y = y;
        this.target = [];
        this.checked = [];
        this.dir = [];
    }

    DQ.Location.prototype = {
        backup: function () {
            this.prePosition.x = this.current.x;
            this.prePosition.y = this.current.y;
            this.prePosition.xh = this.current.xh;
            this.prePosition.yh = this.current.yh;
        },
        left: function () {
            this.backup();
            if (this.current.xh == 0) {
                this.current.xh--;
                this.checked[[-1, this.current.xh]];
            } else {
                this.current.x--;
                this.current.xh = 0;
                delete this.checked[[-1, 0]];
                delete this.checked[[-1, -1]];
                delete this.checked[[-1, 1]];
            }
        },
        right: function () {
            this.backup();
            if (this.current.xh == 0) {
                this.current.xh++;
                this.checked[[1, this.current.xh]];
            } else {
                this.current.x++;
                this.current.xh = 0;
                delete this.checked[[1, 0]];
                delete this.checked[[1, -1]];
                delete this.checked[[1, 1]];
            }
        },
        up: function () {
            this.backup();
            if (this.current.yh == 0) {
                this.current.yh--;
                this.checked[[this.current.xh, -1]];
            } else {
                this.current.y--;
                this.current.yh = 0;
                delete this.checked[[0, -1]];
                delete this.checked[[-1, -1]];
                delete this.checked[[1, -1]];
            }
        },
        down: function () {
            this.backup();
            if (this.current.yh == 0) {
                this.current.yh++;
                this.checked[[this.current.xh, 1]];
            } else {
                this.current.y++;
                this.current.yh = 0;
                delete this.checked[[0, 1]];
                delete this.checked[[-1, 1]];
                delete this.checked[[1, 1]];
            }
        },
        makeTarget: function (dir) {
            /// <param name="dir" type="DQ.DIR">
            /// 向き
            /// </param>

            var me = this;
            this.target.length = 0;
            switch (dir) {
                case DQ.DIR.LEFT:
                    if (this.current.xh != 0) {
                        return;
                    }
                    dq$map_check([-1, this.current.yh]) && this.target.push([-1, this.current.yh]);
                    dq$map_check([-1, 0]) && this.target.push([-1, 0]);
                    break;
                case DQ.DIR.RIGHT:
                    if (this.current.xh != 0) {
                        return;
                    }
                    dq$map_check([1, this.current.yh]) && this.target.push([1, this.current.yh]);
                    dq$map_check([1, 0]) && this.target.push([1, 0]);
                    break;
                case DQ.DIR.UP:
                    if (this.current.yh != 0) {
                        return;
                    }
                    dq$map_check([this.current.xh, -1]) && this.target.push([this.current.xh, -1]);
                    dq$map_check([0, -1]) && this.target.push([0, -1]);
                    break;
                case DQ.DIR.DOWN:
                    if (this.current.yh != 0) {
                        return;
                    }
                    dq$map_check([this.current.xh, 1]) && this.target.push([this.current.xh, 1]);
                    dq$map_check([0, 1]) && this.target.push([0, 1]);
                    break;
            }
            function dq$map_check(dir) {
                return me.checked[dir] == null;
            }
        },
        reset: function () {
            this.current.xh = this.current.yh = 0;
            this.makeTarget(-1);
        },
        prePosition: { x: 0, y: 0, xh: 0, yh: 0 },
        current: { x: 0, y: 0, xh: 0, yh: 0 },
        dir: [],
        checked: [],
        target: []
    }
    DQ.Screen.Map = dqextend(DQ.Control, function (screen, options) {
        /// <summary>
        ///     チップマップを作成します。
        /// </sumamry>
        /// <param name="screen" type="DQ.Screen">
        ///    マップの表示先であるスクリーンを指定
        /// </param>
        /// <param name="url" type="string">
        ///     マップデータを読み込むためのURLを指定
        /// </param>
        /// <param name="options" type="object">
        ///     マップのオプションを指定
        /// </param>
        if (!arguments.length) {
            throw new Error("DQ.Screen.Map.constractor: 引数が少なすぎ");
        }

        var me = this;
        options = DQ.options(options, {
            chipSize: DQ.RESOLUTION,
            npcHeight: 48,
            deltaX: DQ.MAP_DELTA,
            deltaY: DQ.MAP_DELTA,
            sprite: DQ.SPRITE32,
            sprite16: DQ.SPRITE16,
            showShadow: false
        });

        me._npcHeight = options.npcHeight;
        me.chipSize = options.chipSize;
        me.deltaX = options.delta;
        me.deltaY = options.delta;
        me.sprite = options.sprite;
        me.sprite16 = options.sprite16;

        me.top = -me.chipSize;
        me.left = -me.chipSize;
        me._l = -me.chipSize + me.chipSize;
        me._r = -me.chipSize - me.chipSize;
        me._u = -me.chipSize + me.chipSize;
        me._d = -me.chipSize - me.chipSize;

        me.scrolling = false;

        //map上の座標
        me.mapX = 50;
        me.mapY = 50;
        me.location = new DQ.Location(me.mapX, me.mapY);

        //マップの最大幅
        me.mapWidth = 100;
        me.mapHeight = 100;
        //
        me.data = null;
        me.canMove = [];
        me.canMoveT = [];
        //
        me._create(screen, options);

        cur = new Date();

        return me;
    }, {
        dqid: "DQ.Screen.Map",
        _dir: null,
        dragging: false,
        deltaX: DQ.MAP_DELTA,
        deltaY: DQ.MAP_DELTA,
        defaultRank: 0,
        dungeon: false,
        encount: false,
        harf: false,
        location: null,
        layer: [],
        screen: null,
        scrolling: false,
        sprite: null,
        _rx: 0,
        _ry: 0,
        speed: DQ.SCROLL_SPEED,
        showShadow: false,
        _npcHeight: 48,
        npcHeight: function (value) {
            if (!arguments.length) {
                return this._npcHeight;
            }
            this._npcHeight = value;
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].npcHeight && this.layer[i].npcHeight(this._npcHeight);
            }
        },
        _create: function (screen, options) {
            /// <summary>
            /// マップを作成します
            /// </sumamry>

            //マップ用スプライト
            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + "dom.png",
                tg = $('<div>')
                .addClass('dq-map-32px')
                .css({ background: 'gray', position: 'relative', top: this.top, left: this.left });

            this.base(screen, tg);

            this.obj0 = this.obj[0];

            //マップ表示領域の幅と高さ
            this.width(screen.view.width + this.chipSize * 2);
            this.height(screen.view.height + this.chipSize * 2);

            //空のスプライトを配置する
            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth),
                maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);

            this.data = [];
            this.canMove = [];
            this.canMoveT = [];

            if (this.canMove.length == 0) {
                for (var i = 0; i < 64; i++) {
                    this.canMove[i] = 1;
                }
            }
            if (this.canMoveT.length == 0) {
                for (var i = 0; i < 256; i++) {
                    this.canMoveT[i] = 1;
                }
            }
            this.layer1 = [];
            for (var y = 0; y < this.mapHeight; y++) {
                this.data.push([]);
                this.layer1.push([]);
                for (var x = 0; x < this.mapWidth; x++) {
                    this.data[y].push(0);
                    this.layer1[y].push(0);
                }
            }

            this.screen = screen;
            screen.append(tg);

            //マウスドラッグイベントを設定する
            this.chipSize = 16;
            this.normal.apply(this);

            //レイヤーを追加
            this.layer = [];
            this.layer.push(new DQ.Screen.HighLayer(screen, this));

            //マップデータを非同期で受信
            options.url && this.loadMap(options.url);
        },
        small: function () {
            /// <summary>
            ///     マップのチップサイズを16pxへ変更します
            /// </summary>
            if (this.chipSize == 16) {
                return;
            }
            this.chipSize = 16;
            var tg = this.obj.html("").removeClass('dq-map-32px').addClass('dq-map-16px');

            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + "dom.png";
            var maxX = this.maxX = Math.floor(this._width / this.chipSize);
            var maxY = this.maxY = Math.floor(this._height / this.chipSize);

            for (var y = 0; y < maxY; y++) {
                for (var x = 0; x < maxX; x++) {
                    $('<div>').addClass("dq-map-sp").appendTo(tg);
                }
            }

            this.view = $(".dq-map-sp", this.obj).css('background-image', 'url(' + sp + ')');
            this.top = -this.chipSize;
            this.left = -this.chipSize;
            this.obj.css({ 'top': this.top + 'px', 'left': this.left + 'px' });
            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.dt = [];
            this.dl = [];
            this.dp = [];
            for (var d = 0; d < 64; d++) {
                this.dt[d] = -(d % 8) * this.chipSize + 'px';
                this.dl[d] = -Math.floor(d / 8) * this.chipSize + 'px';
                this.dp[d] = this.dl[d] + " " + this.dt[d];
            }
            if (this.shadow && this.shadow.length) {
                for (var i = 0; i < maxY * maxX; i++) {
                    $(this.shadow[i]).remove();
                }
            }
            this.shadow = [];
            for (var i = 0; i < maxY * maxX; i++) {
                $('<div>')
                    .addClass('dq-mp-shadow')
                    .css({ top: Math.floor(i / maxX) * this.chipSize, left: Math.floor(i % maxX) * this.chipSize })
                    .css('background-image', 'url(' + pn + "shadow" + this.chipSize + ".png" + ')')
                    .appendTo(tg);
            }
            this.shadow = $('.dq-mp-shadow', this.obj);
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].small();
            }
        },
        normal: function () {
            /// <summary>
            ///     マップのチップサイズを32pxへ変更します
            /// </summary>
            if (this.chipSize == 32) {
                return;
            }
            this.chipSize = 32;
            var tg = this.obj.html("").removeClass('dq-map-16px').addClass('dq-map-32px');

            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + "dom.png";



            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);

            for (var y = 0; y < maxY; y++) {
                for (var x = 0; x < maxX; x++) {
                    $('<div>').addClass("dq-map-sp").appendTo(tg);
                }
            }
            this.view = $(".dq-map-sp", this.obj).css('background-image', sp);
            this.top = -this.chipSize;
            this.left = -this.chipSize;
            this.obj.css({ 'top': this.top + 'px', 'left': this.left + 'px' })
            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.dt = [];
            this.dl = [];
            this.dp = [];
            for (var d = 0; d < 64; d++) {
                this.dt[d] = -(d % 8) * this.chipSize + 'px';
                this.dl[d] = -Math.floor(d / 8) * this.chipSize + 'px';
                this.dp[d] = this.dl[d] + " " + this.dt[d];
            }
            if (this.shadow && this.shadow.length) {
                for (var i = 0; i < maxY * maxX; i++) {
                    $(this.shadow[i]).remove();
                }
            }
            this.shadow = [];
            for (var i = 0; i < maxY * maxX; i++) {
                $('<div>')
                    .addClass('dq-mp-shadow')
                    .css({ top: Math.floor(i / maxX) * this.chipSize, left: Math.floor(i % maxX) * this.chipSize })
                    .css('background-image', 'url(' + pn + "shadow" + this.chipSize + ".png" + ')')
                    .appendTo(tg);
            }
            this.shadow = $('.dq-mp-shadow', this.obj);
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].normal();
            }
        },
        loading: false,
        loadMap: function (url, to) {
            /// <summary>
            ///     マップデータをサーバーから取得します。
            /// </summary>
            /// <param name="url" type="string">
            ///  マップを読み込むためのURLを指定
            /// </param>
            /// <param name="to" type="array">
            ///  デフォルト以外の初期位置
            /// </param>

            var me = this;
            me.loading = true;
            var hi = this.layer[0];
            var pn = world.project ? world.project.name + "/" : "";
            $.ajax(
            {
                type: "GET",
                url: pn + url,
                cache: false,
                success: function (data) {
                    try {
                        eval('var s=' + data);
                    } catch (ex) {
                        alert(DQ.format("{0}\n{1} : {2}", url, ex.name, ex.message));
                        me.loading = false;
                        return;
                    }
                    me._map = s;
                    me.url = url;

                    me.canMove = s.canMove || [];
                    me.canMoveT = s.canMoveT || [];
                    me.topographyType = s.topographyType || [
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3];
                    me.name = s.name || "";
                    me.uid = s.uid || -1;
                    me.data = s.data;
                    me.numberOfLayer = s.numberOfLayer || 3;
                    me.mapWidth = s.mx || me.mapWidth;
                    me.mapHeight = s.my || me.mapHeight;
                    me.ix = s.ix;
                    me.iy = s.iy;
                    me.mapX = (s.ix != null) ? s.ix - DQ.OX : me.mapX;
                    me.mapY = (s.iy != null) ? s.iy - DQ.OY : me.mapY;
                    me.mapX = (to != null) ? to[0] : me.mapX;
                    me.mapY = (to != null) ? to[1] : me.mapY;
                    //me.location = new DQ.Location(me.mapX, me.mapY);

                    me.view = $(".dq-map-sp", me.obj);
                    me.encount = (s.encount != null) ? s.encount : true;
                    me.dungeon = (s.dungeon != null) ? s.dungeon : false;
                    me.defaultRank = (s.defaultRank != null) ? s.defaultRank : 0;

                    if (s.sprite) {
                        me.sprite = s.sprite;
                    } else {
                        me.sprite = DQ.SPRITE32;
                    }
                    $(".dq-map-sp", this.obj).css('background-image', 'url(' + pn + "images/" + me.sprite + 'dom.png)');

                    me.top = -me.chipSize;
                    me.left = -me.chipSize;
                    me._l = -me.chipSize + me.chipSize;
                    me._r = -me.chipSize - me.chipSize;
                    me._u = -me.chipSize + me.chipSize;
                    me._d = -me.chipSize - me.chipSize;
                    me.location.reset();

                    hi.importSetup(me);
                    hi.data = [];

                    me.onLoaded && me.onLoaded(me);
                    me.loading = false;
                    me._dmax = 0;

                    me.scrolling = false;

                    me.obj0.style.top = me.top + 'px';
                    me.obj0.style.left = me.left + 'px';

                    for (var i = 1; i < me.numberOfLayer; i++) {
                        var nm = "layer" + i;
                        if (!s[nm]) {
                            s[nm] = [];
                            var sl = s[nm];
                            for (var y = 0; y < me.mapHeight; y++) {
                                sl.push([]);
                                for (var x = 0; x < me.mapWidth; x++) {
                                    sl[y].push(0);
                                }
                            }
                        }
                        me[nm] = s[nm];
                    }

                    me.forceMap();

                    me.screen.ly && me.screen.ly.dispose();

                },
                error: function (XMLHttpRequest, textStatus, errorThrown) {
                    alert(DQ.format("{0}\n{1}: {2} {3}", pn + url, XMLHttpRequest.status, XMLHttpRequest.statusText, textStatus));
                    me.loading = false;
                }
            });
        },
        forceMap: function (view, maxX, maxY, dp) {
            view = this.view;
            maxY = this.maxY;
            maxX = this.maxX;
            dp = this.dp;
            if (this.mapX + maxX >= this.mapWidth) {
                this.mapX = this.mapWidth - maxX;
            }
            if (this.mapY + maxY >= this.mapHeight) {
                this.mapY = this.mapHeight - maxY;

            }
            if (this.mapX < 0) {
                this.mapX = 0;
            }
            if (this.mapY < 0) {
                this.mapY = 0;
            }

            var pname = world.project && world.project.name + "/" || "",
                s1 = '0px -' + this.chipSize + 'px',
                s2 = '0px -' + this.chipSize * 2 + 'px';
            for (var y = 0; y < maxY; y++) {
                var my = y * maxX,
                    dy = this.data[this.mapY + y],
                    dly = this.data[this.mapY + y - 1];
                for (var x = 0; x < maxX; x++) {
                    var d = dy[this.mapX + x],
                        dd = dy[this.mapX + x - 1],
                        ll = dly == null ? 0 : dly[this.mapX + x - 1];
                    if (d == null) {
                        d = 63;
                    }
                    view[my + x].style.backgroundPosition = dp[d];
                    if (!this.showShadow) {
                        continue;
                    }
                    if (this.canMove[d] && (dd == 13 || dd == 15)) {
                        if (this.canMove[ll]) {
                            this.shadow[my + x].style.backgroundPosition = s1;
                        } else {
                            this.shadow[my + x].style.backgroundPosition = s2;
                        }
                    } else {
                        this.shadow[my + x].style.backgroundPosition = '0px 0px';
                    }
                }
            }
        },
        _viewMap: function (ox, oy, view, data, maxX, maxY, shadow, dp) {
            data = this.data;
            view = this.view;
            maxY = this.maxY;
            maxX = this.maxX;
            shadow = this.shadow;
            dp = this.dp;
            var pname = world.project && world.project.name + "/" || "",
                s1 = '0px -' + this.chipSize + 'px',
                s2 = '0px -' + this.chipSize * 2 + 'px';
            for (var y = 0; y < maxY; y++) {
                var my = y * maxX,
                    cy = this.mapY + y,
                    cd = data[cy + oy],
                    dy = data[this.mapY + y],
                    dly = data[this.mapY + y - 1];
                for (var x = 0; x < maxX; x++) {
                    var cx = this.mapX + x;
                    var d = dy[this.mapX + x],
                        dd = dy[this.mapX + x - 1];
                    //コストを最小にするため変化が無ければ書き換えない
                    if (d != cd[cx + ox]) {
                        var v = view[my + x].style.backgroundPosition = dp[d];
                    }
                    if (!this.showShadow || !dly) {
                        continue;
                    }
                    var ll = dly[this.mapX + x - 1];
                    if (this.canMove[d] && (dd == 13 || dd == 15)) {
                        if (this.canMove[ll]) {
                            shadow[my + x].style.backgroundPosition = s1;
                        } else {
                            shadow[my + x].style.backgroundPosition = s2;
                        }
                    } else {
                        shadow[my + x].style.backgroundPosition = '0px 0px';
                    }
                }
            }
        },
        move: function (x, y) {
            this.mapX = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            this.mapY = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = this.mapX >= 0 ? this.mapX : 0;
            this.mapY = this.mapY >= 0 ? this.mapY : 0;
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].mapX = this.mapX;
                this.layer[i].mapY = this.mapY;
                this.layer[i].forceMap();
            }
        },
        skip: function (ox, oy) {
            /// <summary>
            /// 指定のオフセット分現在地を移動します。
            /// </summary>
            var x = this.mapX + ox;
            var y = this.mapY + oy;

            var px = this.mapX;
            var py = this.mapY;
            x = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            y = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = x >= 0 ? x : 0;
            this.mapY = y >= 0 ? y : 0;

            var dx = this.mapX - px;
            var dy = this.mapY - py;
            this._viewMap(-dx, -dy);

            this.onScrolled && this.onScrolled(this, null) == false && (this._dir = null);
        },
        setBG: function (x, y, pos) {
            /// <summary>
            ///     背景のマップを書き換える
            /// </summary>
            ///	<param name="x" type="number">
            ///		画面上のx座標
            ///	</param>
            ///	<param name="y" type="number">
            ///		画面上のy座標
            ///	</param>
            ///	<param name="pos" type="number">
            ///		スプライト番号(0～63)
            ///	</param>
            var d = this.data[y][x] = pos;
            var v = this.view[(y - this.mapY) * this.maxX + (x - this.mapX)];
            if (v == null) {
                return false;
            }
            v.style.top = -(d % 8) * this.chipSize + 'px';
            v.style.left = -Math.floor(d / 8) * this.chipSize + 'px';

            return true;
        },
        scrollUp: function () {
            this.top += this.deltaY;
            if (this.top >= this._u) {
                this.top = -this.chipSize;
                this.mapY--;
                this._viewMap(0, 1);
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.up();
                return false;
            } else if (this.top == this._u - this.chipSize / 2 || this.top == this._d + this.chipSize) {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.up();
                return false;
            } else {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollDown: function () {
            this.top -= this.deltaY;
            if (this.top < this._d) {
                this.top = -this.chipSize;
                this.mapY++;
                this._viewMap(0, -1);
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.down();
                return false;
            } else if (this.top == this._d + this.chipSize / 2 || this.top == this._d + this.chipSize) {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.down();
                return false;
            } else {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollLeft: function () {
            this.left += this.deltaX;
            if (this.left >= this._l) {
                this.left = -this.chipSize;
                this.mapX--;
                this._viewMap(1, 0);
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.left();
                return false;
            } else if (this.left == this._l - this.chipSize / 2 || this.left == this._r + this.chipSize) {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.left();
                return false;
            } else {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollRight: function () {
            this.left -= this.deltaX;
            if (this.left < this._r) {
                this.left = -this.chipSize;
                this.mapX++;
                this._viewMap(-1, 0);
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.right();
                return false;
            } else if (this.left == this._r + this.chipSize / 2 || this.left == this._r + this.chipSize) {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.right();
                return false;
            } else {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scroll: function (dir) {
            ///<summary>
            /// 指定の方向へスクロール(歩く)します。
            ///</summary>
            ///<param name="dir" type="string">
            /// 方向を指定(left,right,up,down)
            ///</param>

            switch (dir) {
                case DQ.DIR.UP:
                    if (this.mapY <= 0) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.DOWN:
                    if (this.mapY >= this.mapHeight - this.maxY - 1) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.LEFT:
                    if (this.mapX <= 0) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.RIGHT:
                    if (this.mapX >= this.mapWidth - this.maxX - 1) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                default:
                    this._dir = null;
                    return false;
            }

            //次回スクロール予約
            this._dir = dir;
            !this.scrolling && this.location.makeTarget(dir);

            return true;
        },
        stop: function () {
            this._dir = null;
        },
        _keep: null,
        _dmax: 0,
        animation: function () {
        },
        update: function _map$update() {
            if (!this.scrolling) {
                this.layer[0].walk();
            }
            end = new Date();
            $('#ms').text(end - cur);
            cur = new Date();

            var me = this;
            !this.loading && _scroll();
            var dir = null;

            function _scroll() {
                var delta = { '0': [0, -1], '1': [0, 1], '2': [-1, 0], '3': [1, 0] };
                if (!me._keep && me.onPrescroll && me.scroll(me._dir)) {
                    if (!me.scrolling && me.onPrescroll(me, dir, delta[me._dir])) {
                        me._dir = null;
                        return false;
                    }
                }

                dir = !me._keep ? me._dir : me._keep;
                if (dir == null) {
                    //次がなければ何もしない
                    return false;
                }
                var res;
                switch (dir) {
                    case DQ.DIR.UP:
                        res = me.scrollUp();
                        break;
                    case DQ.DIR.DOWN:
                        res = me.scrollDown();
                        break;
                    case DQ.DIR.LEFT:
                        res = me.scrollLeft();
                        break;
                    case DQ.DIR.RIGHT:
                        res = me.scrollRight();
                        break;
                }

                var hi = me.layer[0];
                hi && hi.scroll(dir);
                if (res) {
                    me.onScrolling && me.onScrolling(me, dir);
                    me._keep = dir;
                } else {
                    this._dir = me.onScrolled && me.onScrolled(me, dir) == false ? null : this._dir;
                    me._keep = null;
                }
                return res;
            }
        },
        onClick: null,
        onLoaded: null,
        onBorder: null,
        onPrescroll: null,
        onScrolling: null,
        onScrolled: null,
        _click: function (e) {
            var me = e.data;
            var x = Math.floor(me.screen.toClientX(e.clientX) / me.chipSize);
            var y = Math.floor(me.screen.toClientY(e.clientY) / me.chipSize);
            me.onClick && me.onClick.apply(me, [e, { left: x, top: y}]);
        },
        click: function (fn) {
            if (!fn) {
                this.obj.unbind("click", this._click);
            } else {
                this.onClick = fn;
                this.obj.bind("click", this, this._click);
            }
        },
        scrolled: function (fn) {
            this.onScrolled = fn;
        },
        prescroll: function (fn) {
            this.onPrescroll = fn;
        },
        keydown: function (fn) {
            this.parent._keydown = fn;
        },
        getNPC: function () {
            return this.layer[0].npc;
        }
    });

    DQ.Screen.HighLayer = dqextend(DQ.Control, function (screen, map) {

        var me = this;
        me.importSetup(map);
        //
        me._create(screen, map);
        //
        me.data = null;

        return me;
    }, {
        dqid: "DQ.Screen.HighLayer",
        screen: null,
        sprite: null,
        _rx: 0,
        _ry: 0,
        npc_top: 0,
        delta: 8,
        _cd: 0,
        deltaX: DQ.MAP_DELTA,
        deltaY: DQ.MAP_DELTA,
        speed: DQ.SCROLL_SPEED,
        _npcHeight: 48,
        _npcOffset: 16,
        npcHeight: function (value) {
            if (!arguments.length) {
                return this._npcHeight;
            }

            this._npcHeight = value;
            this._npcOffset = (value == 48 || value == 24) ? (value - this.chipSize) : 0;
        },
        importSetup: function (map) {
            var me = this;

            me.screen = map.screen;
            me.chipSize = map.chipSize;
            me._npcHeight = map._npcHeight;
            me.deltaY = map.deltaY;  //delta x
            me.deltaX = map.deltaX;
            me.top = map.top;
            me.left = map.left;
            me.scrolling = false;
            me.sprite = map.sprite;

            me.chipSize = map.chipSize;

            //map上の座標
            me.mapX = map.mapX;
            me.mapY = map.mapY;

            //マップの最大幅
            me.mapWidth = map.mapWidth;
            me.mapHeight = map.mapHeight;

            //マップ表示領域の幅と高さ
            me.width(me.screen.view.width + me.chipSize * 2);
            me.height(me.screen.view.height + me.chipSize * 2);

            //スプライトを配置する
            var maxX = me.maxX = Math.min(Math.floor(me._width / me.chipSize), me.mapWidth);
            var maxY = me.maxY = Math.min(Math.floor(me._height / me.chipSize), me.mapHeight);

            this.map = map;
        },
        height: function (pos) {
            if (!arguments.length) {
                return this._height;
            } else {
                this._size._height = this._height = pos;
            }
        },
        width: function (pos) {
            if (!arguments.length) {
                return this._width;
            } else {
                this._size.width = this._width = pos;
            }
        },
        _create: function (screen, url, options) {
            /// <summary>
            /// マップを作成します
            /// </sumamry>


            //マップ用スプライト
            var sp = this.sprite;

            var tg = $('<div>')
                .addClass('dq-highlayer')
                .css({ position: 'absolute', top: this.top, left: this.left });

            this.base(screen, tg);
            this.obj0 = this.obj[0];

            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            //マップ表示領域の幅と高さ
            this.width(screen.view.width + this.chipSize * 2);
            this.height(screen.view.height + this.chipSize * 2);

            //スプライトを配置する
            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);

            this.screen = screen;
            screen.append(tg);
        },
        small: function () {
            /// <summary>
            ///     マップのチップサイズを16pxへ変更します
            /// </summary>
            if (this.chipSize == 16) {
                return;
            }
            this.chipSize = 16;
            var tg = this.obj.html("").removeClass('dq-map-32px dq-highlayer').addClass('dq-map-16px dq-highlayer-16px');

            var sp = this.sprite;
            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);
            this.top = -this.chipSize;
            this.left = -this.chipSize;
            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.obj.css({ top: this.top, left: this.left })

            this.forceMap();
        },
        normal: function () {
            /// <summary>
            ///     マップのチップサイズを32pxへ変更します
            /// </summary>
            if (this.chipSize == 32) {
                return;
            }
            this.chipSize = 32;
            var tg = this.obj.html("").removeClass('dq-map-16px dq-highlayer-16px').addClass('dq-map-32px dq-highlayer');

            var sp = this.sprite;
            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);
            this.top = -this.chipSize;
            this.left = -this.chipSize;
            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.obj.css({ top: this.top, left: this.left })

            this.forceMap();
        },
        walk: function () {
            if (!this.npc) {
                return;
            }
            DQ.NPC.tick();

            for (var i = 0; i < this.npc.length; i++) {
                this.npc[i].wark();
                this.npc[i].stamp(false);
            }
        },
        forceMap: function () {
            if (!this.data) {
                return;
            }
            var view = this.view,
                maxY = this.maxY,
                maxX = this.maxX,
                flag = world.flag,
                pname = world.project && world.project.name + "/" || "";

            if (this.mapX + maxX >= this.mapWidth) {
                this.mapX = this.mapWidth - maxX;
            }
            if (this.mapY + maxY >= this.mapHeight) {
                this.mapY = this.mapHeight - maxY;
            }
            this.obj.text("");
            this.npc = [];
            var inzdex = 100, data = this.map.data, can = this.map.canMove;
            for (var y = 0; y < maxY; y++) {
                for (var x = 0; x < maxX; x++) {
                    var dx = this.mapX + x,
                        dy = this.mapY + y;

                    var da = this.data[[dx, dy]];
                    if (da == null) {
                        continue;
                    }
                    for (var j = 0; j < da.length; j++) {
                        var d = da[j],
                            valid = false,
                            img = null, _img = null, pos = 0, _src = "", _zindex = "", _cls, top = 0, left = 0, offset = 0;
                        d.conditions && d.conditions[0] && eval('valid =' + d.conditions[0]);
                        switch (d.type) {
                            case "box":
                                img = $('<div>').addClass("box")
                                .css('background-image', 'url(' + pname + "images/box" + this.chipSize + ".png" + ')');
                                !valid && (img[0].style.backgroundPosition = '0px -' + this.chipSize + 'px');
                                break;
                            case "door":
                            case "item":
                            case "wall":
                                _src = d.data && d.data.image || (world.debug ? "event" : "");
                                _zindex = d.data && d.data.zindex || "";
                                _cls = d.data && d.data.cls || null;
                                if (_src) {
                                    img = $('<div>')
                                        .addClass("door")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    if (valid) {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                    } else {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + (d.data.top + d.data.length) * this.chipSize + 'px';
                                    }
                                }
                                break;
                            case "load":
                                _src = d.data && d.data.image || (world.debug ? "event" : "");
                                d.data.left = d.data.left == null ? 0 : d.data.left;
                                d.data.top = d.data.top == null ? 0 : d.data.top;
                                _zindex = d.data && d.data.zindex || "";
                                _cls = d.data && d.data.cls || null;
                                if (_src) {
                                    img = $('<div>')
                                        .addClass("box")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                }
                                break;
                            case "roof":
                                _src = d.data && d.data.image || "roof";
                                _zindex = d.data && d.data.zindex || 199;
                                _cls = d.data && d.data.cls || null;
                                if (_src && valid) {
                                    img = $('<div>')
                                        .addClass("door")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    if (valid) {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                    } else {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + (d.data.top + d.data.length) * this.chipSize + 'px';
                                    }
                                    if (!valid) {
                                    }
                                }
                                break;
                            default:
                                if (world.debug) {
                                    img = $('<img>').addClass('npc').attr('src', pname + "images/event32.png");
                                }
                                break;
                        }

                        img && img.appendTo(this.obj).css({ top: y * this.chipSize - offset, left: x * this.chipSize });
                    }
                }
            }
            this.npc = [];
            for (var i = 0; i < world.events.npcs.length; i++) {
                var d = world.events.npcs[i];
                d.data = d.data || {};
                offset = this._npcOffset;
                var px = d.position[0], py = d.position[1],
                    x = px - this.mapX, y = py - this.mapY,
                    AI = d.data.AI || DQ.Event.AI.Stamp,
                    dir = d.data.dir || 0,
                    uid = d.data.uid || 10001,
                    top = y * this.chipSize - offset,
                    left = x * this.chipSize;
                if (x < 0 || this.maxX <= x || y < 0 || this.maxY <= y) {
                    //continue;
                }
                //当たり判定対象でないなら表示対象ではない。
                if (!d.isPreEvent()) {
                    continue;
                }
                img = $('<div>').attr('npc_top', top).addClass("npc" + this._npcHeight)
                .css({ 'z-index': 100 + y, 'background-image': 'url(' + pname + "images/npc_" + uid + "_" + this._npcHeight + ".png" + ')' });
                var _npc = new DQ.NPC({ AI: AI, dir: dir, image: img, pos: [x, y], event: d });
                this.npc.push(_npc);
                _npc.stamp(true);
                img && img.appendTo(this.obj).css({ top: top, left: left });
                img = null;
            }
        },
        _cd: 0,
        delta: 4,
        _viewMap: function (ox, oy) {
            var view = this.view,
                maxY = this.maxY,
                maxX = this.maxX,
                pname = world.project && world.project.name + "/" || ""; ;

            this.obj.text("");
            var flag = world.flag;
            var data = this.map.data, can = this.map.canMove;
            for (var y = 0; y < maxY; y++) {
                for (var x = 0; x < maxX; x++) {
                    var dx = this.mapX + x,
                        dy = this.mapY + y;

                    var da = this.data[[dx, dy]];
                    if (da == null) {
                        continue;
                    }
                    for (var j = 0; j < da.length; j++) {
                        var d = da[j],
                            valid = false,
                            img = null, _img = null, pos = 0, _src = "", _zindex = "", _cls, top = 0, left = 0, offset = 0;
                        d.conditions && d.conditions[0] && eval('valid =' + d.conditions[0]);
                        switch (d.type) {
                            case "box":
                                img = $('<div>').addClass("box")
                                .css('background-image', 'url(' + pname + "images/box" + this.chipSize + ".png" + ')');
                                !valid && (img[0].style.backgroundPosition = '0px -' + this.chipSize + 'px');
                                break;
                            case "door":
                            case "item":
                            case "wall":
                                _src = d.data && d.data.image || (world.debug ? "event" : "");
                                _zindex = d.data && d.data.zindex || "";
                                _cls = d.data && d.data.cls || null;
                                if (_src) {
                                    img = $('<div>')
                                        .addClass("door")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    if (valid) {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                    } else {
                                        img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + (d.data.top + d.data.length) * this.chipSize + 'px';
                                    }
                                }
                                break;
                            case "load":
                                _src = d.data && d.data.image || (world.debug ? "event" : "");
                                d.data.left = d.data.left == null ? 0 : d.data.left;
                                d.data.top = d.data.top == null ? 0 : d.data.top;
                                _zindex = d.data && d.data.zindex || "";
                                _cls = d.data && d.data.cls || null;
                                if (_src) {
                                    img = $('<div>')
                                        .addClass("box")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                }
                                break;
                            case "roof":
                                _src = d.data && d.data.image || "roof";
                                _zindex = d.data && d.data.zindex || 199;
                                _cls = d.data && d.data.cls || null;
                                if (_src && valid) {
                                    img = $('<div>')
                                        .addClass("door")
                                        .css({ 'z-index': _zindex,
                                            'background-image': 'url(' + pname + "images/" + _src + this.chipSize + ".png)"
                                        });
                                    img[0].style.backgroundPosition = '-' + d.data.left * this.chipSize + 'px -' + d.data.top * this.chipSize + 'px';
                                    if (!valid) {
                                        img[0].style.backgroundPositionY = '-' + (d.data.top + d.data.length) * this.chipSize + 'px';
                                    }
                                }
                                break;
                            default:
                                if (world.debug) {
                                    img = $('<img>').attr('src', pname + "images/event" + this.chipSize + ".png");
                                }
                                break;
                        }

                        img && img.appendTo(this.obj).css({ top: y * this.chipSize - offset, left: x * this.chipSize });
                    }
                }
            }
            //this.npc = [];
            for (var i = 0; i < this.npc.length; i++) {
                var d = this.npc[i].event;

                //for (var i = 0; i < world.events.npcs.length; i++) {
                //    var d = world.events.npcs[i];
                //    d.data = d.data || {};

                offset = this._npcOffset;
                var px = d.position[0], py = d.position[1],
                    x = px - this.mapX, y = py - this.mapY,
                    AI = d.data.AI || DQ.Event.AI.Stamp,
                    dir = d.data.dir || 0,
                    uid = d.data.uid || 10001,
                    top = y * this.chipSize - offset,
                    left = x * this.chipSize;
                if (x < 0 || this.maxX <= x || y < 0 || this.maxY <= y) {
                    //continue;
                }
                //当たり判定対象でないなら表示対象ではない。
                if (!d.isPreEvent()) {
                    continue;
                }
                img = $('<div>').attr('npc_top', top).addClass("npc" + this._npcHeight)
                .css({ 'z-index': 100 + y, 'background-image': 'url(' + pname + "images/npc_" + uid + "_" + this._npcHeight + ".png" + ')' });

                //var _npc = new DQ.NPC({ AI: AI, dir: dir, image: img, pos: [x, y], event: d });
                //this.npc.push(_npc);
                var _npc = this.npc[i];
                _npc.position(x, y);
                _npc.image(img);
                _npc.stamp(true);
                img && img.appendTo(this.obj);
                img = null;
            }
        },
        move: function (x, y) {
            this.mapX = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            this.mapY = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = this.mapX >= 0 ? this.mapX : 0;
            this.mapY = this.mapY >= 0 ? this.mapY : 0;
            this.forceMap();
        },
        skip: function (ox, oy) {
            /// <summary>
            /// 指定のオフセット分現在地を移動します。
            /// </summary>
            var x = this.mapX + ox;
            var y = this.mapY + oy;

            var px = this.mapX;
            var py = this.mapY;
            x = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            y = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = x >= 0 ? x : 0;
            this.mapY = y >= 0 ? y : 0;

            var dx = this.mapX - px;
            var dy = this.mapY - py;
            this._viewMap(-dx, -dy);

            if (this.onScrolled) {
                var mp = this;
                this.onScrolled(mp, null);
            }
        },
        scrollUp: function () {
            this.top += this.deltaY;
            if (this.top >= this._u) {
                this.top = -this.chipSize;
                this.mapY--;
                this._viewMap(0, 1);
                this.obj.css('top', this.top);
                this.scrolling = false;
                return false;
            }
            else {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollDown: function () {
            this.top -= this.deltaY;
            if (this.top < this._d) {
                this.top = -this.chipSize;
                this.mapY++;
                this._viewMap(0, -1);
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                return false;
            }
            else {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollLeft: function () {
            this.left += this.deltaX;
            if (this.left >= this._l) {
                this.left = -this.chipSize;
                this.mapX--;
                this._viewMap(1, 0);
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                return false;
            }
            else {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollRight: function () {
            this.left -= this.deltaX;
            if (this.left < this._r) {
                this.left = -this.chipSize;
                this.mapX++;
                this._viewMap(-1, 0);
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                return false;
            }
            else {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scroll: function (dir) {
            if (this.scrolling == false && this.onPrescroll) {
                var mp = this;
                var d;
                switch (dir) {
                    case DQ.DIR.LEFT:
                        d = [0, -1];
                        break;
                    case DQ.DIR.RIGHT:
                        d = [0, 1];
                        break;
                    case DQ.DIR.UP:
                        d = [-1, 0];
                        break;
                    case DQ.DIR.DOWN:
                        d = [1, 0];
                        break;
                    default:
                }
                if (this.onPrescroll(mp, dir, d)) {
                    return false;
                }
            }
            var res;
            if (dir == DQ.DIR.UP) {
                if (this.mapY <= 0) {
                    return false;
                }
                res = this.scrollUp();
            } else if (dir == DQ.DIR.DOWN) {
                if (this.mapY >= this.mapHeight - this.maxY - 1) {
                    return false;
                }
                res = this.scrollDown();
            } else if (dir == DQ.DIR.LEFT) {
                if (this.mapX <= 0) {
                    return false;
                }
                res = this.scrollLeft();
            } else {
                if (this.mapX >= this.mapWidth - this.maxX - 1) {
                    return false;
                }
                res = this.scrollRight();
            }

            if (res == false && this.onScrolled) {
                var mp = this;
                this.onScrolled(mp, dir);
            }

            if (res) {
                this.onScrolling && this.onScrolling(mp, dir);

                var me = this;
                //setTimeout(function () { me.scroll(dir); }, me.speed);
            }
            return res;
        },
        onLoaded: null,
        onScrolling: null,
        onScrolled: null,
        onPrescroll: null,
        onClick: null,
        _onClick: function (e) {
            var me = e.data;
            var x = Math.floor(me.screen.toClientX(e.clientX) / me.chipSize);
            var y = Math.floor(me.screen.toClientY(e.clientY) / me.chipSize);
            this.onClick && this.onClick.apply(me, [e, { left: x, top: y}]);
        },
        click: function (fn) {
            if (!fn) {
                this.obj.unbind("click", this._onClick);
            } else {
                this.onClick = fn;
                this.obj.bind("click", this, this._onClick);
            }
        },
        scrolled: function (fn) {
            this.onScrolled = fn;
        },
        prescroll: function (fn) {
            this.onPrescroll = fn;
        }
    });
});
