﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="../ui/dq-ui.js" />
/// <reference path="../dq-random.js" />
/// <reference path="dq-rpg.js" />
/// <reference path="dq-rpg-groupbox.js" />

/*
** DQ Retro game UI framework JavaScript library version v0.6.0
**
** Copyright (c) 2009-2010 M., Koji
** Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
**
** Date: 2010-04-01 17:34:21 -0900
** Revision: 0006
**
**/
DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg.js", "DQ.RPG.InputBox", function () {

    DQ.RPG.API = {};
    DQ.World.prototype.encount = 0;

    DQ.World.prototype.showStateBox = function (options) {
        /// <summary>
        /// stateBoxを表示します。
        /// </summary>
        options = DQ.options(options, {
            left: '1em',
            top: '10em',
            canSelect: true,
            callback: null
        });
        var callback = options.callback;
        options.click = function (sender, pos) {
            callback && callback(sbox, pos);
            setTimeout(function () { sbox.hide(); sbox.dispose(); }, 0);
        }

        var sbox = new DQ.StateBox(this.screen, options);
        sbox.show();
    }

    DQ.World.prototype.itemMenu = function (sender) {
        if (world.party[0].items.length == 0) {
            world.showMessage([[world.party[0].name, "は、どうぐを持っていない。"].join('')], "");
            return;
        }
        world._itemBox_ = world._itemBox_ || new DQ.RPG.ItemBox(world.screen, { parentMenu: sender });

        //選択
        var me = world;
        world._itemBox_.onUse = function dq$world$itemBox_onUse(player, to, item) {
            /// <summary>
            /// 使用する
            /// </summary> 
            return me.useItem(world._itemBox_, player, to, item);
        }
        world._itemBox_.onTrade = function dq$world$itemBox_onTrade(player, to, item) {
            /// <summary>
            /// 使用する
            /// </summary> 
            return me.trade(player, to, item);
        }
        world._itemBox_.onVisibleChanged = function (eqpbox, visible) {
            if (!visible && world._itemBox_) {
                setTimeout(function () {
                    world._itemBox_ && world._itemBox_.dispose();
                    world._itemBox_ = null;
                }, 0);
            }
        }
        world._itemBox_.player = world.party[0];
        world._itemBox_.show();
    }

    DQ.World.prototype.statusMenu = function (sender) {
        /// <summary>
        /// ステータスを表示します。
        /// </summary>

        world._stateBox_ = world._stateBox_ || new DQ.RPG.StatusBox(world.screen, { parentMenu: sender });

        //選択
        var me = world;
        world._stateBox_.onVisibleChanged = function (stsbox, visible) {
            if (!visible) {
                setTimeout(function () {
                    world._stateBox_ && world._stateBox_.dispose();
                    world._stateBox_ = null;
                }, 0);
            }
        }
        world._stateBox_.player = world.party[0];
        world._stateBox_.show();
    }

    DQ.World.prototype.configMenu = function (sender) {
        /// <summary>
        /// ステータスを表示します。
        /// </summary>

        world._configBox_ = world._configBox_ || new DQ.RPG.ConfigBox(world.screen, { parentMenu: sender });

        //選択
        var me = world;
        world._configBox_.onSave = world.onSave;
        world._configBox_.onLoad = world.onLoad;

        world._configBox_.onVisibleChanged = function (stsbox, visible) {
            if (!visible) {
                setTimeout(function () {
                    world._configBox_ && world._configBox_.dispose();
                    world._configBox_ = null;
                }, 0);
            }
        }
        world._configBox_.show();
    }

    DQ.World.prototype.searchMenu = function (sender) {
        ///<summary>
        ///調べる
        ///</summary>
        sender && sender.parent.hide();

        var xx = world.map.mapX + DQ.OX,
                yy = world.map.mapY + DQ.OY;

        world.events.searchAt([xx, yy], null);
    }
    DQ.World.prototype.equipmentMenu = function (sender) {
        ///<summary>
        /// 装備選択メニューを表示
        ///</summary>

        world._eqpBox_ = world._eqpBox_ || new DQ.RPG.EquipmentBox(world.screen, { parentMenu: sender });

        //選択
        var me = world;
        world._eqpBox_.onEquip = function dq$world$eqpBox_onEquip(player, item) {
            /// <summary>
            /// 使用する
            /// </summary> 
            me.equipWeapon(player, item);
        }
        world._eqpBox_.onVisibleChanged = function (eqpbox, visible) {
            if (!visible) {
                setTimeout(function () {
                    world._eqpBox_ && world._eqpBox_.dispose();
                    world._eqpBox_ = null;
                }, 0);
            }
        }
        world._eqpBox_.player = world.party[0];
        world._eqpBox_.show();
    }

    DQ.World.prototype.magicMenu = function (sender) {
        if (world.party[0].magicIDs.length == 0) {
            world.showMessage([DQ.format("{0} は、呪文を覚えていない", world.party[0].name)], "");
            return;
        }

        world._magicBox_ = world._magicBox_ || new DQ.RPG.MagicBox(world.screen, { parentMenu: sender });

        //選択
        var me = world;
        world._magicBox_.onChant = function dq$world$itemBox_onChant(sender, from, to, magicId, pos) {
            /// <summary>
            /// 使用する
            /// </summary>
            var magic = me.magics.catalog[magicId],
                msg = [],
                result,
                show_message = function () {
                    msg.length && me.showMessage(msg, function () { /* top && top.hide(); */ });
                    return true;
                },
                _chant_effect = function () {
                    switch (magic.effect) {
                        case 10001:
                            world.screen.flashEx("Blue", 1, 15.625 * 2, 3, show_message);
                            break;
                        case 10002:
                            world.screen.flashEx("Red", 1, 15.625 * 2, 2, show_message);
                            break;
                        case 10003:
                            world.screen.flashEx("White", 1, 15.625 * 2, 3, show_message);
                            break;
                        case 10004:
                            //ジャンプなど(今は用意していない)
                        default:
                            show_message();
                    }
                };

            if (magic.chant(from, to, msg) && magic.effect) {
                var jingle = world.sounds.find('name', 'magic02');
                if (jingle) {
                    world.playJingle2(jingle.src, false, function () {
                    });
                    _chant_effect();
                } else {
                    _chant_effect();
                }
            } else {
                show_message();
            }
        }
        world._magicBox_.onVisibleChanged = function (eqpbox, visible) {
            if (!visible) {
                setTimeout(function () {
                    world._magicBox_.dispose();
                    world._magicBox_ = null;
                }, 0);
            }
        }
        world._magicBox_.player = world.party[0];
        world._magicBox_.show();
    }

    DQ.World.prototype.useItem = function world$useItem(top, player, to, itemInfo) {
        /// <summary>
        /// アイテムを使用する
        /// </summary>
        /// <param name="top" type="DQ.ItemBox">
        /// アイテム選択ボックスを指定
        /// </param>

        var item = this.items.catalog[itemInfo.uid],
            msg = [],
            me = this,
            result,
            map_selected = function (sender, pos) {
                item.use(player, sender.data[pos].map, msg);
                top && top.hide();
                world._magicBox_ && world._magicBox_.hide();
                world._itemBox_ && world._itemBox_.hide();
            }


        item.serial = itemInfo.serial;

        switch (item.type) {
            case 'field': //場に適用
                result = item.use(world, msg);
                break;
            case 'jump':
                var towns = world.getVisitedTown();
                if (towns.length == 0) {
                    msg.push("訪れた町はまだ無い。");
                } else {
                    this.selectTo(towns, map_selected);
                }
                result = false;
                break;
            case 'party':
                result = item.use(player, world.party, msg);
                break;
            default:
                result = item.use(player, to, msg);
                break;
        }
        msg.length && me.showMessage(msg, function () { top && top.hide(); });

        return result;
    }

    DQ.World.prototype.selectTo = function (data, callback) {
        this.showStateBox({
            left: '9em',
            top: '3em',
            width: '10em',
            data: data,
            callback: callback
        });
    }

    DQ.World.prototype.useItemForParty = function (item, party, to, text) {
        /// <summary>
        /// パーティー用のアイテムを使用する
        /// </summary>

        /// UIDを指定するような処理はAPIへ持ってくる方針のためにここへ委譲
        var result;
        switch (item.uid) {
            case 10005:
                if (world.party.flag[DQ.RPG.Party.FlagPos.Holy]) {
                    //text.push(DQ.format("すでに使用している。"));
                    //return false;
                }
                world.party.flag[DQ.RPG.Party.FlagPos.Holy] = true;
                world.party.temporary.push(new DQ.RPG.State(world.statuses.find('uid', 10006)));
                result = true;
                break;
            case 10010:
                if (world.party.flag[DQ.RPG.Party.FlagPos.Torch]) {
                    text.push(DQ.format("すでに使用している。"));
                    return false;
                }
                result = world.lightUp();
                if (result) {
                    text.push(DQ.format("周囲があかるくなった。"));
                } else {
                    text.push(DQ.format("ここでは、使用できない。"));
                }
                break;
        }
        return result;
    }

    DQ.World.prototype.equipWeapon = function (player, itemInfo) {
        /// <summary>
        /// 装備する
        /// </summary>
        if (typeof player == "number") {
            player = world.party[player];
        }
        if (!itemInfo.serial) {
            var guid = new DQ.Guid();
            itemInfo.serial = guid.generate();
        }
        player.equip(itemInfo.type, itemInfo.uid, itemInfo.serial);
    }

    DQ.World.prototype.jump = function (name, x, y) {
        if (this.loading || this.map.loading) {
            debug.writeln("jump: loading 中に 呼び出された(やりすぎ)");
            return;
        }
        //同一マップ判定
        if (this.map.name == name) {
            if (arguments.length == 1) {
                x = this.map.ix;
                y = this.map.iy;
            }
            this.map.move(x - DQ.OX, y - DQ.OY);
            return;
        }

        //別マップ
        var me = this, url;
        var mapInfo = this.maps.find("name", name);
        if (!mapInfo) {
            throw new Error(DQ.format("error: 指定のマップが見つかりません:{0}", name));
        } else {
            url = mapInfo.url;
        }
        this.loading = true;
        this.screen.fadeOut(500, function () {
            if (x == null) {
                mapInfo.load();
            } else {
                mapInfo.load([x, y]);
            }
            me.map.key = name;
            me.events.setup(name);
            me.events.playSound();
            me.loading = false;

            return false; //dispose抑止
        });
    }

    DQ.World.prototype.dropItem = function (sender, player, itemId, count) {
        var item = this.items.catalog[itemId],
            msg = [],
        yesNoBox_onClick = function (s, res) {
            yesNoBox.hide();
            if (res == DQ.YesNo.Yes) { //yes
                this.drop(player, { id: itemId, count: count }, msg);

                this.screen.showMessage(msg, function () { sender.hide(); yesNoBox && yesNoBox.dispose(); });
                sender.update();
            } else {
                sender.hide();
            }
        },
        showMessage = function (msg) {
            this.screen.showMessage([msg], function () { sender.hide(); yesNoBox && yesNoBox.dispose(); });
        }
        //親子紐付け
        var yesNoBox = new DQ.YesNo(this.screen, '2em', '10em');
        yesNoBox.text("捨てる？");
        yesNoBox.show(true);
        yesNoBox.onClick = yesNoBox_onClick;

        sender.selected().setSubItem(yesNoBox);
        sender.enabled(false);
    }

    //プレヤーの振る舞い

    DQ.World.prototype.trade = function (from, to, itemInfo) {
        if (typeof from == "number") {
            from = world.party[from];
        }
        if (typeof to == "number") {
            to = world.party[to];
        }
        from.items.remove(itemInfo.serial);
        to.incrementItem(itemInfo.uid, itemInfo.count, itemInfo.serial);
    }

    DQ.World.prototype.chant = function (from, to, magicId, msg) {
        if (typeof from == "number") {
            from = world.party[from];
        }
        if (typeof to == "number") {
            to = world.party[to];
        }
        var magic = this.magics.catalog[magicId];
        magic.chant(from, to, msg);
    }

    DQ.World.drop = function (player, itemInfo, msg) {
        if (typeof player == "number") {
            player = world.party[player];
        }
        var item = this.items.catalog.find(itemInfo.uid);
        player.drop(itemInfo.uid, itemInfo.count);
        msg.push(DQ.format("{0} は、 {1} を捨てました。", player.name, item.text));
    }

    DQ.World.prototype.buy = function (from, to, item, msg) {
        var sellItem = from.items[item.id];
        to.G -= sellItem.G * item.count;
        to.incrementItem(item.id, item.count, null);
    }

    DQ.World.prototype.sell = function (from, to, item, msg) {
        var buyItem = from.items[item.id];
        to.G += buyItem.G;
        to.decrementItem(item.id, item.count);
        msg.push(DQ.format("{0} は、 {1} を売却しました。", player.name, buyItem.text));
    }

    DQ.World.prototype.getBattleAGI = function (cre) {
        return Math.floor(cre.getState('AGI') / 2) + DQ.Random.getValue();
    }

    DQ.World.Topography = {
        Alpine: 0,          //山岳
        Sea: 1,             //海
        PoisonedSwamp: 2,   //毒の沼
        Field: 3,           //平地
        Ice: 4,             //氷原
        Forest: 5,          //森
        Mountain: 6,        //山
        Desert: 7,          //砂漠
        Town: 8,            //町
        Cave: 9,            //洞窟
        Bridge: 10,         //橋
        Other: 11           //その他
    };

    DQ.World.prototype.getTopographyType = function (chip) {
        var seed;
        //チップ番号から地形種別を取得
        var t;
        if (world.map && world.map.topographyType) {
            t = world.map.topographyType[chip];
        } else {
            t = 3;
        }

        return t;
    }

    DQ.World.prototype.isEncount = function (x, y) {
        ///<summary>
        /// ほぼDQ1ルールのエンカウント判定
        ///</summary>

        //歩数0か255ならエンカウントしない
        world.steps++;
        world.steps %= 256;
        if (world.steps <= 1) {
            return false;
        }

        //チップ番号から地形種別を取得
        var t = this.getTopographyType(world.map.data[y][x]);

        //地形種別から基礎値を取得
        var seed;
        if (world.map.dungeon) {
            //ダンジョンなら6
            seed = 6;
        } else {
            seed = DQ.Random.EncountSeed[t];
        }

        //歩数と閾値
        var e = world.steps;
        var th = DQ.Random.EncountThreshold[seed];

        {
            //短い歩数の場合の補正
            if (e < 5) {
                seed = Math.floor(seed / 8);
            } else if (e < 9) {
                seed = Math.floor(seed / 4);
            } else if (e < 12) {
                seed = Math.floor(seed / 2);
            }
        }

        {
            //地形タイプによる補正
            if (e < th) {
                seed = Math.floor(seed * .75);
            } else if (e < th * 2) {
                seed = Math.floor(seed * 1.5);
            } else if (e < th * 3) {
                seed = Math.floor(seed * 2);
            } else {
                seed = Math.floor(seed * 4);
            }
        }

        $('#se').text(seed);
        //乱数 < seed なら エンカウント
        return DQ.Random.judge({ l: 0, g: seed });
    }

    DQ.World.prototype.nextEncount = function () {
        /// <summary>
        ///     次のエンカウント（歩数）を計算する
        /// </summary>
        /// <return type="number">

        var num = Math.floor(Math.random() * 1024.0);
        var bit = 1;
        var res = 1;
        for (var i = 0; i < 10; i++, bit = bit << 1) {
            res += (num & bit) ? 1 : 0;
        }

        return res * 3;
    }

    DQ.World.prototype.chooseMonster = function (rx, ry) {
        /// <summary>
        ///     モンスターを選択する
        /// </summary>
        var ev = this.events.encounts;

        //ランクの決定
        var rank = world.map.defaultRank == undefined ? 0 : world.map.defaultRank;
        for (var i = 0; i < ev.length; i++) {
            var e = ev[i].data;
            if (e.area[2] <= rx && rx <= e.area[3] && e.area[0] <= ry && ry <= e.area[1]) {
                rank = e.monsters;
                break;
            }
        }

        //グループの決定
        var group = DQ.Random.getValue(DQ.Random.Range.MonsterGroup5);

        return world.monsterGroups.catalog.data[rank].group[group];
    }

    DQ.World.prototype.canEncount = function (mons) {
        ///<summary>
        ///聖水の効果を確認する。
        ///</summary>

        if (world.party.flag[DQ.RPG.Party.FlagPos.Holy]) {
            //聖水発動中
            //DQ1方式
            var playerMax = 0, i;
            for (i = 0; i < world.party.length; i++) {
                if (world.party[i].getAttack() > playerMax) {
                    playerMax = world.party[i].getAttack();
                }
            }

            var monsterMax = 0;
            for (i = 0; i < mons.length; i++) {
                var mons = world.monsters.find(mons[i].name);
                if (mons.getAttack() > monsterMax) {
                    monsterMax = mons.getAttack();
                }
            }

            return playerMax * 20 < monsterMax;
        } else {
            return true;
        }

    }

    DQ.World.prototype.openBox = function (player, box, msg) {
        if (typeof player == "number") {
            player = world.party[player];
        }
        msg.push(DQ.format("{0} は、宝箱を開けた。", player.name));

        if (box.item) {
            var item = this.items.catalog[box.item];
            msg.push(DQ.format("なんと!\n {0} を、見つけた！", item.text));

            var res = player.incrementItem(item.uid, 1, null);
            if (res) {
                msg.push(DQ.format("{0} は、 {1} を手に入れた。", player.name, item.text));
            } else {
                msg.push(DQ.format("{0} は、荷物がいっぱいだ。これ以上はアイテムを持てない", player.name));
                return false;
            }
        }
        if (box.G) {
            world.party.G += box.G;
            msg.push(DQ.format("{0} は、 {1} Gを\n手に入れた。", player.name, box.G));
        }

        return true;
    };

    DQ.World.prototype.getItem = function (player, data, msg) {
        ///<summary>
        /// 生でアイテムを取得する
        ///</summary>
        if (typeof player == "number") {
            player = world.party[player];
        }
        var item = this.items.catalog[data.uid];
        var res = player.incrementItem(item.uid, 1, null);
        if (res) {
            msg.push(DQ.format("{0} は、 {1} を手に入れた。", player.name, item.text));
        } else {
            msg.push(DQ.format("これ以上は荷物を持てない"));
            return false;
        }

        return true;
    };

    DQ.World.prototype.searchAt = function (player, data, msg) {
        if (typeof player == "number") {
            player = world.party[player];
        }
        msg.push(DQ.format("{0} は、自分の足元を調べた。", player.name));

        if (data.item) {
            var item = this.items.catalog[data.item];
            msg.push(DQ.format("なんと!\n {0} を、見つけた！", item.text));

            var res = player.incrementItem(item.uid, 1, null);
            if (res) {
                msg.push(DQ.format("{0} は、 {1} を手に入れた。", player.name, item.text));
            } else {
                msg.push(DQ.format("これ以上は荷物を持てない"));
                return false;
            }
        }
        if (data.G) {
            world.party.G += data.G;
            msg.push(DQ.format("{0} は、 {1} Gを\n手に入れた。", player.name, data.G));
        }

        return true;
    };

    DQ.World.prototype.getReward = function (enemy) {
        ///<summary>
        /// 戦闘後の報酬を決定します。
        ///</summary>
        ///<param name="enemy" type="oject">
        /// 戦闘時の敵グループを指定
        ///</param>

        var G = 0, EXP = 0,
            items = [];
        for (var i = 0; i < enemy.length; i++) {
            if (enemy[i].isDead()) {
                // アイテムを落としたか判定
                if (DQ.Random.judge(DQ.Random.Range.DropItem[enemy[i].drop])) {
                    //落とした。
                    debug.writeln(DQ.format("{0} item drop.", enemy[i].name));
                    var group = DQ.Random.getValue(DQ.Random.Range.DropItemGroup);
                    if (enemy[i].items[group] != -1) {
                        items.push({ uid: enemy[i].items[group], count: 1, serial: -1 });
                    }
                }
                G += enemy[i].G;
                EXP += enemy[i].EXP;
            }
        }
        return { G: G, EXP: EXP, Items: items };
    }

    DQ.World.prototype.shareReward = function (party, reward, msg) {
        /// <summary>
        /// 取得した報酬をパーティー内メンバーで分配します。
        /// </summary>

        //経験値と金は均等割
        if (reward.G) {
            var G = parseInt(reward.G);
            party.G += G;
            if (party.length == 1) {
                msg.push(DQ.format("{0} は、 {1} Gを見つけた。", party[0].name, G));
            } else {
                msg.push(DQ.format("{0} 達は、 {1} Gを見つけた。", party[0].name, G));
            }
        }

        for (var i = 0; i < party.length; i++) {
            var EXP = Math.floor(reward.EXP / party.length);
            EXP && msg.push(DQ.format("{0} は、 {1} の経験値を獲得した。", party[i].name, EXP));

            party[i].EXP += EXP;
        }

        //アイテムを分配
        for (var i = 0; i < reward.Items.length; i++) {
            var itemInfo = reward.Items[i];
            var item = this.items.catalog[itemInfo.uid];

            for (var j = 0; j < party.length; j++) {
                if (party[j].incrementItem(itemInfo.uid, itemInfo.count, itemInfo.serial)) {
                    msg.push(DQ.format("{0} は、 {1} を手に入れた。", party[j].name, item.text));
                    break;
                }
            }
            if (j == party.length) {
                msg.push(DQ.format("荷物が一杯なので、{0} を捨てた。", item.text));
            }
        }
    }

    DQ.World.prototype.levelUp = function (party, msg) {
        /// <summary>
        /// パーティーのメンバーを可能ならレベルアップさせます。
        /// </summary>

        for (var i = 0; i < party.length; i++) {
            var player = party[i];
            var uid = player.uid == undefined ? 10001 : player.uid;
            var cur = world.levels.find('uid', uid).levels[player.LV - 1];
            var next = world.levels.find('uid', uid).levels[player.LV];
            if (next == null) {
                //レベルの上限に達している
                continue;
            }
            if (next.EXP <= player.EXP) {
                player.LV++;
                msg.push("$jingle:levelup");
                msg.push(DQ.format("{0} は、レベルが上がった。", player.name));
                var HP = (next.HP - cur.HP);
                var MP = (next.MP - cur.MP);
                var STR = (next.STR - cur.STR);
                var VIT = (next.VIT - cur.VIT);
                var AGI = (next.AGI - cur.AGI);
                HP > 0 &&
                    msg.push(DQ.format("最大HPが {0} ポイント上がった", HP));
                MP > 0 &&
                    msg.push(DQ.format("最大MPが {0} ポイント上がった", MP));
                STR > 0 &&
                    msg.push(DQ.format("STRが {0} ポイント上がった", STR));
                VIT > 0 &&
                    msg.push(DQ.format("VITが {0} ポイント上がった", VIT));
                AGI > 0 &&
                    msg.push(DQ.format("AGIが {0} ポイント上がった", AGI));

                if (next.magics) {
                    for (var i = 0; i < next.magics.length; i++) {
                        for (var j = 0; j < player.magicIDs.length; j++) {
                            if (player.magicIDs[j] == next.magics[i]) {
                                break;
                            }
                        }
                        if (j < player.magicIDs.length) {
                            continue;
                        }
                        player.magicIDs.push(next.magics[i]);
                        msg.push(DQ.format(
                            "{0} は、 {1} の呪文を覚えた。",
                            player.name,
                            world.magics.catalog[next.magics[i]].text));
                    }
                }
                player.levelUp({ maxHP: HP, maxMP: MP, STR: STR, VIT: VIT, AGI: AGI });

            }
        }
    }

    DQ.World.prototype.lightUp = function () {
        /// <summary>
        /// ダンジョンを明るくします。
        /// </summary>
        /// <return>
        /// 明かりをつけられたらtrue, だめならfalse
        /// </return>

        if (world._dark) {
            world._dark.hide();
            world._dark.dispose();
            world._dark = null;
        }
        if (world.map.dungeon) {
            world._light = world.screen.appendLayer({ cls: "dq-window dq-torch", name: "torch" });
            world.party.flag[DQ.RPG.Party.FlagPos.Torch] = true;
            return true;
        }
        return false;
    }

    DQ.World.prototype.sunshine = function () {
        /// <summary>
        /// 陽光の下に戻る
        /// </summary>
        if (world._dark) {
            world._dark.hide();
            world._dark.dispose();
            world._dark = null;
        }
        if (world._light) {
            world._light.hide();
            world._light.dispose();
            world._light = null;
        }
        world.party.flag[DQ.RPG.Party.FlagPos.Torch] = false;
    }
    DQ.World.prototype.lightDown = function () {
        ///<summary>
        ///たいまつの効果を切ります。
        ///</summary>

        if (world._light) {
            world._light.hide();
            world._light.dispose();
            world._light = null;
        }
        if (world.map.dungeon) {
            world._dark = world.screen.appendLayer({ cls: "dq-window dq-dark", name: "dark" });
        }
    }

    DQ.World.prototype.getVisitedTown = function () {
        var flag = this.flag;
        var list = [];
        for (var nm in world.maps.catalog) {
            if (!world.maps.catalog.hasOwnProperty(nm)) {
                continue;
            }
            var map = world.maps.catalog[nm];
            var pos = map.flag;
            if (pos == null || pos == -1) {
                continue;
            }
            if (pos >= DQ.MAX_FLAG) {
                throw new Error("error: DQ.World.getVistTown: maps flag pos is too big");
            }
            if (flag[pos]) {
                list.push({ name: map.name, text: map.description, value: map.position, map: map });
            }
        }

        return list;
    }

    //--------------------------
    //戦闘時のアルゴリズムを提供
    //--------------------------
    DQ.World.prototype.genRandom = function () {
        var value = 32;
        for (var i = 0; i < 16; i++) {
            value += Math.floor(DQ.Random.getValue() / 13);
        }
        return value;
    }
    DQ.World.prototype.getPower = function (base) {
        // Math.floor((dmg + (dmg + 1) * DQ.Random.getValue() / 256) / 2);

        return Math.floor(base * this.genRandom() / 256);
    }

    DQ.World.prototype.cancelCommand = function (monster, cmd, party, enemy) {
        if (cmd.command == "escape") {
            return Math.floor(world.party[0].getState('STR') / 2) < monster.getAttack();
        }
        if (cmd.command == "magic") {
            var result = true;
            switch (cmd.by.status) {
                case 10004: //sleep
                    for (var i = 0; i < party.length; i++) {
                        result &= party[i].getState('sleep');
                    }
                    break;
                case 10005: //silent
                    for (var i = 0; i < party.length; i++) {
                        result &= party[i].getState('silent');
                    }
                    break;
                default:
                    result = false;
            }

            return result;
        }
    }
    DQ.World.prototype.monsterAI = function (monster, party, enemy) {
        ///<summary>
        /// 8パターンの行動を選択(DQ2)
        ///</summary>
        for (var i = 0; i < 100; i++) {
            var rule = DQ.Random.getValue(DQ.Random.Range.Rule[monster.AI]);
            var cmd = world.genCommand(monster.action[rule]);
            if (cmd.by) {
                debug.writeln(DQ.format("{0}: rule:{1} by:{2} uid:{3}", monster.name, rule, cmd.by.name, cmd.by.uid));
            } else {
                debug.writeln(DQ.format("{0}: rule:{1} command:{2}", monster.name, rule, cmd.command));
            }
            if (monster.AI == 0) {
                return cmd;
            }
            if (!world.cancelCommand(monster, cmd, party, enemy)) {
                return cmd;
            }
        }

        return world.genCommand(0);
    }

    DQ.World.prototype.isCriticalHit = function (from, to) {
        //DQ1方式
        return from.friend && DQ.Random.getValue([{ l: 0, g: 8 }, { l: 8, g: 256}]) == 0;
    }

    DQ.World.prototype.isHit = function (from, to) {
        //DQ1方式
        if (from.friend) {
            if (to.getState('sleep')) {
                //睡眠中は確実に当たる
                return true;
            }
            var avoid = to.resistance[3] * 4;
            return DQ.Random.getValue([{ l: 0, g: avoid }, { l: avoid, g: 256}]) == 1;
        } else {
            return DQ.Random.getValue([{ l: 0, g: 4 }, { l: 4, g: 256}]) == 1;
        }
    }

    DQ.World.prototype.isAvoid = function (from, to) {
        //DQ1方式
        //hitとavoidに区別がない。

        return false;
    }

    DQ.World.prototype.escape = function (who, from) {
        if (from.isDead() || !from.canAction()) {
            //死亡または行動不能(DQ1では眠っている)なら無条件
            return true;
        }
        if (who.friend) {
            //DQ1方式
            var rate = [
            64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
            64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
            96, 96, 96, 96, 96, 96, 96, 96, 96, 96,
            128, 128, 128, 128, 128, 128, 128, 128,
            256, 256, 256, 256, 256, 256, 256, 256];

            var rdm = DQ.Random.getValue(),
            agi = who.getState('AGI') * rdm,
            idx = world.monsters.findIndex(from.title),
            def = rate[idx] * from.getDefence() * 2;

            debug.writeln(DQ.format("rdm: {4}, idx:{0}, rate[idx]:{1}, agi:{2} - def:{3}", idx, rate[idx], agi, def, rdm));
            return agi - def >= 0;
        } else {
            //敵は2/3
            return DQ.Random.getValue([{ l: 0, g: 171 }, { l: 171, g: 256}]) == 0;
        }
    }

    DQ.World.prototype.getCriticalDamage = function (from, to) {
        return Math.floor(from.getAttack() - (from.getAttack() / 2) * DQ.Random.getValue() / 256);
    }

    DQ.World.prototype.getDamage = function (from, to) {
        //DQ1方式
        if (from.friend) {
            var dmg = Math.floor((from.getAttack() - to.getDefence() / 2));
            debug.writeln(DQ.format("player:base damage: {0} - {1} / 2 = {2}", from.getAttack(), to.getDefence(), dmg));
            if (dmg < 2) {
                dmg = DQ.Random.getValue([{ l: 0, g: 128 }, { l: 128, g: 256}]);
                debug.writeln(DQ.format("lower rule:damage = {0}", dmg));
            } else {
                dmg = world.getPower(dmg);
                debug.writeln(DQ.format("normal rule:damage = {0}", dmg));
            }

            return dmg;
        } else {
            //敵
            var attack = from.getAttack();
            var dmg = Math.floor((attack * 2 - to.getDefence()) / 2);
            debug.writeln(DQ.format("1:base damage: {0}", dmg));

            if (dmg < 2) {
                dmg = 1 + DQ.Random.getValue([{ l: 0, g: 128 }, { l: 128, g: 256}]);
                debug.writeln(DQ.format("lower rule:damage = {0}", dmg));
            } else {
                dmg = world.getPower(dmg);
                debug.writeln(DQ.format("normal rule:damage = {0}", dmg));
            }

            return dmg;
        }
    }

    DQ.World.prototype.genCommand = function (no) {
        var cmd;
        switch (no) {
            case 0:
                cmd = new DQ.RPG.Command.Attack();
                cmd.by = new DQ.RPG.Weapon();
                break;
            case 4:
                cmd = new DQ.RPG.Command.Defence();
                break;
            case 5:
                cmd = new DQ.RPG.Command.Escape();
                break;
            case 6:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20001];
                break;
            case 7:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20002];
                break;
            case 8:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20003];
                break;
            case 9:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20004];
                break;
            case 10:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20005];
                break;
            case 11:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20006];
                break;
            case 12:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20007];
                break;
            case 13:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20008];
                break;
            case 14:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20009];
                break;
            case 15:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20010];
                break;
            case 16:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20011];
                break;
            case 17:
                cmd = new DQ.RPG.Command.Magic();
                cmd.by = world.magics.catalog[20012];
                break;
            case 18:
                cmd = new DQ.RPG.Command.Escape();
                cmd.by = world.magics.catalog[20013];
                break;
            case 19:
                cmd = new DQ.RPG.Command.Escape();
                cmd.by = world.magics.catalog[20014];
                break;
            default:
                cmd = new DQ.RPG.Command.Attack();
                cmd.by = new DQ.RPG.Weapon();
                break;
        }

        return cmd;
    }

    DQ.World.prototype.preemptiveAttack = function (player, enemy) {
        ///<summary>
        /// 先制攻撃を判定する。0未満なら先制攻撃される
        ///</summary>
        if (typeof player == "number") {
            player = world.party[player];
        }

        return
        Math.floor(
                player.getState('AGI') * DQ.Random.getValue() -
                DQ.Random.getValue() / 4 * enemy.getDefence() * 2) > 0;
    }

    DQ.World.prototype.isExpireStatus = function (from, status) {
        ///<summary>
        ///状態異常の解除判定を実施する
        ///</summary>
        ///<return>
        /// 解除ならtrueを返す。
        ///</return>

        switch (status.uid) {
            case 10004:
                if (from.friend) {
                    return DQ.Random.getValue([{ l: 0, g: 128 }, { l: 128, g: 256}]) == 0;
                } else {
                    return DQ.Random.getValue([{ l: 0, g: 86 }, { l: 86, g: 172 }, { l: 172, g: 256}]) == 0;
                }
            case 10005:
                return world.battle.battle ? false : true; //DQ1使用だとずっと継続だって
            case 10006:
                status.count--;
                return status.count <= 0;
            default:
                return false;
        }
    }

    DQ.World.prototype.hitMagic = function (magic, from, to) {
        ///<summary>
        ///魔法(呪文)の当たり判定を算出します。
        ///</summary>

        switch (magic.uid) {
            case 10002:
            case 10010:
                var avoid = to.resistance[2] * 16;
                debug.writeln(DQ.format("player: fire.isHist: avoid = {0}", avoid));
                return DQ.Random.getValue([{ l: 0, g: avoid }, { l: avoid, g: 256}]) == 1;
            case 10003:
                var avoid = to.resistance[0] * 16;
                debug.writeln(DQ.format("player: sleep.isHist: avoid = {0}", avoid));
                return DQ.Random.getValue([{ l: 0, g: avoid }, { l: avoid, g: 256}]) == 1;
            case 20003:
                return true;
                break;
            case 10005:
                var avoid = to.resistance[1] * 16;
                return DQ.Random.getValue([{ l: 0, g: avoid }, { l: avoid, g: 256}]) == 1;
            case 20002:
                return DQ.Random.getValue([{ l: 0, g: 4 }, { l: 4, g: 256}]) == 1;
            default:
                return true;
        }
    }

    DQ.World.prototype.getMagicDamage = function (magic, from, to) {
        ///<summary>
        ///魔法(呪文)の当たり判定を算出します。
        ///</summary>

        if (from.friend) {
            return world.getPower(magic.HP);
        } else {
            //あとでロトの効果を反映する
            //方法としては火属性への抵抗として表現したい

            return world.getPower(magic.HP);
        }
    }

    DQ.World.prototype.stampDamageFloor = function () {
        var avoid = false;
        for (var i = 0; i < world.party.length; i++) {
            var player = world.party[i];
            if (!player.isDead()) {
                if (player.isAvoid(null, DQ.RPG.DamageProperty.Swamp)) {
                    avoid = true;
                } else {
                    player.hp(player.hp() - 1);
                    if (player.isDead()) {
                        //個人死亡時の処理
                    }
                }
            }
        }
        if (!avoid) {
            world.screen.flashEx("Red", 1, 15.625 * 2, 2, null);
        }
        if (world.party.totalDestruction()) {
            world.showMessage(DQ.format("{0}は、気絶した。", world.party[0].name), function () {
                world.gameOver();
            });
        }
    }

    DQ.World.prototype.startBattle = function (options) {
        options = DQ.options(options, {
            sound: "battle2",
            enemy: [],
            lvBox: world.levelBox,
            callback: null
        });
        if (options.enemy.length == 0) {
            throw new Error(DQ.format("DQ.World.statBattle:: enemy must specify."));
        }
        if (options.callback != null && options.callback != world.postBattle) {
            var user_callback = options.callback;
            options.callback = function (engine, state) {
                var win = world.postBattle(engine, state);
                user_callback(win);
            }
        }
        var snd = world.sounds.find("name", options.sound);
        snd && world.playSound(snd.src);

        this.battle.chip = this.map.data[this.map.mapY + DQ.OY][this.map.mapX + DQ.OX];
        this.battle.setup(world.party, options.enemy, options.lvBox, options.callback);
    }

    DQ.World.prototype.postBattle = function (engine, state) {

        //ターン終了イベント処理
        var win = false;
        if (state == DQ.RPG.BEngine.State.BattleEnd) {
            //戦闘が終了

            //パーティー全滅
            if (world.battle.party.totalDestruction()) {
                world.gameOver();
                return;
            }

            world.steps = 0;

            var reward = world.getReward(engine.enemy),
                    msg = [];

            if (world.battle.party.length == 0) {
                msg.push(DQ.format("{0} は、逃げ出した。", world.party[0].name));
            } else {
                msg.push(DQ.format("敵をやっつけた。"));
                win = true;
            }

            //戦利品の分配
            var bParty = new DQ.RPG.Party();
            for (var i = 0; i < world.party.length; i++) {
                if (world.party[i].entryBattle) {
                    bParty.push(world.party[i]);
                }
            }
            world.shareReward(bParty, reward, msg);
            //レベル判定
            world.levelUp(world.party, msg);

            world.events.playSound();
            if (msg.length) {
                engine.visual.messageBox.waitLastClick = true;
                engine.visual.messageBox.push(msg);
                engine.visual.messageBox.show(function () { engine.visual.messageBox.hide(); });
            }
        }

        return win;
    }
    DQ.World.prototype.joinMember = function (player, msg) {
        for (var i = 0; i < world.party.length; i++) {
            if (player.name == world.party[i].name) {
                break;
            }
        }
        if (i == world.party.length) {
            world.party.push(player);
            msg.push(DQ.format("{0} がパーティに加わった。", player.name));
        }
    }
    DQ.World.prototype.leaveMember = function (player, msg) {
        world.party.remove(player);
        msg.push(DQ.format("{0} がパーティを抜けた。", player.name));
    }
});
