﻿/// <reference path="../../js/jquery.js" />
/// <reference path="../../js/dq.js" />
/// <reference path="../../js/dq-core.js" />
/// <reference path="../../js/dq-rpg.js" />
/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0006
*/

DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg.js", "DQ.RPG.InputBox", function () {
    DQ.RPG.MagicCatalog = function (options) {
        /// <summary>
        ///     まほうのカタログを管理
        /// </summary>

        options = DQ.options(options, {
            url: "data/magic.txt",
            callback: null
        });

        this.onLoaded = options.callback;
        this.url = options.url;
        arguments.length && this.load(this.url);
    }

    DQ.RPG.MagicCatalog.prototype = {
        catalog: [],
        onLoaded: function () { },
        load: function (url) {
            var me = this;
            DQ.loadCatalog(url, function (data) {
                try {
                    eval('var s=' + data);
                } catch (e) {
                    me.onLoaded && me.onLoaded(me, false);
                    throw new Error(DQ.format("DQ.RPG.MagicCatalog.load: format error.({0})", +e.message));
                }
                me._data = s.catalog || s.data;
                me.version = s.version;
                me.catalog = new Array();
                for (var i = 0; i < me._data.length; i++) {
                    me.catalog[me._data[i].uid] = new DQ.RPG.Magic(me._data[i]);
                }
                me.onLoaded && me.onLoaded.apply(me);
            });
        },
        find: function (title) {
            var c = this.catalog;
            for (var nm in c) {
                if (c.hasOwnProperty(nm) && c[nm].title == title) {
                    return c[nm];
                }
            }
            return null;
        },
        choose: function (titles) {
            var r = Math.floor(Math.random() * titles.length);
            return this.find(titles[r]);
        }
    }

    DQ.RPG.Magic = function (options) {
        ///<summary>
        ///魔法(呪文)の効果や判定を管理します。
        ///</summary>

        options = DQ.options(options, {
            property: DQ.RPG.DamageProperty.Magic

        });
        if (options.type == "fire") {
            options.property = DQ.RPG.DamageProperty.Fire;
        }
        for (var nm in options) {
            options.hasOwnProperty(nm) && (this[nm] = options[nm]);
        }
        !this.text && (this.text = this.name);
        this._strategy = this._buildStrategy(options);
    }
    DQ.RPG.Magic.prototype = {
        _buildStrategy: function (options) {
            for (var i = 0; i < DQ.RPG.Magic.table.length; i++) {
                if (DQ.RPG.Magic.table[i].type == options.type || DQ.RPG.Magic.table[i].uid == options.uid) {
                    return new DQ.RPG.Magic.table[i].strategy(this);
                }
            }
            return new DQ.RPG.MagicStg(this);
        },
        __state: null,
        state: function (value) {
            if (!arguments.length) {
                return this.__state;
            }
            if (value instanceof DQ.RPG.State) {
                this.__state = value;
            } else {
                this.__state = new DQ.RPG.State(value);
            }
        },
        chant: function (from, to, text) {
            text.push(DQ.format("{0} は、 {1} を唱えた。", from.name, this.text));
            if (!this._strategy.judge(from, to, text)) {
                return false;
            }

            this._strategy.chant(from, to, text);
            from.mp(from.mp() - this.MP);
            return true;
        }
    }

    DQ.RPG.Magic.Category = { Heal: "回復", Attack: "攻撃", Indirect: "間接", Party: "パーティ" };
    DQ.RPG.Magic.table = [];

    DQ.RPG.MagicStg = function (owner) {
        this.owner = owner;
    }
    DQ.RPG.MagicStg.prototype = {
        judge: function (from, to, text) {
            if (from.mp() < this.owner.MP) {
                text.push(DQ.format("MPが足りないようだ。"));
                return false;
            }
            if (!from.canChant()) {
                text.push(DQ.format("{0}は、呪文を封じられている。", from.name));
                return false;
            }
            return true;
        },
        chant: function (from, to, text) {
            var dmg = this.owner.HP, delta;
            dmg = world.getMagicDamage(this.owner, from, to);
            dmg = to.gainDamage(this.owner.property, dmg);
            dmg = dmg < 0 ? 0 : dmg;

            if (to.friend == from.friend) {
                delta = (to.hp() + dmg >= to.maxHP()) ?
                        to.maxHP() - to.hp() : dmg;
                if (delta == 0) {
                    text.push(DQ.format("{0} は 回復しなかった。", to.name));
                } else {
                    text.push(DQ.format("{0} ポイントのHPを回復した。", delta));
                }
                to.hp(to.hp() + delta);
            } else {
                delta = (to.hp() - dmg <= 0) ? to.hp() : dmg;
                if (delta == 0) {
                    text.push(DQ.format("{0} の魔法は効かなかった。", from.name));
                } else {
                    text.push(DQ.format("{0} ポイントのダメージを与えた。", delta));
                }
                to.hp(to.hp() - delta);
            }
            return dmg;
        },
        update: function () {
        }
    }

    DQ.RPG.Magic.Indirect = dqextend(DQ.RPG.MagicStg, function (owner) {
        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            //二重は無し
            var status = new DQ.RPG.State(world.statuses.find("uid", this.owner.status));
            for (var i = 0; i < to.temporary.length; i++) {
                if (to.temporary[i].uid == this.owner.status) {
                    text.push(DQ.format(status.text, to.name));
                    return false;
                }
            }
            if (!world.hitMagic(this.owner, from, to)) {
                text.push(DQ.format("{0} には、効かなかった。", to.name));
                return false;
            } else {
                return true;
            }
        },
        chant: function (from, to, text) {
            var status = new DQ.RPG.State(world.statuses.find("uid", this.owner.status));
            to.temporary.push(status); //状態を追加する
            text.push(DQ.format(status.a_text, to.name));
            return true;
        },
        update: function () {
        }
    });

    DQ.RPG.Magic.Fire = dqextend(DQ.RPG.MagicStg, function (owner) {
        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            if (!world.hitMagic(this.owner, from, to)) {
                text.push(DQ.format("{0} に、かわされた。", to.name));
                return false;
            } else {
                return true;
            }
        },
        chant: function (from, to, text) {
            var dmg = this.owner.HP, delta;
            dmg = world.getMagicDamage(this.owner, from, to);
            dmg = to.gainDamage(this.owner.property, dmg);

            delta = (to.hp() - dmg <= 0) ? to.hp() : dmg;
            if (delta == 0) {
                text.push(DQ.format("{0} の魔法は効かなかった。", from.name));
            } else {
                text.push(DQ.format("{0} ポイントのダメージを与えた。", delta));
            }
            to.hp(to.hp() - delta);

            return dmg;
        },
        update: function () {
        }
    });

    DQ.RPG.Magic.Heal = dqextend(DQ.RPG.MagicStg, function (owner) {
        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            if (to.hp() == to.maxHP()) {
                text.push("HPがいっぱいだ。");
                return false;
            }
            return true;
        },
        chant: function (from, to, text) {
            var st = to.parmanent,
                hp = world.getMagicDamage(this.owner, from, to),
                delta = (to.hp() + hp >= to.maxHP()) ?
                        to.maxHP() - to.hp() : hp;
            if (delta < 0) {
                delta = 0;
            }
            if (delta == 0) {
                text.push(DQ.format("{0} は 回復しなかった。", to.name));
            } else {
                text.push(DQ.format("{0} ポイントのHPを回復した。", delta));
            }
            to.hp(to.hp() + delta);
        }
    });

    DQ.RPG.Magic.Poison = dqextend(DQ.RPG.MagicStg, function (owner) {
        ///<summary>
        ///毒のストラテジー
        ///</summary>

        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            //既に侵されていれば失敗。
            for (var i = 0; i < to.temporary.length; i++) {
                if (to.temporary[i]._strategy instanceof DQ.RPG.State.Poison) {
                    text.push(to.name + "は、既に毒に侵されている。");
                    return false;
                }
            }
            if (!world.hitMagic(this.owner, from, to)) {
                text.push(to.name + "には、効果が無かった。");
                return false;
            } else {
                return true;
            }
        },
        chant: function (from, to, text) {
            to.temporary.push(new DQ.RPG.State(this.owner)); //状態を追加する
            text.push(to.name + "は、毒に侵された。");
            return true;
        }
    });

    DQ.RPG.Magic.Cure = dqextend(DQ.RPG.MagicStg, function (owner) {
        ///<summary>
        ///治癒のストラテジー
        ///</summary>

        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            for (var i = 0; i < to.temporary.length; i++) {
                if (to.temporary[i]._strategy instanceof DQ.RPG.State.Poison) {
                    return true; //100%成功
                }
            }
            return false;
        },
        chant: function (from, to, text) {
            for (var i = 0; i < to.temporary.length; i++) {
                if (to.temporary[i]._strategy instanceof DQ.RPG.State.Poison) {
                    to.temporary.remove(i); //要素の削除
                    text.push(to.name + "は毒が治った。");
                    return true;
                }
            }
            text.push(to.name + "は毒に侵されていない。");
            return false;
        }
    });

    DQ.RPG.Magic.Party = dqextend(DQ.RPG.MagicStg, function (owner) {
        ///<summary>
        ///治癒のストラテジー
        ///</summary>

        this.base(owner);
    }, {
        judge: function (from, to, text) {
            if (!__dqsuper__(this, "judge", arguments)) {
                return false;
            }
            switch (this.owner.uid) {
                case 10004:
                    if (!world.map.dungeon) {
                        text.push(DQ.format("ここでは、使用できない。"));
                        return false;
                    }
                    break;
                case 10006:
                    var map = world.maps.find("name", world.map.key);
                    if (map == null) {
                        throw new Error(DQ.format("指定のマップが見つかりません。:world.map.key = {0}", world.map.key));
                    }
                    if (map.type == "field") {
                        text.push(DQ.format("ここでは、使用できない。"));
                        return false;
                    }
                    break;
                case 10007:
                    if (world.map.dungeon) {
                        text.push(DQ.format("ダンジョンでは使用できない。"));
                        return false;
                    }
                    break;
            }
            return true;
        },
        chant: function (from, to, text) {
            switch (this.owner.uid) {
                case 10004:
                    if (world.party.flag[DQ.RPG.Party.FlagPos.Torch]) {
                        text.push(DQ.format("すでに使用している。"));
                        return false;
                    }
                    result = world.lightUp();
                    if (result) {
                        text.push(DQ.format("周囲があかるくなった。"));
                    } else {
                        text.push(DQ.format("ここでは、使用できない。"));
                    }
                    break;
                case 10006:
                    var name = world.map.key;
                    var map = world.maps.find("name", name);
                    if (map.type == "field") {
                        text.push(DQ.format("ここでは、使用できない。"));
                        return false;
                    }
                    world.jump("map", map.position[0], map.position[1]);
                    break;
                case 10007:
                    world.jump(to.name, to.x, to.y);
                    break;
            }
            return false;
        }
    });

    DQ.RPG.Magic.table.push({ type: "heal", strategy: DQ.RPG.Magic.Heal });
    DQ.RPG.Magic.table.push({ type: "fire", strategy: DQ.RPG.Magic.Fire });
    DQ.RPG.Magic.table.push({ type: "cure", strategy: DQ.RPG.Magic.Cure });
    DQ.RPG.Magic.table.push({ type: "poison", strategy: DQ.RPG.Magic.Poison });
    DQ.RPG.Magic.table.push({ type: "indirect", strategy: DQ.RPG.Magic.Indirect });
    DQ.RPG.Magic.table.push({ type: "party", strategy: DQ.RPG.Magic.Party });

});