﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0003
*/

DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg.js", "DQ.RPG.InputBox", function () {
    DQ.RPG.ShopBox = dqextend(DQ.Control, function (screen, options) {
        options = DQ.options(options, {
            cls: "dq-item",
            player: null,
            disableNotUse: true
        });

        this.player = options.player;
        this.disableNotUse = options.disableNotUse;

        this.base(screen, $('<div>'));
        this.visible(false);

        this._create(screen, options);
    }, {
        onUse: null,
        _create: function (screen, options) {
            this._group = new DQ.StateGroup(screen, [
                {
                    name: 'who', options: {
                        left: '21em',
                        top: '10em',
                        width: '6em',
                        canSelect: true,
                        text: _("From")
                    }
                },
                {
                    name: "cmd", options: {
                        width: '8em',
                        height: 'auto',
                        left: '21em',
                        top: '5em',
                        showName: false,
                        canSelect: true,
                        text: _("Action"),
                        data: [
                            { text: _("Buy") },
                            { text: _("Sell") },
                            { text: _("Cancel")}]
                    }
                },
                {
                    name: "item", options: {
                        cls: 'dq-item',
                        left: '7em',
                        top: '2em',
                        width: '14em',
                        height: '11em',
                        row: 10,
                        canSelect: true
                    }
                },
                {
                    name: "desc", options: {
                        cls: 'dq-descrip',
                        left: '7em',
                        top: '16em',
                        width: '22em',
                        height: '8em',
                        showName: true,
                        canSelect: false,
                        text: _("Description")
                    }
                },
                {
                    name: 'G', options: {
                        cls: 'dq-ebox',
                        width: '8em',
                        height: 'auto',
                        left: '21em',
                        top: '2em',
                        showName: false,
                        canSelect: false
                    }
                },
                {
                    name: 'to', options: {
                        width: '6em', height: 'auto',
                        left: DQ.L1, top: DQ.T3, showName: false, canSelect: true,
                        text: _("To")
                    }
                }
            ]);

            var me = this,
                _item, _command, _pos, _itemPos, _player,
                _to, _toPos,
            party_toString = function (party) {
                var data = [];
                for (var i = 0; i < party.length; i++) {
                    data.push({ text: party[i].name });
                }
                return data;
            },
            getCurrent = function (player) {
                var it = [], names = ["つかうもの", "だいじなもの"];
                var ct = world.items.catalog;
                for (var i = 0; i < player.items.length; i++) {
                    var itemInfo = player.items[i],
                    item = ct[itemInfo.uid];

                    if (item && item.category == names[0]) {
                        it.push({
                            'text': ['<span class="name">', item.text, '</span><span class="count">', itemInfo.count, '</span>'].join(''),
                            'orgPos': i
                        });
                    }
                }

                return it;
            },
            _setupInfo = function (sender, pos) {
                var data = [];

                _uid = _item.uid;
                _serial = _item.serial;
                desc = world.items.catalog[_uid].description;
                me._group.desc.text(_item.text);

                for (var i = 0; i < world.party.length; i++) {
                    var player = world.party[i];
                    var orgA = player.getAttack(),
                        orgD = player.getDefence(),
                        nxtA,
                        nxtD,
                        desc,
                        back;

                    if (me.shop.items[pos].type == "item") {
                        orgA = 0;
                        for (var j = 0; j < player.items.length; j++) {
                            if (player.items[j].uid == _item.uid) {
                                orgA = player.items[j].count;
                            }
                        }
                        data.push(
                        { 'text': ['<span class="dq-st-nm">', player.name,
                                    '</span><span class="dq-st-va">', orgA, '</span><span>→</span><span class="dq-st-va">', orgA + 1, '</span>'].join('')
                        }
                            );
                    } else {
                        switch (_item.type) {
                            case 'weapon':
                                back = player.equipmentIDs().weapon.uid;
                                player.equipmentIDs().weapon.uid = _uid;
                                break;
                            case 'armor':
                                back = player.equipmentIDs().armor.uid;
                                player.equipmentIDs().armor.uid = _uid;
                                break;
                            case 'shield':
                                back = player.equipmentIDs().shield.uid;
                                player.equipmentIDs().shield.uid = _uid;
                                break;
                            case 'accesary':
                                back = player.equipmentIDs().accesary.uid;
                                player.equipmentIDs().accesary.uid = _uid;
                                break;
                        }
                        nxtA = player.getAttack();
                        nxtD = player.getDefence();
                        switch (_item.type) {
                            case 'weapon':
                                player.equipmentIDs().weapon.uid = back;
                                break;
                            case 'armor':
                                player.equipmentIDs().armor.uid = back;
                                break;
                            case 'shield':
                                player.equipmentIDs().shield.uid = back;
                                break;
                            case 'accesary':
                                player.equipmentIDs().accesary.uid = back;
                                break;
                        }
                        data.push(
                            { 'text': ['<span class="dq-st-nm">', player.name, '</span><span class="dq-st-st">ATK:</span><span class="dq-st-va">', orgA,
                                        '</span><span>→</span><span class="dq-st-va">', nxtA, '</span>'].join('')
                            });
                        data.push(
                            { 'text': ['<span class="dq-st-nm">&nbsp;</span><span class="dq-st-st">DEF:</span><span class="dq-st-va">', orgD,
                                        '</span><span>→</span><span class="dq-st-va">', nxtD, '</span>'].join('')
                            });
                    }
                }

                data.push({ 'text': ['<div class="dq-st-des">', desc, '</div>'].join('') });
                me._group.desc.setData(data);
            }

            if (options.parentMenu) {
                options.parentMenu.setSubItem(this._group.who);
            }

            this._group.onVisibleChanged = function (group, sender, visible) {
                if (!visible && sender.name == "cmd") {
                    me._group.cmd.hide();
                    me._group.who.hide();
                    world.screen.showMessage("ありがとうございました！\n今後とも、ごひいきに！", function () {
                        me.hide();
                    });
                }
                if (!visible && sender.name == "item") {
                    if (me._group.who.visible()) {
                        me._group.desc.hide();
                        me._group.focus("who");
                    } else {
                        me._group.desc.hide();
                        me._group.focus("cmd");
                    }
                }
                if (!visible && sender.name == "who") {
                    me._group.focus("cmd");
                }
                if (!visible && sender.name == "to") {
                    me._group.focus("item");
                }
            }
            this._group.onSelectedIndexChanged = function (group, sender, pos) {
                _pos = pos;
                if (sender.name == "who") {
                    _player = world.party[pos];
                    group.item.setData(getCurrent(_player));
                } else if (sender.name == "to") {
                    _toPos = pos;
                    _to = world.party[pos];
                } else if (sender.name == "item") {
                    if (_command == 0) {
                        _item = world.items.catalog[me.shop.items[pos].uid];
                        _setupInfo(sender, pos);
                    } else if (_command == 1) {
                        _item = me.player.items[group.item.data[pos].orgPos];
                        _item.type = world.items.catalog[_item.uid].type;
                        _item.text = world.items.catalog[_item.uid].text;
                    } else {
                        return;
                    }
                }
            }
            function _msgBox_visibleChanged() {
                me._group.G.setData([{ text: '<span id="gold">' + world.party.G + 'G</span>'}]);
                world.screen.msgBox().onVisibleChanged = null;
                world.screen.msgBox().hide();
                me._group.item.hide();
                if (me._group.who.visible()) {
                    me._group.desc.hide();
                    me._group.who.hide();
                    me._group.focus("cmd");
                } else {
                    me._group.desc.hide();
                    me._group.focus("cmd");
                }
            }
            function _buy(pos) {
                var item = world.items.catalog[me.shop.items[pos].uid];
                if (item == null) {
                    throw new Error(DQ.format(
                        "DQ.RPG.Shop._buy: Item not found pos:{0}, uid: {1}", pos, me.shop.items[pos].uid));
                }
                var item = item.clone();
                var serial = _to.buy(item.clone());
                if (!serial) {
                    world.screen.showMessage(
                        "金が足りない。",
                        _msgBox_visibleChanged);
                } else {
                    item.serial = serial;
                    if (item.category == DQ.RPG.Item.Category.Item ||
                        item.category == DQ.RPG.Item.Category.Event) {
                        world.screen.showMessage([
                            item.text + "ですね。\n少々お待ちを！",
                                "はい、どうぞ！" + _to.name + "さん",
                                "ほかにもまだ御用は？"],
                            _msgBox_visibleChanged);
                    } else {
                        world.screen.showMessage([
                            item.text + "ですね。\n少々お待ちを！",
                                "はい、どうぞ！" + _to.name + "さん",
                        // コマンドから装備してみる。
                                "今すぐ装備するかい？",
                                "$yesno:3",
                                "せっかく買ったんだから装備しなくちゃね",
                                '$cmd:world.equipWeapon(' + _toPos + ', { serial: "' + item.serial + '", uid: ' + item.uid + ', type: "' + item.type + '"});',
                                "$shift:1",
                                "ほかにもまだ御用は？"
                                ],
                            _msgBox_visibleChanged);

                    }
                }
            }
            function _sell(pos) {
                var witem = world.items.catalog[_item.uid];
                world.screen.showMessage([
                    _item.text + "ですね、かしこまりました。",
                    "その品物なら、\n" + witem.Sell + "ゴールドで\n引き取りますが、いいですか？", "$yesno:2", "ありがとうございました。","$shift:1","そうですか"],
                    function () {
                        var data = [];
                        if (world.screen.msgBox().yesOrNo == 0) {
                            _to.sell(witem);
                        } else {
                            data.push("また、お願いします。");
                        }
                        data.push("ほかにもまだ御用は？");
                        world.screen.showMessage(data, _msgBox_visibleChanged);
                    });
            }
            this._group.onClick = function (group, sender) {
                if (sender.name == "who") {
                    group.item.show();
                    group.focus("item");
                    me.setupSell(_player);
                } else if (sender.name == "to") {
                    me._group.to.hide();
                    _buy(_itemPos);
                } else if (sender.name == "item") {
                    _item = me.player.items[group.item.data[sender.selectedIndex()].orgPos];
                    _itemPos = sender.selectedIndex();
                    if (_command == 0) {
                        if (world.party.length > 1) {
                            group.focus("to");
                            group.to.show(true);
                        } else {
                            _to = world.party[0];
                            _buy(_itemPos);
                        }
                    } else {
                        _to = world.party[0];
                        _sell(_itemPos);
                    }
                } else if (sender.name == "cmd") {
                    _command = sender.selectedIndex();
                    switch (_command) {
                        case 0:
                            me.setupBuy(me.shop);
                            me._group.item.show();
                            me._group.desc.show();
                            me._group.focus("item");
                            break;
                        case 1:
                            if (world.party.length > 1) {
                                me._group.who.show();
                            } else {
                                world.screen.showMessage(
                                    "どの品物を売ってくださいますか？",
                                    function () {
                                        me.setupSell(world.party[0]);
                                        me._group.item.show();
                                        me._group.focus("item");
                                        setTimeout(function () {
                                        }, 0);
                                    });
                            }
                            break;
                        case 2:
                            me.hide();
                            break;
                    }

                } else if (sender.name == "to") {
                    var hide = false,
                        to = world.party[sender.selectedIndex()];
                    switch (_command) {
                        case 0:
                            hide = me.onUse && me.onUse(me, to, _item, _itemPos) || true;
                            break;
                        case 1:
                            world.trade(me.player, to, _item);
                            me.update();
                            break;
                    }
                    hide && me.hide();
                }
            }
            this._group.onLeave = function (group, sender) {
            }
        },
        shopOpen: function () {
        },
        player: null,
        setup: function (player) {
            player = (player instanceof DQ.RPG.Creature) ? player : world.party[0];
            this.player = player;
            this._group.G.setData([{ text: '<span id="gold">' + world.party.G + 'G</span>'}]);

            var party_toString = function (party) {
                var data = [];
                for (var i = 0; i < party.length; i++) {
                    data.push({ text: party[i].name });
                }
                return data;
            }

            //だれ
            this._group.who.setData(party_toString(world.party));
        },
        setupBuy: function (shop) {
            shop = (shop instanceof DQ.RPG.ShopInfo) ? shop : world.shop;
            this.shop = shop;

            var it = [],
            //names = ["つかうもの", "だいじなもの"],
                ct = world.items.catalog;


            var data = [];
            for (var i = 0; i < shop.items.length; i++) {
                var nm = world.items.catalog[shop.items[i].uid].text;
                data.push({ 'text': ['<span class="name">', nm, '</span><span class="count">', shop.items[i].Buy, '</span>'].join('') });
            }

            this._group.item.setData(data);

            var member = [];
            for (var i = 0; i < world.party.length; i++) {
                member.push({ text: world.party[i].name });
            }
            this._group.to.setData(member);
        },
        setupSell: function (player) {
            player = (player instanceof DQ.RPG.Creature) ? player : world.party[0];
            this.player = player;

            var it = [],
            //names = ["つかうもの", "だいじなもの"],
                ct = world.items.catalog;

            for (var i = 0; i < player.items.length; i++) {
                var itemInfo = player.items[i],
                    item = ct[itemInfo.uid];

                if (item) {
                    it.push({
                        text: ['<span class="name">', item.text, '</span><span class="count">', itemInfo.count, '</span>'].join(''),
                        orgPos: i,
                        enabled: item.category != "だいじなもの" && !itemInfo.equip
                    });
                }
            }

            this._group.item.setData(it);

            var member = [];
            for (var i = 0; i < world.party.length; i++) {
                member.push({ text: world.party[i].name });
            }
            this._group.to.setData(member);
        },
        show: function () {
            __dqsuper__(this, "show", arguments);

            this._group.G.show();
            this._group.cmd.show();
            this._group.who.hide();
            this._group.item.hide();
            this._group.to.hide();

            this._group.cmd.select();
            this.onVisibleChanged && this.onVisibleChanged(this, true);

            world.levelBox.hide();
        },
        hide: function () {
            if (!this._visible) {
                return;
            }
            __dqsuper__(this, "hide", arguments);
            this._group && this._group.hide();
            this.onVisibleChanged && this.onVisibleChanged(this, false);
            world.levelBox && world.levelBox.show();
        },
        update: function () {
            this.setup();
        }
    });

    DQ.RPG.Shop = function (screen) {
        this.screen = screen;
        this.shopBox = new DQ.RPG.ShopBox(screen);
    }

    DQ.RPG.Shop.prototype = {
        opened: false,
        setup: function (party, shopInfo, callback) {
            var me = this;

            me.opend = true;
            me.shopBox.shop = shopInfo;
            world.screen.showMessage(
                ["いらっしゃいませ！\nここは、", shopInfo.name, "です。\nどんな、ごようですか？"].join(''),
                function () {
                    me.shopBox.show();
                });

            me.shopBox.onVisibleChanged = function (sender, visible) {
                if (!visible) {
                    me.opend = false;
                    callback && callback();
                }
            }

            this.shopBox.setup(party[0]);
        }
    }

    DQ.RPG.ShopInfo = function (uid, name, data) {
        this.uid = uid;
        this.name = data.name || "";
        this.items = data.items;
        this.G = data.G || 0;
    }

    DQ.RPG.ShopInfo.prototype = {
        uid: 0,
        name: "",
        items: [],
        G: 0,
        getStringList: function () {
            var res = [];
            for (var i = 0; i < this.items.length; i++) {
                var item = world.items[this.items[i].uid];
                res.push({ name: item.text, buy: this.items[i].Buy });
            }

            return res;
        }
    }

    DQ.RPG.ShopInfoCatalog = function (options) {
        /// <summary>
        ///     お店の品目データを管理します。
        /// </summary>

        options = DQ.options(options, {
            url: "data/shop.txt",
            callback: null
        });

        this.onLoaded = options.callback;
        this.url = options.url;
        this.load(this.url);
    }

    DQ.RPG.ShopInfoCatalog.prototype = {
        onLoaded: null,
        catalog: [],
        load: function (url) {
            var me = this;
            DQ.loadCatalog(url, function (data) {
                try {
                    eval('var s=' + data);
                } catch (e) {
                    me.onLoaded && me.onLoaded(me, false);
                    throw new Error(DQ.format("DQ.RPG.ShopInfoCatalog.load: format error.({0})", +e.message));
                }
                me.data = s.data;
                me.catalog = [];
                for (var i = 0; i < s.data.length; i++) {
                    var item = new DQ.RPG.ShopInfo(s.data[i].uid, s.data[i].name, s.data[i]);
                    me.catalog[item.uid] = item;
                }
                me.onLoaded && me.onLoaded.apply(me);
            });
        },
        find: function (title) {
            var res = null;
            $.each(this.data, function () {
                if (this.title == title) {
                    res = this;
                }
            });

            return res;
        }
    }

    DQ.RPG.HotelBox = dqextend(DQ.Control, function (screen, options) {
        options = DQ.options(options, {
            cls: "dq-item",
            player: null,
            disableNotUse: true
        });

        this.player = options.player;
        this.disableNotUse = options.disableNotUse;

        this.base(screen, $('<div>'));
        this.visible(false);

        this._create(screen, options);
    }, {
        onUse: null,
        _create: function (screen, options) {
            this._group = new DQ.StateGroup(screen, [
                {
                    name: "cmd", options: {
                        width: '8em',
                        height: 'auto',
                        left: DQ.L4,
                        top: '5em',
                        showName: false,
                        canSelect: true,
                        text: _("Action"),
                        data: [
                            { text: _("Stay") },
                            { text: _("Cancel")}]
                    }
                },
                {
                    name: 'G', options: {
                        cls: 'dq-ebox',
                        width: '8em',
                        height: 'auto',
                        left: DQ.L4,
                        top: '2em',
                        showName: false,
                        canSelect: false
                    }
                }
            ]);

            var me = this,
                _item, _command, _pos, _itemPos, _player,
                _to,
            party_toString = function (party) {
                var data = [];
                for (var i = 0; i < party.length; i++) {
                    data.push({ text: party[i].name });
                }
                return data;
            }


            if (options.parentMenu) {
                options.parentMenu.setSubItem(this._group.who);
            }

            this._group.onVisibleChanged = function (group, sender, visible) {
                if (!visible && sender.name == "cmd") {
                    me._group.cmd.hide();
                    if (me.hasStay) {
                        world.screen.showMessage(
                            "おはようございます。\nでは　いってらっしゃいませ。",
                            function () {
                                me.hide();
                            });
                    } else {
                        world.screen.showMessage(
                            "さよなら旅のひと。\nあまり無理を\nなさいませぬように･･･。",
                            function () {
                                me.hide();
                            });
                    }
                }
            }
            this._group.onSelectedIndexChanged = function (group, sender, pos) {
            }

            function _msgBox_visibleChanged() {
                me._group.G.setData([{ text: '<span id="gold">' + world.party.G + 'G</span>'}]);
                world.screen.msgBox().onVisibleChanged = null;
                world.screen.msgBox().hide();
                me.hide();
            }

            function _stay(G) {
                if (world.party.G < G) {
                    world.screen.showMessage(
                        "お金が足りないようですね。",
                        _msgBox_visibleChanged);
                } else {
                    world.party.G -= G;
                    me.setup(world.party);
                    world.screen.showMessage(
                        "それでは\nごゆっくりお休みくださいませ。",
                        function () {
                            world.party.each(function () {
                                this.recoverAll();
                            }, null, false);
                            me.hasStay = true;
                            var jingle = world.sounds.find("name", "hotel");
                            jingle && world.playJingle2(jingle.src);
                            world.screen.fadeIn(2500, _msgBox_visibleChanged);
                        });
                }
            }

            this._group.onClick = function (group, sender) {
                if (sender.name == "cmd") {
                    _command = sender.selectedIndex();
                    switch (_command) {
                        case 0:
                            _stay(me.shop.G);
                            break;
                        case 1:
                            me.hide();
                            break;
                    }
                }
            }
            this._group.onLeave = function (group, sender) {
            }
        },
        hasStay: false,
        setup: function (party) {
            this._group.G.setData([{ text: '<span id="gold">' + party.G + 'G</span>'}]);
            this.hasStay = false;
        },
        shop: null,
        show: function () {
            __dqsuper__(this, "show", arguments);

            this._group.G.show();
            this._group.cmd.show();
            this._group.cmd.select();
            this.onVisibleChanged && this.onVisibleChanged(this, true);
        },
        hide: function () {
            if (!this._visible) {
                return;
            }
            __dqsuper__(this, "hide", arguments);
            this._group && this._group.hide();
            this.onVisibleChanged && this.onVisibleChanged(this, false);
        },
        update: function () {
            this.setup();
        }
    });

    DQ.RPG.Hotel = function (screen) {
        this.screen = screen;
        this.hotelBox = new DQ.RPG.HotelBox(screen);
    }

    DQ.RPG.Hotel.prototype = {
        opend: false,
        setup: function (party, shopInfo, callback) {
            var me = this;

            me.opend = true;
            world.screen.showMessage(
                [shopInfo.name, "にようこそ。\n", "ひと晩、", shopInfo.G, "ゴールドですが\nお泊まりになりますか？"].join(''),
                function () {
                    me.hotelBox.show();
                });

            me.hotelBox.onVisibleChanged = function (sender, visible) {
                if (!visible) {
                    me.opend = false;
                    callback && callback();
                }
            }
            me.hotelBox.shop = shopInfo;
            me.hotelBox.setup(party);
        }
    }
});