﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="dq-ui.js" />
/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0004
*/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {
    DQ.UI.ListBox = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// ListBoxコントロールを作成します。
        /// </summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        /// <param name="object" type="string/jQuery">
        /// ListBoxコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //options 省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-listbox');

        options = DQ.options(options, {
            name: ""
        });

        this.base(parent, object);
        this.name = options.name;

        var cl = $('>select', this.obj);
        this.client = cl.length ? cl : $('<select size="3" />');
        this.client.addClass('dq-inner').attr('name', options.name).appendTo(this.obj);
        this.items = [];

        this.client.bind("change", this, function listbox$click(e) {
            var me = e.data,
            sl = me.client[0],
            pos = sl.selectedIndex;
            me._selectedValue = pos == -1 ? null : sl.options[pos].value;
            me.onSelectedIndexChanged && me.onSelectedIndexChanged(me);
            me.onSelectedValueChanged && me.onSelectedValueChanged(me);
        });

        this.show();
    }, {
        onDataSourceChanged: null,
        onDisplayMemberChanged: null,
        onSelectedIndexChanged: null,
        onSelectedValueChanged: null,
        add: function (item) {
            this.items.push(item);
            var d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                m = $('<option>').attr('value', v).text(d);

            this.client.append(m);
        },
        clear: function () {
            this.items = [];
            $('option', this.client).remove();
            this._dataSource = null;
        },
        _dataSource: null,
        dataSource: function (object) {
            if (!arguments.length) {
                return this._dataSource;
            }
            if (this._dataSource != object) {
                this._dataSource = object;
                for (var i = 0; i < object.length; i++) {
                    this.add(object[i]);
                }
                object.length && this.selectedIndex(0);
                this.onDataSourceChanged && this.onDataSourceChanged(sender);
            }
        },
        dispose: function () {
            this.client.unbind("change");
            __dqsuper__(this, "dispose");
        },
        _displayMember: "",
        displayMember: function (name) {
            if (!arguments.length) {
                return this._displayMember;
            }
            if (this._displayMember != name) {
                this._displayMember = name;
                this._draw();
                this.onDisplayMemberChanged && this.onDisplayMemberChanged(sender);
            }
        },
        _draw: function () {
            $('>option', this.client).remove();
            for (var i = 0; i < this.items.length; i++) {
                var item = this.items[i],
                d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                m = $('<option value="' + v + '">' + d + '</option>');
                this.client.append(m);
            }
        },
        enabled: function (value) {
            __dqsuper__(this, "enabled", [value]);
            if (value) {
                this.client.attr('disabled', '');
            } else {
                this.client.attr('disabled', 'disabled');
            }
        },
        _valueMember: "",
        valueMember: function (name) {
            if (!arguments.length) {
                return this._valueMember;
            }
            this._valueMember = name;
        },
        dqid: "DQ.UI.ListBox",
        getItemText: function (value) {
            return this._displayMember == "" ? item.toString() : item[this._displayMember];
        },
        getItemValue: function (pos) {
            return this._valueMember == "" ? this.items[pos].toString() : this.items[pos][this._valueMember];
        },
        items: null,
        selectedIndex: function (pos) {
            var sl = this.client[0];
            if (!arguments.length) {
                return sl.selectedIndex;
            }
            this._selectedValue = pos == -1 ? null : sl.options[pos].value;
            this._text = pos == -1 ? "" : sl.options[pos].text;

            if (sl.selectedIndex != pos) {
                sl.selectedIndex = pos;
                this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                this.onSelectedValueChanged && this.onSelectedValueChanged(this);
            }
        },
        _selectedValue: null,
        selectedValue: function (value) {
            if (!arguments.length) {
                return this._selectedValue;
            }
            var sl = this.client[0];
            for (var i = 0; i < sl.options.length; i++) {
                if (value == sl.options[i].value) {
                    sl.selectedIndex = i;
                    this._selectedValue = value;
                    this._text = sl.options[i].text;
                    this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                    this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    break;
                }
            }
        },
        show: function () {
            __dqsuper__(this, "show");
            var fs = parseInt(this.obj.css('font-size'));
            var sz = Math.floor(this.obj.height() / fs);
            this.client.attr('size', sz);
            this.client.width(this.obj.width());
        },
        text: function (value) {
            if (!arguments.length) {
                return this._text;
            }
            for (var i = 0; i < sl.options.length; i++) {
                if (value == sl.options[i].text) {
                    sl.selectedIndex = i;
                    this._selectedValue = value;
                    this._text = sl.options[i].text;
                    this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                    this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    break;
                }
            }
        }
    });

    DQ.UI.ComboBox = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// Comboxを作成します。
        /// </summary>
        /// <param name="name" type="string">
        /// 内包するSelectタグのNAME属性を指定
        /// </param>
        /// <param name="object" type="jQuery">
        /// ComboBoxコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //optionsの省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-combobox');

        options = DQ.options(options, {
            name: ""
        });

        this.base(parent, object);
        this.name = options.name;

        this.obj.addClass('dq-combobox');
        var cl = $('>select', this.obj);
        this.client = cl.length ? cl : $('<select size="1" />');
        this.client.addClass('dq-inner').attr('name', this.name).appendTo(this.obj);
        this.items = [];

        this.client.bind("change", this, function listbox$click(e) {
            var me = e.data,
            sl = me.client[0],
            pos = sl.selectedIndex;
            me._selectedValue = pos == -1 ? null : sl.options[pos].value;
            me.onSelectedIndexChanged && me.onSelectedIndexChanged(me);
            me.onSelectedValueChanged && me.onSelectedValueChanged(me);
        });

        this.show();
    }, {
        onDataSourceChanged: null,
        onDisplayMemberChanged: null,
        onSelectedIndexChanged: null,
        onSelectedValueChanged: null,
        add: function (item) {
            this.items.push(item);
            var d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                m = $('<option>').attr('value', v).text(d);

            this.client.append(m);
        },
        clear: function () {
            this.items = [];
            $('option', this.client).remove();
            this._dataSource = null;
        },
        _dataSource: null,
        dataSource: function (object) {
            if (!arguments.length) {
                return this._dataSource;
            }
            if (this._dataSource != object) {
                this._dataSource = object;
                for (var i = 0; i < object.length; i++) {
                    this.add(object[i]);
                    object[i].selected && this.selectedIndex(i);
                }
                this.client[0].selectedIndex == -1 && object.length && this.selectedIndex(0);
                this.onDataSourceChanged && this.onDataSourceChanged(sender);
            }
        },
        dispose: function () {
            this.client.unbind("change");
            __dqsuper__(this, "dispose");
        },
        _displayMember: "",
        displayMember: function (name) {
            if (!arguments.length) {
                return this._displayMember;
            }
            if (this._displayMember != name) {
                this._displayMember = name;
                this._draw();
                this.onDisplayMemberChanged && this.onDisplayMemberChanged(sender);
            }
        },
        _draw: function () {
            $('>option', this.client).remove();
            for (var i = 0; i < this.items.length; i++) {
                var item = this.items[i],
                d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                m = $('<option value="' + v + '">' + d + '</option>');

                this.client.append(m);
            }
        },
        _valueMember: "",
        valueMember: function (name) {
            if (!arguments.length) {
                return this._valueMember;
            }
            this._valueMember = name;
        },
        dqid: "DQ.UI.ComboBox",
        enabled: function (value) {
            __dqsuper__(this, "enabled", [value]);
            if (value) {
                this.client.attr('disabled', '');
            } else {
                this.client.attr('disabled', 'disabled');
            }
        },
        getItemText: function (value) {
            return this._displayMember == "" ? item.toString() : item[this._displayMember];
        },
        getItemValue: function (pos) {
            return this._valueMember == "" ? this.items[pos].toString() : this.items[pos][this._valueMember];
        },
        items: null,
        selectedIndex: function (pos) {
            var sl = this.client[0];
            if (!arguments.length) {
                return sl.selectedIndex;
            }

            this._selectedValue = pos == -1 ? null : sl.options[pos].value;
            this._text = pos == -1 ? "" : sl.options[pos].text;

            if (sl.selectedIndex != pos) {
                sl.selectedIndex = pos;
                this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                this.onSelectedValueChanged && this.onSelectedValueChanged(this);
            }
        },
        _selectedValue: null,
        selectedValue: function (value) {
            if (!arguments.length) {
                return this._selectedValue;
            }
            var sl = this.client[0];
            for (var i = 0; i < sl.options.length; i++) {
                if (value == sl.options[i].value) {
                    sl.selectedIndex = i;
                    this._selectedValue = value;
                    this._text = sl.options[i].text;
                    this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                    this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    break;
                }
            }
        },
        show: function () {
            __dqsuper__(this, "show");
            this.client.width(this.obj.width());
        },
        text: function (value) {
            if (!arguments.length) {
                return this._text;
            }
            for (var i = 0; i < sl.options.length; i++) {
                if (value == sl.options[i].text) {
                    sl.selectedIndex = i;
                    this._selectedValue = value;
                    this._text = sl.options[pos].text;
                    this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                    this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    break;
                }
            }
        }
    });

    DQ.UI.DropDownBox = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// Comboxを作成します。
        /// </summary>
        /// <param name="name" type="string">
        /// 内包するSelectタグのNAME属性を指定
        /// </param>
        /// <param name="object" type="jQuery">
        /// ComboBoxコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //optionsの省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-dropdown');
        this.base(parent, object);
        options = DQ.options(options, {
            height: 'auto',
            width: this.obj.width() || '5em',
            name: ""
        });
        this.name = options.name;

        this.obj.addClass('dq-dropdown');
        this.client = $('<table>').attr({ cellspacing: '0', cellpadding: '0' }).width('100%').appendTo(this.obj);

        var tb = $('<tbody>').appendTo(this.client),
            tr = $('<tr>').appendTo(tb),
            td1 = $('<td>').addClass('dq-dtext').appendTo(tr),
            text_ = this._textBox = $('<input>').attr('type', 'text').addClass('dq-dw-textbox').appendTo(td1),
            td2 = this._dropDownBox = $('<td>').addClass('dq-down').appendTo(tr),
            me = this;

        //dropDownList
        this.obj.width(options.width);
        this.canvas = $('<div>').addClass('dq-dw-canvas').width(this.obj.width()).height(options.height).appendTo(parent.obj);
        this.items = [];

        var sel = $('>select', this.obj);
        if (sel.length) {
            var i = 0;
            $('>option', sel).each(function (v) {
                var item = {};
                var text = me._displayMember == null ? "text" : me._displayMember;
                var value = me._valueMember == null ? "value" : me._valueMember;
                item[text] = v.text;
                item[value] = v.value;
                me.items.push(item);

                var m = $('<div>').addClass('dq-dw-item').text(v.text).attr({ 'value': v.value, 'index': i }).appendTo(me.canvas);
                if (v.selected) {
                    me.selectedIndex(i);
                }
                i++;
            });
            sel.remove();
        }
        this._textBox.bind('change', this, function (e) {
            var me = e.data;
            me.text(this.value);
        });
        this.canvas.hide();
        this.show();
    }, {
        onDataSourceChanged: null,
        onDisplayMemberChanged: null,
        onSelectedIndexChanged: null,
        onSelectedValueChanged: null,
        add: function (item) {
            this.items.push(item);
            var d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                m = $('<div>').addClass('dq-dw-item').text(d).attr({ 'value': v, 'index': this.items.length - 1 });

            this.canvas.append(m);
        },
        canvas: null,
        clear: function () {
            this.items = [];
            $('div', this.canvas).remove();
            this._dataSource = null;
        },
        _dataSource: null,
        dataSource: function (object) {
            if (!arguments.length) {
                return this._dataSource;
            }
            if (this._dataSource != object) {
                this._dataSource = object;
                for (var i = 0; i < object.length; i++) {
                    this.add(object[i]);
                }
                this.onDataSourceChanged && this.onDataSourceChanged(sender);
                this._draw();
            }
        },
        dispose: function () {
            this.client.unbind("change");
            __dqsuper__(this, "dispose");
        },
        _displayMember: "",
        displayMember: function (name) {
            if (!arguments.length) {
                return this._displayMember;
            }
            if (this._displayMember != name) {
                this._displayMember = name;
                this._draw();
                this.onDisplayMemberChanged && this.onDisplayMemberChanged(sender);
            }
        },
        _draw: function () {
            $('>div', this.canvas).remove();
            for (var i = 0; i < this.items.length; i++) {
                var item = this.items[i],
                    d = this._displayMember == "" ? item.toString() : item[this._displayMember],
                    v = this._valueMember == "" ? item.toString() : item[this._valueMember],
                    m = $('<div>').addClass('dq-dw-item').text(d).attr({ 'value': v, 'index': i });

                this.canvas.append(m);
            }
            var me = this;
            $('.dq-dw-item', this.canvas).bind("click", this, function (e) {
                var item = e.data;
                var pos = parseInt($(this).attr('index'));
                me.selectedIndex(pos);
                me.canvas.hide();
                //DQ.page().obj.unbind('click', _dropDown$click);

            }).hover(
                function () {
                    $(this).toggleClass('dq-dw-item-hover');
                },
                function () {
                    $(this).toggleClass('dq-dw-item-hover');
                }
            );
        },
        _valueMember: "",
        valueMember: function (name) {
            if (!arguments.length) {
                return this._valueMember;
            }
            this._valueMember = name;
        },
        dqid: "DQ.UI.DropDownBox",
        getItemText: function (pos) {
            return this._displayMember == "" ? this.items[pos].toString() : this.items[pos][this._displayMember];
        },
        getItemValue: function (pos) {
            return this._valueMember == "" ? this.items[pos].toString() : this.items[pos][this._valueMember];
        },
        items: null,
        _selectedIndex: -1,
        selectedIndex: function (pos) {
            if (!arguments.length) {
                return this._selectedIndex;
            }

            this._selectedValue = pos == -1 ? null : this.getItemValue(pos);
            this._text = pos == -1 ? "" : this.getItemText(pos);
            this._textBox[0].value = this._text;
            $('div', this.canvas).removeClass('dq-dw-item-hover dq-dw-item-select');
            if (this._selectedIndex != pos) {
                this._selectedIndex = pos;
                $("div:eq(" + pos + ")", this.canvas).addClass('dq-dw-item-select');
                this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                this.onSelectedValueChanged && this.onSelectedValueChanged(this);
            }
        },
        _selectedValue: null,
        selectedValue: function (value) {
            if (!arguments.length) {
                return this._selectedValue;
            }
            $('div', this.canvas).removeClass('dq-dw-item-hover dq-dw-item-select');
            for (var i = 0; i < this.items.length; i++) {
                if (value == this.getItemValue(i)) {
                    this._selectedIndex = i;
                    this._selectedValue = value;
                    this._text = this.getItemText(i);
                    this._textBox[0].value = this._text;
                    $("div:eq(" + pos + ")", this.canvas).addClass('dq-dw-item-select');
                    this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                    this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    break;
                }
            }
        },
        show: function () {
            __dqsuper__(this, "show", arguments);

            this._dropDownBox.bind('click', this, function (e) {
                var me = e.data;
                var p = me.obj.position();
                var fm = me.findForm();
                if (fm) {
                    var pp = fm.client.position();
                    p.top += pp.top;
                    p.left += pp.left;
                } else {
                    p = me.pointToScreen(new DQ.Point(0, 0));
                    p.top = p.y + 4;
                    p.left = p.x;
                }

                me.canvas.css({ top: p.top + me.obj.height(), left: p.left });
                me.canvas.show();
                DQ.page().obj.one('click', function _dropDown$click() {
                    me.canvas.hide();
                });
                e.stopImmediatePropagation();

            }).hover(
                function () {
                    $(this).toggleClass('dq-down-hover');
                },
                function () {
                    $(this).toggleClass('dq-down-hover');
                }
            );
        },
        text: function (value) {
            if (!arguments.length) {
                return this._text;
            }
            var pre = this._selectedIndex;
            this._selectedIndex = -1;
            this._selectedValue = "";
            this._text = value;
            this._textBox[0].value = this._text;
            for (var i = 0; i < this.items.length; i++) {
                if (value == this.getItemText(i)) {
                    this._selectedIndex = i;
                    this._selectedValue = this.getItemValue(i);
                    $("div:eq(" + i + ")", this.canvas).addClass('dq-dw-item-select');
                    if (pre != i) {
                        this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
                        this.onSelectedValueChanged && this.onSelectedValueChanged(this);
                    }
                    break;
                }
            }
        }
    });

    DQ._ctrl_['dq-listbox'] = DQ.UI.ListBox;
    DQ._ctrl_['dq-combobox'] = DQ.UI.ComboBox;
    DQ._ctrl_['dq-dropdownbox'] = DQ.UI.DropDownBox;
});
