﻿/// <reference path="jquery.js" />
/// <reference path="dq.js" />
/// <reference path="dq-core.js" />
/// <reference path="ui/dq-ui.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0005
*/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {

    DQ.UI.MessageBox = {}
    DQ.UI.MessageBox.Buttons = { OK: 1, OKCancel: 2, AbortRetryIgnore: 3, YesNoCancel: 4, YesNo: 5, RetryCancel: 6 }
    DQ.UI.MessageBox.Icon = { None: 1, Hand: 2, Exclamation: 3, Asterisk: 4, Question: 5 }
    DQ.UI.MessageBox.DefaultButton = { Button1: 1, Button2: 2, Button3: 3 }

    DQ.UI.MessageBox.show = function _msgBox$show(message, caption, button, icon, defaultButton, callback) {
        /// <summary>
        /// メッセージボックスを表示します。
        /// </summary>
        /// <param name="message" type="string">
        /// 表示するメッセージを指定
        /// </param>
        /// <param name="caption" type="string">
        /// ヘッダーへ表示するタイトルを指定
        /// </param>
        /// <param name="button" type="DQ.UI.MessageBox.Buttons">
        /// ボタンの種類を指定
        /// </param>
        /// <param name="button" type="DQ.UI.MessageBox.Icon">
        /// メッセージとともに表示するICONの種類を指定
        /// </param>

        if (!arguments.length) {
            throw new DQ.InvalidArguments("messageを指定してください。");
        }

        //省略された引数を補完
        callback = (typeof arguments[arguments.length - 1] == "function") ? arguments[arguments.length - 1] : null;
        caption = (typeof caption != "function") && caption || "";
        button = (typeof button != "function") && button || DQ.UI.MessageBox.Buttons.OK;
        icon = (typeof icon != "function") && icon || DQ.UI.MessageBox.Icon.Asterisk;
        defaultButton = (typeof defaultButton != "function") && defaultButton || DQ.UI.MessageBox.DefaultButton.Button1;

        var me = this;

        var form = DQ.UI.MessageBox._form = new DQ.UI.Form(DQ.page(), {
            windowState: DQ.UI.FormWindowState.Normal,
            maximizeBox: false,
            minimizeBox: false,
            top: 100,
            left: 100,
            initializeComponent: function msgbox$initializeComponent(sender) {
                sender.text(caption);
            }
        }, $('<div>').addClass("dq-form").attr({ 'z-index': 999, width: 'auto', height: 'auto' }));
        form.formBorderStyle(DQ.UI.BorderStyle.fixedSingle);

        var tbl = $('<table>').attr({ cellspacing: "0", border: "0" }).appendTo(form.client),
            tr1 = $('<tr>').addClass("dq-mbox-top").appendTo(tbl),
            tr2 = $('<tr>').appendTo(tbl),
            btn = $('<td>').attr({ align: "right", colspan: "2" }).appendTo(tr2),
            b = [];

        switch (button - 1) {
            case 0:
                b.push(new DQ.UI.Button(form, { 'text': _("OK") }));
                b[0].dialogResult = DQ.UI.DialogResult.OK;
                break;
            case 1:
                b.push(new DQ.UI.Button(form, { 'text': _("OK") }));
                b.push(new DQ.UI.Button(form, { 'text': _("Cancel") }));
                b[0].dialogResult = DQ.UI.DialogResult.OK;
                b[1].dialogResult = DQ.UI.DialogResult.Cancel;
                break;
            case 2:
                b.push(new DQ.UI.Button(form, { 'text': _("Abort") }));
                b.push(new DQ.UI.Button(form, { 'text': _("Retry") }));
                b.push(new DQ.UI.Button(form, { 'text': _("Ignore") }));
                b[0].dialogResult = DQ.UI.DialogResult.Abort;
                b[1].dialogResult = DQ.UI.DialogResult.Retryl;
                b[2].dialogResult = DQ.UI.DialogResult.Ignore;
                break;
            case 3:
                b.push(new DQ.UI.Button(form, { 'text': _("Yes") }));
                b.push(new DQ.UI.Button(form, { 'text': _("No") }));
                b.push(new DQ.UI.Button(form, { 'text': _("Cancel") }));
                b[0].dialogResult = DQ.UI.DialogResult.Yes;
                b[1].dialogResult = DQ.UI.DialogResult.No;
                b[2].dialogResult = DQ.UI.DialogResult.Cancel;
                break;
            case 4:
                b.push(new DQ.UI.Button(form, { 'text': _("Yes") }));
                b.push(new DQ.UI.Button(form, { 'text': _("No") }));
                b[0].dialogResult = DQ.UI.DialogResult.Yes;
                b[1].dialogResult = DQ.UI.DialogResult.No;
                break;
            case 5:
                b.push(new DQ.UI.Button(form, { 'text': _("Retry") }));
                b.push(new DQ.UI.Button(form, { 'text': _("Cancel") }));
                b[0].dialogResult = DQ.UI.DialogResult.Retry;
                b[1].dialogResult = DQ.UI.DialogResult.Cancel;
                break;
            default:
                b.push(new DQ.UI.Button(form, { 'text': _("OK") }));
                b[0].dialogResult = DQ.UI.DialogResult.OK;
                break;
        }
        //ボタンの移動
        for (var i = 0; i < b.length; i++) {
            btn.append(b[i].obj);
        }
        //アイコン表示
        var iconPos = ['', 'dq-none', 'dq-hand', 'dq-exclamation', 'dq-asterisk', 'dq-question'];
        var img = $('<td>').addClass("dq-mbox-icon").appendTo(tr1).append($('<div>').addClass(iconPos[icon]));

        //テキスト表示
        $('<td>').addClass("dq-mbox-text").attr('align', "left").appendTo(tr1).html(message);

        //アクションを追加
        function _mbox_click(sender) {
            callback && callback(sender.dialogResult);
            form.close();
            delete DQ.UI.MessageBox._form;

            return false;
        }
        for (var i = 0; i < b.length; i++) {
            b[i].width('5em');
            b[i].css('position', 'static');
            b[i].obj.addClass('dq-mbox-button');
            b[i].onClick = _mbox_click;
        }

        //ボタン列の幅調整
        btn.width(100 * b.length + 8);

        //フォームサイズの調整
        form.show();
        form.height(tbl.height() + 24);
        form.width(tbl.width());

        form.startPosition(DQ.UI.FormStartPosition.CenterScreen); //幅を確定した後でないとね

        form.showDialog();
        setTimeout(function () {
            b[defaultButton - 1].focus();
        }, 80);
    }
});