﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/*
** DQ Retro game UI framework JavaScript library version v0.6.0
**
** Copyright (c) 2009-2010 M., Koji
** Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
**
** Date: 2009-11-01 17:34:21 -0900
** Revision: 0007
**
**/

DQ.getObject = function (cls) {
    return DQ._ctrl_[cls];
}
DQ.parseHtml = function (form, parent, dom) {
    /// <summary>
    ///  DQ.UI.Form用のDOMを解析してコントロールを作成します。
    /// </summary>
    /// <param name="form" type="DQ.UI.Form">
    /// 子コントロールを追加するFormコントロールを指定
    /// </param>
    /// <param name="parent" type="DQ.Control">
    /// 親コントロールを指定
    /// </param>
    /// <param name="dom" type="Element">
    /// 解析対象のDOMを指定
    /// </param>
    var c = ((typeof dom == "string") ? $('#' + dom) : dom).children();
    for (var i = 0; i < c.length; i++) {
        var cls = c[i].className || c[i].getAttribute('class'),
            id = c[i].id || c[i].getAttribute('id'),
            nextParent = parent;
        if (cls && cls.match(/^dq-/)) {
            if (DQ.getObject(cls)) {
                nextParent = form[id] = new (DQ.getObject(cls))(parent, {}, id);
            }
        }
        if (c[i].nodeName == "DIV" || c[i].nodeName == "FIELDSET") {
            DQ.parseHtml(form, nextParent, $(c[i]));
        }
    }
}

DQ.UI = {};

//DQ.Control拡張
DQ.Control.prototype.update = function () {
    this.dock(this._dock);
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].update && this.controls[i].update();
    }
}
DQ.Control.prototype.resize = function () {
    if (!this._visible) {
        return;
    }
    this.dock(this._dock);
    for (var i = 0; i < this.controls.length; i++) {
        this.controls[i].resize && this.controls[i].resize();
    }
}
DQ.Control.prototype.clientSize = function (value) {
    ///<summary>
    /// クライアント領域のサイズを取得します。
    ///</summary>
    if (arguments.length) {
        throw new Error("DQ.Control.clientSize: 現在のバージョンでは設定はできません。");
    }
    return new DQ.Size(this.client.width(), this.client.height());
}
DQ.Control.prototype.dropShadow = function () {
    ///<summary>
    /// コントロールへ影をdropShadow効果を付与する
    ///</summary>
    return;
    this._shadow = [];
    var style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], ''),
            t = parseInt(style.top == 'auto' ? 0 : style.top),
            l = parseInt(style.top == 'auto' ? 0 : style.left),
            w = this.obj.outerWidth(),
            h = this.obj.outerHeight();

    this._shadow.push($('<div>').addClass("dq-shadow"));
    this._shadow.push($('<div>').addClass("dq-shadow"));
    this._shadow.push($('<div>').addClass("dq-shadow"));
    this.resizeShadow();
}

DQ.Control.prototype.resizeShadow = function () {
    var style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], ''),
            t = parseInt(style.top == 'auto' ? 0 : style.top),
            l = parseInt(style.top == 'auto' ? 0 : style.left),
            w = this.obj.outerWidth(),
            h = this.obj.outerHeight();

    this._shadow[0].css({ top: t + 2, left: l + 2 }).width(w + 4).height(h + 4);
    this._shadow[1].css({ top: t + 3, left: l + 3 }).width(w + 2).height(h + 2);
    this._shadow[2].css({ top: t + 4, left: l + 4 }).width(w - 0).height(h - 0);
    this._insertShadow();
}
DQ.Control.prototype._insertShadow = function () {
    for (var i = 0; i < this._shadow.length; i++) {
        this.obj.before(this._shadow[i]);
    }
}
DQ.Control.prototype._shadow = [];

DQ.InvalidArguments = function (message) {
    this.name = "InvalidArguments";
    this.message = message;
}
DQ.InvalidArguments.prototype = new Error();

DQ.Control.prototype.dock = function (value) {
    ///<summary>
    ///コンテナへのコントロールの固定する境界線を定義します。
    ///</summary>
    ///<param name="value" type="DQ.UI.DockStyle">
    /// DQ.UI.DockStyleを指定します。
    ///</summary>

    if (!arguments.length) {
        return this._dock;
    }
    if (value == DQ.UI.DockStyle.None && value == this._dock) {
        return;
    }
    this._dock = value;

    var DS = DQ.UI.DockStyle;
    switch (value) {
        case DS.Left:
            this.obj.css({ 'position': 'static', 'float': 'left', 'left': '0px' });
            break;
        case DS.Right:
            this.obj.css({ 'position': 'static', 'float': 'right' });
            break;
        case DS.Top:
            this.obj.css({ 'position': 'static', 'float': 'left' });
            break;
        case DS.Bottom:
            this.obj.css({ 'position': 'absolute', 'float': 'left', 'bottom': '0px' });
            break;
        case DS.Fill:
            this.obj.css({ 'position': 'static', 'float': 'left' });
            break;
        case DS.None:
            this.obj.css({ 'position': 'absolute', 'float': 'none' });
            break;
    }

    //親がDQ.Pageならボックスの中の可能性があるので実際の親要素のサイズを取得する
    //ように修正
    var style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], ''),
            p = this.parent,
            mw = parseInt(style.marginLeft) + parseInt(style.marginRight),
            mh = parseInt(style.marginTop) + parseInt(style.marginBottom),
            pw = parseFloat(style.paddingLeft) + parseFloat(style.paddingRight),
            ph = parseFloat(style.paddingTop) + parseFloat(style.paddingBottom),
            bt = parseInt(style.borderTopWidth),
            bb = parseInt(style.borderBottomWidth),
            bl = parseInt(style.borderLeftWidth),
            br = parseInt(style.borderRightWidth),
            h = (p instanceof DQ.Page ? this.obj.parent().height() :
                (p.clientSize ? p.clientSize().height : p.height()))
                - (isNaN(mh) ? 0 : mh)
                - (isNaN(ph) ? 0 : ph)
                - (isNaN(bt) ? 0 : bt)
                - (isNaN(bb) ? 0 : bb),
            w = ((p instanceof DQ.Page) ? this.obj.parent().width() :
                (p.clientSize ? p.clientSize().width : p.width()))
                - (isNaN(mw) ? 0 : mw)
                - (isNaN(pw) ? 0 : pw)
                - (isNaN(bl) ? 0 : bl)
                - (isNaN(br) ? 0 : br),
            l = 0;
    if (p instanceof DQ.UI.GroupBox) {
        h -= 16;
    }
    //自分の前までの兄弟のサイズを反映
    for (var i = 0; i < p.controls.length; i++) {
        var v = p.controls[i];
        if (v == this) {
            break;
        }
        if (v._dock) {
            if (v._dock == DS.Top || v._dock == DS.Bottom) {
                h -= v.obj.outerHeight({ margin: true });
            }
            if (v._dock == DS.Left || v._dock == DS.Right) {
                w -= v.obj.outerWidth({ margin: true });
            }
            if (v._dock == DS.Left) {
                l += v.obj.outerWidth({ margin: true });
            }
        }
    }
    switch (value) {
        case DS.Left:
            this.height(h);
            break;
        case DS.Right:
            this.height(h);
            break;
        case DS.Top:
            this.size(w, this._height);
            break;
        case DS.Bottom:
            var pl = parseInt(pstyle.paddingLeft);
            this.obj.css({ left: l + pl, bottom: pstyle.paddingBottom });
            this.size(w, this._height);
            break;
        case DS.Fill:
            this.size(w, h);
            break;
        case DS.None:
            this.size(this._width, this._height);
            break;
    }
}

DQ.UI.DialogResult = { Abort: 0, Cancel: 1, Ignore: 2, No: 3, None: 4, OK: 5, Retry: 6, Yes: 7 }
DQ.UI.FormWindowState = { Normal: 0, Maximized: 1, Minimized: 2 }
DQ.UI.FormStartPosition = { CenterParent: 0, CenterScreen: 1, Manual: 2, WindowsDefaultBounds: 3, WindowsDefaultLocation: 4 }
DQ.UI.BorderStyle = { None: 0, fixedSingle: 1, Sizalble: 2, dockable: 3 }
DQ.UI.DockStyle = { None: 0, Top: 1, Left: 2, Bottom: 3, Right: 4, Fill: 5 }
DQ.UI.SortOrder = { Ascending: 0, Descending: 1, None: 2 }
DQ.UI.View = { LargeIcon: 0, Details: 1, SmallIcon: 2, List: 3, Tile: 4 }
DQ.UI.LayoutStyle = { None: 0, Inline: 1, Absolute: 2 }

DQ.Control.prototype._dock = DQ.UI.DockStyle.None;

DQ.UI.Panel = dqextend(DQ.Control, function dq$panel(parent, options, object) {
    /// <summary>
    /// パネルを作成します。
    /// </summary>
    ///<param name="options" type="object">
    /// オプションを指定
    ///</param>
    /// <param name="object" type="string/jQuery">
    /// Panelコントロールの外殻を指すオブジェクトを指定
    /// </param>

    //options 省略判定
    object = !object && options && (typeof options == "string" || options.jquery) && options || object;
    options = DQ.options(options, {
        width: 100,
        height: 100
    });
    object = object || $('<div>').addClass("dq-panel").height(options.height).width(options.width);

    this.base(parent, object);
    this._create(options);

}, {
    _borderStyle: DQ.UI.BorderStyle.None,
    borderStyle: function (style) {
        /// <summary>
        /// パネルの枠の種類を取得または設定します。
        /// </summary>
        ///<param name="style" type="DQ.UI.BorderStyle">
        /// 設定するDQ.UI.BorderStyleを設定
        ///</param>
        if (!arguments.length) {
            return this._borderStyle;
        } else {
            this._borderStyle = style;
            switch (style) {
                case DQ.UI.BorderStyle.None:
                    this.obj
                    .removeClass('dq-border-fixedSingle')
                    .addClass('dq-border-none');
                    this.header.hide();
                    break;
                case DQ.UI.BorderStyle.fixedSingle:
                    this.obj
                    .removeClass('dq-border-none')
                    .addClass('dq-border-fixedSingle');
                    this.header.hide();
                    break;
                case DQ.UI.BorderStyle.dockable:
                    this.obj
                    .removeClass('dq-border-none')
                    .addClass('dq-border-fixedSingle');
                    this.header.show();
                    break;
            }
        }
    },
    _windowState: DQ.UI.FormWindowState.Normal,
    _create: function (options) {
        this._location = new DQ.Point(0, 0);
        this._size = new DQ.Size(0, 0);
        var c = this.obj.children();
        this.header = $('<div>').addClass("dq-header").hide();
        this._title = $('<div>').addClass("dq-panel-title").appendTo(this.header);

        this._exit = $('<div>')
            .addClass("dq-form-button").attr('title', "hide")
            .append($('<div>').addClass("dq-form-exit")).appendTo(this.header);
        this._min = $('<div>').addClass("dq-form-button").attr('title', "close").appendTo(this.header);
        var ds = DQ.UI.DockStyle, cd = this._dock,
            dir = (cd == ds.None || cd == ds.Left) ? "left" : (cd == ds.Right) ? "right" : (cd == ds.Bottom) ? "down" : "up";
        this._minInner = $('<div>').addClass("dq-form-" + dir).appendTo(this._min);
        $('.dq-form-button', this.header).hover(
            function () {
                $(this).addClass('dq-btn-hover');
            },
            function () {
                $(this).removeClass('dq-btn-hover');
            }
        );
        var _slide_click = function (e) {
            var me = e.data;
            if (me._windowState != DQ.UI.FormWindowState.Minimized) {
                me._title.hide();
                me.client.fadeOut();
                me._backupSize = new DQ.Size(me.width(), me.height());
                var o = (me._dock == ds.Top || me._dock == ds.Bottom) ? { height: '22px'} : { width: '24px' };
                me.obj.animate(o, "normal", function () {
                    me._height = me.obj.height();
                    me._width = me.obj.width();
                    me.parent.resize();
                    switch (me._dock) {
                        case ds.None:
                        case ds.Left:
                            me._minInner.removeClass('dq-form-left').addClass('dq-form-right');
                            break;
                        case ds.Right:
                            me._minInner.removeClass('dq-form-right').addClass('dq-form-left');
                            break;
                        case ds.Bottom:
                            me._minInner.removeClass('dq-form-down').addClass('dq-form-up');
                            break;
                        default:
                            me._minInner.removeClass('dq-form-up').addClass('dq-form-down');
                            break;
                    }
                });
                me._windowState = DQ.UI.FormWindowState.Minimized;
            } else {
                if (me._dock == ds.Top || me._dock == ds.Bottom) {
                    me.height(me._backupSize.height);
                } else {
                    me.width(me._backupSize.width);
                }
                me.show();
                me._title.show();
                me.client.fadeIn();
                me.parent.resize();
                me._windowState = DQ.UI.FormWindowState.Normal;
            }
            return false;
        }
        this._exit.bind("click", this, function (e) {
            var me = e.data;
            me.hide();
            return false;
        });
        this._min.bind("click", this, _slide_click);
        this.client = $('<div>').addClass("dq-panel-body");
        this.client.append(c);
        this.text(this.obj.text());

        this.obj.empty();
        this.obj.append(this.header);
        this.obj.append(this.client);

        this.canSelect = false;
        this.visible(true);
        this.borderStyle(DQ.UI.BorderStyle.None);
        this.exitBox(options.exitBox == true);
        this.minimizeBox(options.minimizeBox == true);
    },
    clientSize: function () {
        var style = this.client[0].currentStyle || document.defaultView.getComputedStyle(this.client[0], ''),
            pw = parseInt(style.paddingLeft) + parseInt(style.paddingRight),
            ph = parseInt(style.paddingTop) + parseInt(style.paddingBottom),
            w = this.obj.width() - (isNaN(pw) ? 0 : pw),
            h = this.obj.height() - (isNaN(ph) ? 0 : ph);


        if (this.header.css('display') != 'none') {
            h -= this.header.height();
        }
        return new DQ.Size(w, h);
    },
    _dock: DQ.UI.DockStyle.None,
    dock: function (value) {
        ///<summary>
        ///コンテナへのコントロールの固定する境界線を定義します。
        ///</summary>
        ///<param name="value" type="DQ.UI.DockStyle">
        /// DQ.UI.DockStyleを指定します。
        ///</summary>

        if (!arguments.length) {
            return this._dock;
        }
        this._dock = value;
        var ds = DQ.UI.DockStyle,
            cd = this._dock,
            dir = (cd == ds.None || cd == ds.Left) ? "left" : (cd == ds.Right) ? "right" : (cd == ds.Bottom) ? "down" : "up";
        this._minInner.attr('class', null).addClass('dq-form-' + dir);

        var DS = DQ.UI.DockStyle;
        switch (value) {
            case DS.Left:
                this.obj.css({ 'position': 'static', 'float': 'left', 'left': '0px' });
                break;
            case DS.Right:
                this.obj.css({ 'position': 'static', 'float': 'right' });
                break;
            case DS.Top:
                this.obj.css({ 'position': 'static', 'float': 'left' });
                break;
            case DS.Bottom:
                this.obj.css({ 'position': 'absolute', 'float': 'left', 'bottom': '0px' });
                break;
            case DS.Fill:
                this.obj.css({ 'position': 'static', 'float': 'left' });
                break;
            case DS.None:
                this.obj.css({ 'position': 'absolute', 'float': 'none' });
                break;
        }

        //親がDQ.Pageならボックスの中の可能性があるので実際の親要素のサイズを取得する
        //ように修正
        var pstyle = this.obj.parent()[0].currentStyle || document.defaultView.getComputedStyle(this.obj.parent()[0], '');

        var style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], ''),
            p = this.parent,
            mw = parseInt(style.marginLeft) + parseInt(style.marginRight),
            mh = parseInt(style.marginTop) + parseInt(style.marginBottom),
            h = (p instanceof DQ.Page ? this.obj.parent().height() :
                (p.clientSize ? p.clientSize().height : Math.min(p.height(), this.obj.parent().height())))
                - (isNaN(mh) ? 0 : mh)
                - parseInt(style.borderTopWidth)
                - parseInt(style.borderBottomWidth),
            w = (p instanceof DQ.Page ? this.obj.parent().width() :
                (p.clientSize ? p.clientSize().width : Math.min(p.width(), this.obj.parent().width())))
                 - (isNaN(mw) ? 0 : mw)
                 - parseInt(style.borderLeftWidth)
                 - parseInt(style.borderRightWidth),
            l = 0;

        //兄弟のサイズを反映する。
        for (var i = 0; i < p.controls.length; i++) {
            var v = p.controls[i];
            if (v == this) {
                break;
            }
            var vs = v.obj[0].currentStyle || document.defaultView.getComputedStyle(v.obj[0], ''),
                vw = parseInt(vs.marginLeft) + parseInt(vs.marginRight),
                vh = parseInt(vs.marginTop) + parseInt(vs.marginBottom);

            vw = (isNaN(vw) ? 0 : vw);
            vh = (isNaN(vh) ? 0 : vh);
            if (v._dock) {
                if (v._dock == DS.Top || v._dock == DS.Bottom) {
                    h -= v.obj.outerHeight({ margin: true });
                }
                if (v._dock == DS.Left || v._dock == DS.Right) {
                    w -= v.obj.outerWidth({ margin: true });
                }
                if (v._dock == DS.Left) {
                    l += v.obj.outerWidth({ margin: true });
                }
            }
        }
        switch (value) {
            case DS.Left:
                this.height(h);
                break;
            case DS.Right:
                this.height(h);
                break;
            case DS.Top:
                this.size(w, this._height);
                break;
            case DS.Bottom:
                var pl = parseInt(pstyle.paddingLeft);
                this.obj.css({ left: l + pl, bottom: pstyle.paddingBottom });
                this.size(w, this._height);
                break;
            case DS.Fill:
                this.size(w, h);
                break;
            case DS.None:
                this.size(this._width, this._height);
                break;
        }

        w = this.obj.width() - 10;
        w -= this._minimizeBox ? 18 : 0;
        w -= this._exitBox ? 18 : 0;
        this._title.width(w);
    },
    dqid: "DQ.UI.Panel",
    _location: null,
    location: function (x, y) {
        if (!arguments.length) {
            return this._location;
        }
        if (y == null) {
            this._location = x;
            y = x.y;
            x = x.x;
        } else {
            this._location = new DQ.Point(x, y);
        }
        this.obj.css({ 'left': x, 'top': y });
    },
    dispose: function () {
        __dqsuper__(this, "dispose", arguments);
    },
    _exitBox: true,
    exitBox: function (flag) {
        if (!arguments.length) {
            return flag;
        }
        this._exitBox = flag;
        this._exitBox ? this._exit.show() : this._exit.hide();
    },
    _minimizeBox: true,
    minimizeBox: function (flag) {
        if (!arguments.length) {
            return flag;
        }
        this._minimizeBox = flag;
        this._minimizeBox ? this._min.show() : this._min.hide();
    },
    text: function (data) {
        if (!arguments.length) {
            return this._text;
        }
        this._text = data;
        this._title.text(data);
    },
    resize: function () {
        this.dock(this._dock);
        for (var i = 0; i < this.controls.length; i++) {
            var c = this.controls[i];
            c.resize && c.resize();
        }
    },
    select: function () {
        ///<summary>
        /// 所属コントロールで一番上のコントロールを選択
        ///</summary>

        if (this.controls.length == 0) {
            return;
        }
        var c = null;
        for (var i = 0; i < this.controls.length; i++) {
            var v = this.controls[i];
            if (v._visible &&
                (v.canSelect == null || v.canSelect) &&
                (v._enabled == null || v._enabled) &&
                (c == null || c.zIndex <= v.zIndex)) {
                c = v;
                if (v._focused) {
                    break;
                }
            }
        }
        c && c.select();
    }
});

DQ.UI.draggable = function (control, options) {

    //削除
    if (options == "destroy") {
        if (!control._drag_handle) {
            return;
        }
        control._drag_handle.unbind("mousedown", _drag_start);
        delete control._drag_handle;
        return;
    }

    if (control._drag_handle) {
        return;
    }
    var _stop_drag = function () {
    },
    _drag_start = function (e) {
        var me = e.data;

        me._rx = e.pageX;
        me._ry = e.pageY;

        DQ.document.bind("mousemove", me, _move);
        DQ.document.one("mouseup", me, _drag_end);

        e.preventDefault();
    },
    _move = function (e) {
        var me = e.data;
        var lrx = e.pageX;
        var lry = e.pageY;
        if (lrx != me._rx || lry != me._ry) {
            if (me._clone == null) {
                me._clone = $('<div>').addClass("dq-form dq-form-border")
                    .css({ top: me.obj.css('top'), left: me.obj.css('left') })
                    .appendTo(me.parent.client);
                me._clone.width(me.obj.width());
                me._clone.height(me.obj.height());
                me._clone.show();
                me._dragging = true;
            }
            var t = me._clone[0].offsetTop + (lry - me._ry);
            var l = me._clone[0].offsetLeft + (lrx - me._rx);
            me._clone.css({ 'top': t, 'left': l });
            me._rx = lrx;
            me._ry = lry;
        }

        e.preventDefault();
    },
    _drag_end = function (e) {
        var me = e.data;
        DQ.document.unbind("mousemove", _move);
        if (me._dragging) {
            me._dragging = false;
            me.obj.css({ 'top': me._clone.css('top'), 'left': me._clone.css('left') });
            me._resize();
            me._clone.remove();
            delete me._clone;
        }
        delete me._rx;
        delete me._ry;

        _stop_drag();

        return true;
    }

    options = options || {};
    control._drag_handle = options.handle || control.obj;
    control._drag_handle.bind("mousedown", control, _drag_start);
}

DQ.UI.Form = dqextend(DQ.Control, function (parent, options, object) {
    ///<summary>
    /// ユーザー インターフェイスを構成するダイアログ ボックスを表示するFormクラスを作成します。
    ///</summary>

    options = DQ.options(options, {
        maximizeBox: true,
        minimizeBox: true,
        onClosing: null,
        onLoaded: null,
        onVisibleChanged: null,
        onResize: null,
        onResizeBegin: null,
        onResizeEnd: null,
        parseHtml: false,
        width: 480,
        height: 240,
        formBorderStyle: DQ.UI.BorderStyle.Sizalble,
        startPosition: DQ.UI.FormStartPosition.WindowDefaultLocation,
        windowState: DQ.UI.FormWindowState.Maximized
    });
    var me = this, c;

    object = object || $('<div>').addClass('dq-form').css({ width: options.width, height: options.height });
    c = ((typeof object == "string") ? $('#' + object) : object).children();

    this.base(parent, object);
    this._createFrame(options);

    c && c.appendTo(this.client);

    this.onClosing = options.onClosing;
    this.onVisibleChanged = options.onVisibleChanged;
    this.onResize = options.onResize;
    this.onResizeBegin = options.onResizeBegin;
    this.onResizeEnd = options.onResizeEnd;

    var _max_normal = function () {
        if (me._windowState == DQ.UI.FormWindowState.Normal) {
            me.windowState(DQ.UI.FormWindowState.Maximized);
        } else {
            me.windowState(DQ.UI.FormWindowState.Normal);
        }
        return false;
    },
    _slide_click = function () {
        if (me._windowState != DQ.UI.FormWindowState.Minimized) {
            me.windowState(DQ.UI.FormWindowState.Minimized);
        } else {
            me.windowState(me._defaultWindowState);
        }
        return false;
    }

    this.text(options.text || "Form1");

    //draggable
    options.windowState != DQ.UI.FormWindowState.Maximized && DQ.UI.draggable(this, { handle: this.header });

    //resizable
    DQ.UI.resizable(this, {
        start: this._start_resize,
        stop: function _form_stop_resize() { me._stop_resize.apply(me, arguments); },
        resize: function _form_resize() { me._resize.apply(me, arguments); }
    });

    options.parseHtml && DQ.parseHtml(this, this, this.obj);
    options.initializeComponent && options.initializeComponent(this);
    this.startPosition(options.startPosition);

    //影
    this._shadow = [];
    if (options.windowState == DQ.UI.FormWindowState.Normal &&
        (typeof options.shadow == "undefined" || options.shadow != false)) {
        this.dropShadow();
    }

    if (options.windowState != DQ.UI.FormWindowState.Normal) {
        var me = this;
        setTimeout(function () {
            me.windowState(options.windowState);
        }, 0);
    }

    //ページ読み込み完了ならonLoadイベントも実行
    this.onLoaded = options.onLoaded;
    DQ.page().loaded && this.onLoaded && this.onLoaded(this);

    this.hide();

    this._exit.bind("click", function () {
        me.close.apply(me);
        return false;
    });
    this._min.bind("click", _slide_click);
    this._max.bind("click", _max_normal);
    this.header.bind("dblclick", _max_normal);
    this.obj.bind("click", function () {
        if (me.isActiveControl()) {
            return true;
        }
        me.bringToFront();
        me.select();
        return false;
    });

    this.obj.mouseup(function (e) { return me._mouseup(e); });

    this.minimizeBox(options.minimizeBox == true);
    this.maximizeBox(options.maximizeBox == true);
    this.formBorderStyle(options.formBorderStyle);

    return this;
}, {
    _shadow: [],
    onLoaded: null,
    onResizeBegin: null,
    onResizeEnd: null,
    onResize: null,
    _sth: 0,
    _start_resize: function (e, ui) {
        this.onResizeBegin && this.onResizeBegin(me, e);
    },
    _stop_resize: function (e, ui) {
        this.resize(e, ui);
        this.onResizeEnd && this.onResizeEnd(this, e);
    },
    _resize: function form$_resize(e, ui) {
        var me = this;
        me._size._height = me._height = me.obj.height();
        me._size._width = me._width = me.obj.width();
        var h = me.header.height(),
            hstyle = me.header[0].currentStyle || document.defaultView.getComputedStyle(me.header[0], ''),
            style = me.obj[0].currentStyle || document.defaultView.getComputedStyle(me.obj[0], ''),
            hh = parseFloat(hstyle.marginTop) + parseFloat(hstyle.marginBottom),
        bh = parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);
        var mr = (me.header.css('display') != "none" ? h + hh + bh : bh);

        me.client.height(me._height - mr);
        me._shadow.length && me.resizeShadow();
        me.onResize && me.onResize(me, e);
    },
    _insertShadow: function () {
        if (this.obj) {
            for (var i = 0; i < this._shadow.length; i++) {
                this.obj.before(this._shadow[i]);
            }
        }
    },
    _createFrame: function (options) {
        ///<summary>
        /// Formコントロールの外観を作成します。
        ///</summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        this.obj.addClass('dq-border-fixedSingle dq-drop-shadow');
        this.header = $('<div>').addClass('dq-form-header');
        this.header.appendTo(this.obj);

        this._logo = $('<div>').addClass('dq-form-logo').appendTo(this.header);
        this._title = $('<div>').addClass('dq-form-title').appendTo(this.header);
        this._exit = $('<div>').addClass('dq-form-button').attr('title', 'close').append($('<div>').addClass('dq-form-exit')).appendTo(this.header);
        this._max = $('<div>').addClass('dq-form-button').attr('title', 'maximize').append($('<div>').addClass('dq-form-max')).appendTo(this.header);
        this._min = $('<div>').addClass('dq-form-button').attr('title', 'minimize').append($('<div>').addClass('dq-form-min')).appendTo(this.header);
        this.client = $('<div>').addClass('dq-form-content').appendTo(this.obj);
        $('.dq-form-button', this.header).hover(
            function () {
                $(this).addClass('dq-btn-hover');
            },
            function () {
                $(this).removeClass('dq-btn-hover');
            }
        );
    },
    _activeControl: null,
    activeControl: function (control) {
        if (!arguments.length) {
            return this._activeControl;
        }
        this._activeControl && this._activeControl.onLeave && this._activeControl.onLeave(this._activeControl);
        this._activeControl = control;
    },
    _acceptButton: null,
    acceptButton: function (value) {
        if (!arguments.length) {
            return this._acceptButton;
        }
        this._acceptButton = value;
        value.dialogResult = DQ.UI.DialogResult.OK;
    },
    bringToFront: function () {
        this.parent.client.append(this.obj);
        this._insertShadow();
    },
    _cancelButton: null,
    cancelButton: function (value) {
        if (!arguments.length) {
            return this._cancelButton;
        }
        this._cancelButton = value;
        value.dialogResult = DQ.UI.DialogResult.Cancel;
    },
    clientSize: function () {
        ///<summary>
        /// クライアント領域のサイズを取得します。
        ///</summary>

        var me = this, h = me.header.height(),
            hstyle = me.header[0].currentStyle || document.defaultView.getComputedStyle(me.header[0], ''),
            style = me.obj[0].currentStyle || document.defaultView.getComputedStyle(me.obj[0], ''),
            hh = parseFloat(hstyle.marginTop) + parseFloat(hstyle.marginBottom),
        bh = parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);
        var mr = (me.header.css('display') != "none" ? h + hh + bh : bh);

        return new DQ.Size(this._width, this._height - mr - this._sth);
    },
    close: function () {
        var cancel = false;
        this.onClosing && (cancel = this.onClosing(this, cancel));
        if (cancel) {
            return;
        }
        this.onClosed && this.onClosed(this);
        var p = this.parent;
        this.dispose();
        setTimeout(function () { p.select(); }, 50);
    },
    contextMenu: null,
    dialogResult: DQ.UI.DialogResult.None,
    dispose: function () {
        DQ.UI.resizable(this, "destroy");
        this.header.unbind("dblclick");
        this.header.unbind("click");
        this.obj.unbind("mousedown");
        this._max.unbind("click");
        this._min.unbind("click");
        this._mshadow && this._mshadow.remove();
        for (var i = 0; i < this._shadow.length; i++) {
            this._shadow[i].remove();
        }
        this._gray && this._gray.remove();

        __dqsuper__(this, "dispose");
    },
    _formBorderStyle: DQ.UI.BorderStyle.Sizalble,
    formBorderStyle: function form$formBorderStyle(style) {
        var me = this;
        if (!arguments.length) {
            return this._formBorderStyle;
        } else {
            if (this._formBorderStyle == style) {
                return;
            }
            this._formBorderStyle = style;
            switch (style) {
                case DQ.UI.BorderStyle.None:
                    this.obj.removeClass('dq-border-fixedSingle');
                    this.obj.addClass('dq-form-none');
                    this.header.hide();
                    DQ.UI.resizable(this, "destroy");
                    break;
                case DQ.UI.BorderStyle.fixedSingle:
                    this.obj.removeClass('dq-border-none');
                    this.obj.addClass('dq-form-fixedSingle');
                    this.header.show();
                    DQ.UI.resizable(this, "destroy");
                    break;
                case DQ.UI.BorderStyle.Sizalble:
                    this.obj.removeClass('dq-form-none');
                    this.obj.addClass('dq-border-fixedSingle');
                    this.header.show();
                    DQ.UI.resizable(this, {
                        start: this._start_resize,
                        stop: this._stop_resize,
                        resize: function _form_resize() { me._resize.apply(me, arguments); }
                    });
                    break;
            }
            this.resize();
        }
    },
    height: function (value) {
        if (!arguments.length) {
            return this._height;
        } else {
            __dqsuper__(this, "height", arguments);
            !this._suspendLayoutFlag && this._resize();
        }
    },
    width: function (value) {
        if (!arguments.length) {
            return this._width;
        } else {
            __dqsuper__(this, "width", arguments);
            !this._suspendLayoutFlag && this._resize();
        }
    },
    _exitBox: true,
    exitBox: function (flag) {
        if (!arguments.length) {
            return flag;
        }
        this._exitBox = flag;
        this._exitBox ? this._exit.show() : this._exit.hide();
    },
    _maximizeBox: true,
    maximizeBox: function (flag) {
        if (!arguments.length) {
            return flag;
        }
        this._maximizeBox = flag;
        this._maximizeBox ? this._max.show() : this._max.hide();
    },
    _minimizeBox: true,
    minimizeBox: function (flag) {
        if (!arguments.length) {
            return flag;
        }
        this._minimizeBox = flag;
        this._minimizeBox ? this._min.show() : this._min.hide();
    },
    dqid: "DQ.UI.Form",
    header: null,
    keyPreview: false,
    _keydown: function form$_keydown(e) {
        var res = true;
        this._activeControl && this._activeControl._keydown && (res = this._activeControl._keydown(e));
        return res;
    },
    _keyup: function (e) {
        var res = true;
        this._activeControl && this._activeControl._keyup && (res = this._activeControl._keyup(e));
        if (res && this._acceptButton && e.keyCode == 13) {
            this._acceptButton.performClick();
            res = false;
        } else if (res && this._cancelButton && e.keyCode == 27) {
            this._cancelButton.performClick();
            res = false;
        }
        return res;
    },
    _keypress: function (e) {
        var res = true;
        this._activeControl && this._activeControl._keypress && (res = this._activeControl._keypress(e));
        return res;
    },
    _mouseup: function (e) {
        if (e.button == 2 && this.contextMenu) {
            this.contextMenu.obj.css({ 'top': e.pageY, 'left': e.pageX });
            this.contextMenu.show();
            e.stopImmediatePropagation();
            return false;
        }
    },
    _restoreBounds: false,
    restoreBounds: function () {
        return this._restoreBounds;
    },
    resize: function () {
        ///<summary>
        ///コントロール(および子コントロール)のサイズを再調整します。
        ///</summary>
        if (this._windowState != DQ.UI.FormWindowState.Minimized) {
            this._resize(arguments);
            for (var i = 0; i < this.controls.length; i++) {
                var c = this.controls[i];
                c.resize && c.resize();
            }
        }
    },
    select: function () {
        ///<summary>
        ///Formを親コンテナのactiveControlにします。
        ///</summary>
        __dqsuper__(this, "select");
        $('.dq-form').addClass('dq-inactive');
        this.obj.removeClass('dq-inactive');

        //子コントロールを選択
        if (this.controls.length == 0) {
            return;
        }
        var c = null;
        for (var i = 0; i < this.controls.length; i++) {
            var v = this.controls[i];
            if (v._visible &&
                (v.canSelect == null || v.canSelect) &&
                (v._enabled == null || v._enabled) &&
                (c == null || c.zIndex <= v.zIndex)) {
                c = v;
            }
        }
        c && (c._focused = true);
        this._activeControl = c;
    },
    showDialog: function (callback) {
        var me = this;
        this.dialogResult = DQ.UI.DialogResult.Cancel;
        if (!this._gray) {
            this._gray = $('<div>')
                .css({ position: 'fixed', top: 0, left: 0, 'background-color': "Black" })
                .width('100%')
                .height($(window).height())
                .css('opacity', '0.1');
        }
        setTimeout(function () {
            if (me._shadow.length) {
                me._shadow[0].before(me._gray);
            } else {
                me.obj && me.obj.before(me._gray);
            }
        }, 0);
        this._showDialog_callback = callback;
        this.show();
    },
    show: function () {
        __dqsuper__(this, "show", arguments);
        for (var i = 0; i < this._shadow.length; i++) {
            this._shadow[i].show();
        }
        this._resize();
        this.select();
        var me = this;
        var p = this.findForm();
        if (this.isActiveControl()) {
            setTimeout(function () {
                !me._disposed && me.bringToFront();
                !me._disposed && me.select();
            }, 0);
        }
    },
    hide: function () {
        if (this._gray) {
            this._showDialog_callback && this._showDialog_callback(this);
            delete this._showDialog_callback;

            this._gray.remove();
            delete this._gray;
        }

        __dqsuper__(this, "hide", arguments);
        for (var i = 0; i < this._shadow.length; i++) {
            this._shadow[i].hide();
        }
    },
    _startPosition: DQ.UI.FormStartPosition.WindowsDefaultLocation,
    startPosition: function _form$startPosition(value) {
        if (!arguments.length) {
            return this._startPosition;
        } else {
            this._startPosition = value;
            var sp = DQ.UI.FormStartPosition,
                mw = this.obj.width() / 2,
                mh = this.obj.height() / 2,
                w, h;
            switch (value) {
                case sp.CenterParent:
                    w = this.parent.obj.width() / 2;
                    h = this.parent.obj.height() / 2;
                    this.location(w - mw, h - mh);
                    break;
                case sp.CenterScreen:
                    w = $(window).width() / 2;
                    h = $(window).height() / 2;
                    this.location((w - mw > 0) ? w - mw : 0, (h - mh > 0) ? h - mh : 0);
                    break;
                case sp.Manual:
                    break;
                default:
                    this.location(5, 5);
                    break;
            }
        }
    },
    size: function (value) {
        __dqsuper__(this, "size", arguments);
        if (arguments.length) {
            this._resize();
        }
    },
    text: function (data) {
        if (!arguments.length) {
            return this._title.text();
        }
        this._text = data;
        this._title.text(data);
    },
    _windowState: DQ.UI.FormWindowState.normal,
    windowState: function (state) {
        var me = this;
        if (!arguments.length) {
            return this._windowState;
        }
        if (this._windowState == state) {
            return;
        }

        this.suspendLayout();

        var style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], '')
        this._defaultWindowState = this._windowState;
        this._windowState = state;
        switch (state) {
            case DQ.UI.FormWindowState.Normal:
                this._title.css('max-width', '');
                this._max.attr('ttile', 'maximzie');
                this.obj.animate(
                    {
                        width: this._restoreBounds.width,
                        height: this._restoreBounds.height,
                        top: this._restoreBounds.y,
                        left: this._restoreBounds.x
                    },
                    function () {
                        me.width(me._restoreBounds.width);
                        me.height(me._restoreBounds.height);
                        me.client.fadeIn();
                        me._restoreBorder && me.formBorderStyle(me._restoreBorder);
                        me._resize();
                        for (var i = 0; i < me._shadow.length; i++) {
                            me._shadow[i].show();
                        }
                        DQ.UI.resizable(me, {
                            start: me._start_resize,
                            stop: me._stop_resize,
                            resize: function _form_resize() { me._resize.apply(me, arguments); }
                        });
                    }
                );
                this.obj.removeClass('dq-form-maximum');
                this.client.width('100%');
                $(document.body).css('overflow', '');
                this._minimizeBox ? this._min.show() : this._min.hide();
                DQ.UI.draggable(this, { handle: this.header });
                break;
            case DQ.UI.FormWindowState.Maximized:
                var l = style.left == 'auto' ? 0 : style.left,
                    t = style.top == 'auto' ? 0 : style.top;

                this._max.attr('tile', 'nomalize');
                DQ.UI.draggable(this, "destroy");
                this._restoreBounds = new DQ.Rectangle(l, t, this._width, this._height);
                this._restoreBorder = this.formBorderStyle();
                this._title.css('max-width', '');
                this.formBorderStyle(DQ.UI.BorderStyle.None);
                me.header.show();
                for (var i = 0; i < me._shadow.length; i++) {
                    me._shadow[i].hide();
                }
                this.width('100%');
                this.height('100%');
                this.css({ 'top': 0, 'left': 0 });
                this._size._height = this._height = this.obj.height();
                this._size._width = this._width = this.obj.width();
                DQ.UI.resizable(this, "destroy");
                this.obj.addClass('dq-form-maximum');

                $(document.body).css('overflow', 'hidden');
                this._minimizeBox ? this._min.show() : this._min.hide();
                DQ.UI.draggable(this, "destroy");
                break;
            case DQ.UI.FormWindowState.Minimized:
                this._restoreBounds = new DQ.Rectangle(this.obj.css('left'), this.obj.css('top'), this._width, this._height);
                this._restoreBorder = this.formBorderStyle();
                this._title.css('max-width', '4em');
                this.obj.animate({ width: '10em', height: this.header.height() }, function () {
                    me.client.fadeOut();
                    DQ.UI.resizable(me, "destroy");
                    me._resize();
                    for (var i = 0; i < me._shadow.length; i++) {
                        me._shadow[i].show();
                    }
                });
                $(document.body).css('overflow', '');
                this.obj.removeClass('dq-form-maximum');
                this._min.hide();
                DQ.UI.draggable(this, { handle: this.header });
                break;
        }
        this.resumeLayout();
        this.resize();
    }
});

DQ.UI.ToolStripItemCollection = dqextend(Array, function (parent, options) {
    ///<summary>
    ///MenuItemなどのコントロールを格納するコレクションを作成します。
    ///</summary>
    ///<param name="parent" type="DQ.Control">
    ///親コントロールを指定
    ///</parent>

    this.base(options);
    this.parent = parent;
    this.obj = $('<div>');
    this.parent.append(this.obj, true);
    if (parent instanceof DQ.UI.MenuStrip || parent instanceof DQ.UI.Toolbar) {
        this.rootControl = parent;
    }
}, {
    obj: null,
    push: function (value) {
        this.__super.push.apply(this, arguments);
        this.parent.add(value);
    },
    add: function (value) { this.push(value); },
    addRange: function (value) {
    },
    bind: function (url) {
        this._clear();
        var b = this.parent._text;
        this.parent.text("データを読み込みます");
        this._clear();
        var me = this;
        setTimeout(function () {
            $.get(url, "", function (data) {
                me.parent.text(b);
                eval("var s = " + data);
                me.json(s);
            });
        }, 1000);
    },
    json: function (data) {
        for (var nm in data) {
            var node = new DQ.UI.TreeNode(this.treeView, nm);
            this.push(node);
            node.json(data[nm]);
        }
    },
    _clear: function () {
        for (var i = 0; i < this.length; i++) {
            this[i].dispose();
        }
        this.length = 0;
    },
    dispose: function () {
        this._clear();
        this.obj.remove();
    }
});

DQ.UI.Toolbar = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    ///     ToolButtonのコレクションを保持するコンテナを定義
    /// </summary>
    ///	<param name="parent" type="DQ.Control">
    ///		親コントロールを指定
    ///	</param>
    ///	<param name="options" type="object">
    ///		ツールボタンへの設定値を指定
    ///	</param>
    ///	<param name="object" type="string/object">
    ///		コントロールの外観を示すオブジェクトを指定
    ///	</param>

    object = object || $('<div>').addClass('dq-toolbar');
    options = DQ.options(options, {
        image: null,
        items: null
    });
    this.base(parent, object);
    this._create(options);
}, {
    _create: function (options) {
        this.items = [];
        var dl = $('>dl', this.obj);
        this.client = dl.length ? dl : $('<dl>').appendTo(this.obj);
        $('>div', this.obj).appendTo(this.client);

        var menus = {};
        options.items = options.items || this.parseHtml(this.client, menus);
        this.parseObject(options);
        //this.addRange(options.items);
        options.image && this.imageList(options.image);

        this._dock = DQ.UI.DockStyle.Top;
    },
    dqid: "DQ.UI.Toolbar",
    addRange: function (items) {
        for (var nm in items) {
            var item;
            if (items[nm] instanceof DQ.UI.ToolButton) {
                item = items[nm];
            } else {
                items[nm].name = items[nm].name || nm;
                item = new DQ.UI.ToolButton(this, items[nm], items[nm].object);
            }
            this.parent[item.name] && (this.parent[item.name] = item);
            this.items.push(item);
        }
    },
    add: function (item) {
        if (item instanceof DQ.UI.ToolButton || item instanceof DQ.UI.MenuItem) {
        } else {
            item = new DQ.UI.ToolButton(this, item, item.object);
        }
        this.parent[item.name] && (this.parent[item.name] = item);
        this.items.push(item);
    },
    imageList: function (value) {
        if (!arguments.length) {
            return this._imageList;
        }
        if (typeof value == "string") {
            this._imageList = new DQ.UI.ImageList(value);
        } else {
            this._imageList = value;
        }
        $('.image', this.client).css('background-image', 'url(' + this._imageList.sprite + ')');
        $('.dq-toolbar-btn', this.client).css('background-image', 'url(' + this._imageList.sprite + ')');
    },
    parseHtml: function (object, menus) {
        var ch = $(object).children();
        menus = menus || {};
        for (var i = 0; i < ch.length; i++) {
            var value = ch[i];
            if (value.nodeName == "DT" || value.nodeName == "DIV") {
                var id = value.id || "toolButton_" + DQ.UI.Toolbar.count++;
                var item = menus[id] = {
                    name: id,
                    toolTip: value.title,
                    object: $(value).removeClass('dq-toolbar-button'),
                    cssClass: value.getAttribute('class')
                };
                $(value).removeClass(item.cssClass);
                item.text = item.object.text();
                this.parent[id] = {};
            }
            else if (value.nodeName == "DD") {
                this.parseHtml(value, item);
                $(value).empty().show();
            } else if (value.nodeName == "UL") {
                menus.dropDownItems = {};
                this.parseHtml(value, menus.dropDownItems);
            } else if (value.nodeName == "LI") {
                this.parent[value.id] = {};
                item = menus[value.id] = {};
                this.parseHtml(value, item);
                $('ul', value).remove();
                item.text = $(value).html();
            }
        }

        return menus;
    },
    parseObject: function (options) {
        var me = this;
        this.parent.obj.bind("click", function () {
            me.cancelTo && me.cancelTo.hide();
            me._dropDownMenu = false;
            return true;
        });
        $.each(options.items, function (nm, val) {
            val.name = nm;
            if (!val.dropDownItems) {
                me.add(val);
            } else {
                var pos = me.items.length, o, dd;
                if (!val.object) {
                    o = $('<dt>').attr({ tabIndex: pos, index: pos }).addClass('dq-toolbar-button').appendTo(me.client);
                    dd = $('<dd>').appendTo(me.client);
                } else {
                    o = val.object.attr({ tabIndex: pos, index: pos }).addClass('dq-toolbar-button');
                    dd = o.next();
                    o.appendTo(me.client);
                    dd.appendTo(me.client);
                }
                o.text('');
                var div = $('<div>').appendTo(dd),
                mn = new DQ.UI.MenuItem(me, val, div),
                oi = $('<div>').addClass('dq-toolbar-inner').appendTo($('<div>').addClass('dq-toolbar-outer').appendTo(o)),
                bg = $('<div>').addClass('dq-toolbar-btn').appendTo(oi),
                ott = $('<div>').addClass('dq-toolbar-text').appendTo(oi),
                otd = $('<div>').addClass('dq-toolbar-down').appendTo(oi),
                img = options.image || me.imageList() || DQ.TOOL_IMAGE;

                bg.css('background-image', 'url(' + img + ')');
                val.cssClass && bg.addClass(val.cssClass);

                me.add(mn);
                ott.html(mn.text());
                otd.html("&nbsp;");
                var accessKey = $('u', o).text();
                accessKey && o.attr('accesskey', accessKey);
                o.attr('title', mn.toolTip);
                o.bind("click", me, function (e) {
                    var p = parseInt($(this).attr('index'));
                    var mn = me.items[p];
                    if (me._dropDownMenu && me.cancelTo == mn) {
                        mn.hide();
                        me.cancelTo = null;
                        me._dropDownMenu = false;
                    } else {
                        me._dropDownMenu = true;
                        me.cancelTo = mn;
                        mn.obj.css({ top: $(this).position().top + this.offsetHeight, left: this.offsetLeft });
                        me.items[p].show();
                    }
                    e.stopImmediatePropagation();
                });
                o.bind("focusin", me, function (e) {
                    if (me._dropDownMenu) {
                        me.cancelTo && me.cancelTo.hide();
                        var p = parseInt($(this).attr('index'));
                        me.items[p].show();
                        me.cancelTo = me.items[p];
                        mn.obj.css({ top: $(this).position().top + this.offsetHeight, left: this.offsetLeft });
                    }
                });
                o.hover(
                    function (s) {
                        $(this).addClass('dq-toolbar-hover');
                        me._dropDownMenu && o[0].focus();
                    },
                    function (s) {
                        $(this).removeClass('dq-toolbar-hover');
                    }
                );
                mn.hide();
            }
        });
    }
});

DQ.UI.Toolbar.count = 1;

DQ.UI.ToolButton = dqextend(DQ.Control, function (parent, options, object) {
    ///	<summary>
    ///	    ツールバー上のボタンを作成します。
    ///	</summary>
    ///	<returns type="function" />
    ///	<param name="parent" type="DQ.Control">
    ///		親コントロール(DQ.Toolbar)を指定
    ///	</param>
    ///	<param name="options" type="object">
    ///		ツールボタンへの設定値を指定
    ///	</param>
    ///	<param name="object" type="object">
    ///		ツールボタンへの設定値を指定
    ///	</param>

    object = object || $('<div>');
    options = options || {};
    if (typeof object == "object") {
        var hint = options.toolTip || object.text();
        if (object[0].nodeName == "DIV") {
            object = $('<dt>').append(object);
        }
        object.empty();
        object.attr({ 'tabIndex': '100', 'title': hint }).addClass('dq-toolbar-button');
    } else {
        var hint = options.toolTip || options.text;
        object = $('<dt/>').attr({ 'tabIndex': '100', 'title': hint }).addClass('dq-toolbar-button');
    }
    this.base(parent, object);
    this._create(parent, options, object);
}, {
    checked: false,
    _create: function (parent, options, object) {
        ///	<summary>
        ///	    ツールバー上のボタンを作成します。
        ///	</summary>
        ///	<returns type="function" />
        ///	<param name="parent" type="DQ.Control">
        ///		親コントロール(DQ.Toolbar)を指定
        ///	</param>
        ///	<param name="options" type="string">
        ///		Tooltipへの表示名を指定
        ///	</param>
        ///	<param name="object" type="object">
        ///		ツールボタンへの設定値を指定
        ///	</param>

        this.name = options.name || "";

        var ot = $('<div>').addClass('dq-toolbar-outer').appendTo(object),
            inn = $('<div>').addClass('dq-toolbar-inner').appendTo(ot),
            bg = $('<div>').addClass('dq-toolbar-btn').appendTo(inn),
            bt = $('<div>').addClass('dq-toolbar-text').appendTo(inn),
            tx = this._text = options.text || "",
            st = this.showText = (typeof options.showText != "undefined") ? options.showText : true,
            ch = this.checkBox = (typeof options.checkBox != "undefined") ? options.checkBox : false,
            img = this.image = options.image || this.parent.imageList || DQ.TOOL_IMAGE,
            me = this;

        bt.text(tx);
        !st && bt.addClass('dq-none');
        bg.css('background-image', 'url(' + img + ')');
        options.cssClass && bg.addClass(options.cssClass);
        this.onClick = options.click;
        inn.bind("click", this, function (e) {
            if (!me._enabled) {
                return;
            }
            if (e.data.checkBox) {
                e.data.toggle.apply(e.data);
            }
            e.data.onClick && e.data.onClick.apply(e.data, arguments);
        });
        object.mousedown(function () {
            if (!me._enabled) {
                return;
            }
            object
                .addClass('dq-toolbar-check')
                .one("mouseup", function () { object.removeClass('dq-toolbar-check'); });
        });
        object.hover(
            function () {
                if (!me._enabled) {
                    return;
                }
                object.addClass('dq-toolbar-hover');
            },
            function () {
                if (!me._enabled) {
                    return;
                }
                object.removeClass('dq-toolbar-hover');
            }
        );

        return this;
    },
    dqid: "DQ.UI.ToolButton",
    toggle: function () {
        if (this.checked) {
            this.checked = false;
            this.obj.removeClass('dq-toolbar-check');
        } else {
            this.checked = true;
            this.obj.addClass('dq-toolbar-check');
        }
    },
    check: function (flag) {
        if (this.checked) {
            this.obj.removeClass('dq-toolbar-check');
        }
        this.checked = flag;
        if (this.checked) {
            this.obj.addClass('dq-toolbar-check');
        } else {
            this.obj.removeClass('dq-toolbar-check');
        }
    }
});

DQ.UI.MenuStrip = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    ///     ToolButtonのコレクションを保持するコンテナを定義
    /// </summary>
    ///	<returns type="function" />
    ///	<param name="parent" type="DQ.Control">
    ///		親コントロールを指定
    ///	</param>
    ///	<param name="options" type="object">
    ///		ツールボタンへの設定値を指定
    ///	</param>

    object = object || $('<div>').addClass("dq-menustrip");
    this.base(parent, object);

    options = DQ.options(options, {
        zIndex: 10
    });
    options.menuItems = options.menuItems || this.parseHtml(this.client, options.menuItems) || {};
    this.client.text('');
    this._create(parent, options, this.obj);
}, {
    _dock: DQ.UI.DockStyle.Top,
    _dropDownMenu: false,
    dqid: "DQ.UI.MenuStrip",
    _create: function (parent, options, object) {
        ///<summary>
        /// Menustripコントロールを作成します。
        ///</summary>

        this.items = [];
        this.showSubmenuTab = true;
        this.autoShowSubmenu = true;
        this.zIndex = options.zIndex;

        var js = options.menuItems,
            me = this;

        if (typeof js == "string") {
            //文字列ならJSONと解釈
            eval('js = ' + object);
            this.parseObject(js);
        } else if (typeof js == "object") {
            this.parseObject(options.menuItems);
        } else if (!object && js) {
            $.each(options, function (nm, val) {
                $('<div>').addClass("name").html(nm).appendTo(this.obj);
            });
        } else {
            this.parseHtml(this.client, options);
            this.parseObject(options);
        }

        $('.name', this.obj).hover(
            function (s) {
                $(this).addClass('hover');
            },
            function (s) {
                $(this).removeClass('hover');
            }
        );
    },
    imageList: function (value) {
        if (!arguments.length) {
            return this._imageList;
        }
        if (typeof value == "string") {
            this._imageList = new DQ.UI.ImageList(value);
        } else {
            this._imageList = value;
        }
        $('.image', this.client).css('background-image', 'url(' + this._imageList.sprite + ')');
    },
    parseObject: function (options) {
        var me = this;
        this.client = $('<dl>').appendTo(this.obj);
        this.parent.obj.bind("click", function () {
            me.cancelTo && me.cancelTo.hide();
            me._dropDownMenu = false;
            return true;
        });
        $.each(options, function (nm, val) {
            val.name = nm;

            var pos = me.items.length,
                o = $('<dt>').attr({ tabIndex: pos, index: pos }).addClass("name").appendTo(me.client),
                dd = $('<dd>').appendTo(me.client),
                div = $('<div>').appendTo(dd),
                mn = new DQ.UI.MenuItem(me, val, div);

            me.items.push(mn);
            o.html(mn.text());
            var accessKey = $('u', o).text();
            accessKey && o.attr('accesskey', accessKey);
            o.attr('title', mn.toolTip);
            o.bind("click", me, function (e) {
                var p = parseInt($(this).attr('index'));
                var mn = me.items[p];
                if (me._dropDownMenu && me.cancelTo == mn) {
                    mn.hide();
                    me.cancelTo = null;
                    me._dropDownMenu = false;
                } else {
                    me._dropDownMenu = true;
                    me.cancelTo = mn;
                    mn.obj.css({ 'top': $(this).position().top + this.offsetHeight, 'left': this.offsetLeft });
                    me.items[p].show();
                }
                e.stopImmediatePropagation();
            });
            o.bind("focusin", me, function (e) {
                if (me._dropDownMenu) {
                    me.cancelTo && me.cancelTo.hide();
                    var p = parseInt($(this).attr('index'));
                    me.items[p].show();
                    me.cancelTo = me.items[p];
                    mn.obj.css({ 'top': $(this).position().top + this.offsetHeight, 'left': this.offsetLeft });
                }
            });
            o.hover(
                    function (s) {
                        $(this).addClass('hover');
                        me._dropDownMenu && o[0].focus();
                    },
                    function (s) {
                        $(this).removeClass('hover');
                    }
                );
            mn.hide();
        });
    },
    parseHtml: function (elem, menus) {
        ///<summary>
        /// HTML(DOM)を解析してoptions.dropDownMenuItems相当を作成します。
        ///</summary>
        var me = this;
        var item;
        menus = menus || {};
        var ch = $(elem).children();
        for (var i = 0; i < ch.length; i++) {
            var value = ch[i];
            value.id = value.id || "menuItem_" + DQ.UI.MenuItem.count++;
            if (value.nodeName == "DL") {
                me.parseHtml(value, menus);
                continue;
            } else if (value.nodeName == "DT") {
                item = menus[value.id] = { 'name': value.id, 'toolTip': value.title, 'text': $(value).html() };
            } else if (value.nodeName == "DD") {
                me.parseHtml(value, item);
                continue;
            } else if (value.nodeName == "UL") {
                menus.dropDownItems = {};
                me.parseHtml(value, menus.dropDownItems);
                continue;
            } else if (value.nodeName == "LI") {
                me.parent[value.id] = {};
                item = menus[value.id] = {};
                me.parseHtml(value, item);
                $('ul', value).remove();
                item.text = $(value).html();
            }
        }

        return menus;
    },
    show: function () {
        !this._visible && this.obj.slideDown();
        this._visible = true;
    },
    hide: function (ignore) {
        if (!ignore) {
            this._visible = false;
            this.obj.slideUp();
            return;
        }
        $(this.items).each(function (nm, menuItem) {
            if (ignore != menuItem) {
                menuItem.menuItems.length && menuItem.menuItems[0].hide && menuItem.menuItems[0].hide();
            }
        });
    },
    toggle: function () {
        this.obj.css({
            top: this.top + this.screen.view.top,
            left: this.left + this.screen.view.left
        }).toggle();
    },
    displayWidth: function () {
        return this.obj.width();
    },
    displayHeight: function () {
        return this.obj.height();
    },
    perfomeClick: function (fn) {
    },
    select: function (value) {
        if (!arguments.length || value < 0 || this.items.length <= value) {
            return;
        }
        $('> li > div', this.obj).removeClass('dq-menu-hover');
        this.items[pos].select();
        this._selectedIndex = value;
        this.selected = this.items[pos];
    }
});

DQ.UI.MenuItem = dqextend(DQ.Control, function (parent, options, object) {
    ///<summary>
    /// MenuItemのインスタンスを作成します。
    ///</summary>
    options = options || {};
    object = object || $('<div>');
    id = options.dropDownItems && !(parent instanceof DQ.UI.MenuItem) ? options.name : ""; //何がしたいのか自分でもわからん・・・
    id && object.attr('id', id + "MenuItem");
    this.base(parent, id ? id + "MenuItem" : object);

    this._create(parent, options, object, id);
},
{
    dqid: "DQ.UI.MenuItem",
    toolTip: "",
    cancelHide: false,
    cancelShow: false,
    selected: false,
    _context$outerClick: function (e) {
        var me = e.data;
        me.hide();
        return true;
    },
    _create: function _menuItem$_create(parent, options, object, id) {

        var fm = this.findForm();
        options.name && fm && fm[options.name] && (fm[options.name] = this);

        this.dropDownItems = [];
        this.name = options.name || "";
        this.toolTip = options.toolTip || "";
        this.shortKey = options.shortKey || "";
        this.enabled((options.enabled != null) ? options.enabled : true);

        //dropDwonItems用の外側MenuItemの作成
        if (options.dropDownItems) {
            if (parent instanceof DQ.UI.MenuItem) { //dropDownMenuItem
                this._root = parent._root;
                var to = this._subMenu = $('<div>').addClass('dq-menuitem').appendTo(this._root).hide();
                this.client = $('<ul />').appendTo(to);
            } else { //ルートメニュー
                this._root = this.obj.parent();
                this.obj.addClass('dq-menuitem');
                this.client = $('<ul />').appendTo(this.obj);
                if (!(parent instanceof DQ.UI.MenuStrip) && !(parent instanceof DQ.UI.Toolbar)) {
                    var me = this;
                    this.parent.obj.bind("click", me, function _context$outerClick() {
                        me.hide();
                        return true;
                    });
                }
            }
        }

        if (parent instanceof DQ.UI.MenuItem) {
            var pos = parent.dropDownItems.length,
                div = this.anchor = $('<div>').attr('tabIndex', pos).appendTo(object),
                rbr = options.dropDownItems ? "&raquo;" : "&nbsp;";

            this._tabIndex = pos;
            if ($.trim(options.text) == "-") { // as a separater.
                this.isSeparater = true;
                div.addClass('separater');
                object.addClass('li-sepa');
                this._imgObj = $('<span class="sepa-back"></span>').appendTo(div);
                this._textObj = $('<span class="sepa">&nbsp;</span>').appendTo(div);
            } else {
                this.isSeparater = false;
                div.addClass('item');
                this._imgObj = $('<span>').addClass("back").html('<span class="image">&nbsp;</span>').appendTo(div);
                this.textObj = $('<span>').addClass("text").text(this._text).appendTo(div);
                this.shortObj = $('<span>').addClass("short").text(this.shortKey).appendTo(div);
                this.rbrObj = $('<span>').addClass("mask").html(rbr).appendTo(div);
            }
            var me = this;
            !this.isSeparater && this.obj.hover(
                function () {
                    me.select.apply(me);
                },
                function () {
                    me.anchor.removeClass('hover');
                }
            );
            this.imageIndex(options.imageIndex == null ? -1 : options.imageIndex);
        }

        this.text(options.text || this.name);

        if (options.dropDownItems) {
            this.addRange(options.dropDownItems);
            //this._resize(this.client); // for ie7
        }

        this.onClick = typeof options.click != "undefined" ? options.click : null;

        function _getTopLevelMenuItem(me) {
            var p = me;
            while (p.parent instanceof DQ.UI.MenuItem) {
                p = p.parent;
            }
            return p;
        }
        function _click(e) {
            var me = e.data;
            if (!me._enabled) {
                return true;
            }
            //e.stopImmediatePropagation();
            me.onClick && me.onClick.apply(me, arguments);
            _getTopLevelMenuItem(me).hide();
            //return false;
        }

        !this.isSeparater && this.obj.bind("click", this, _click);

        return this;
    },
    _resize: function (client) {
        var max = 0, smax = 0;
        $('.text', client).each(function () {
            max = Math.max($(this).outerWidth(), max);
        });
        $('.short', client).each(function () {
            smax = Math.max($(this).outerWidth(), smax);
        });
        var margin = this.dropDownItems.length && this.dropDownItems[0]._imgObj.outerWidth() || 0;
        !isNaN(max) && client.width(max + smax + margin + 16);
        !isNaN(max) && $('.sepa', client).width(max + smax + 15);
    },
    hide: function () {
        __dqsuper__(this, "hide", arguments);
        var d = this.dropDownItems;
        this.selected = false;
        for (var i = 0; i < d.length; i++) {
            d[i].selected = false;
            d[i]._subMenu && d[i]._subMenu.hide();
        }
    },
    append: function (element) {
        if (element.dqid) {
            this.client.append(element.obj);
            this.controls.push(element);
        } else {
            this.client.append(element);
        }
        this._addControls(this, this.controls.length - 1);
    },
    add: function (val) {
        var item;
        if (val instanceof DQ.UI.MenuItem) {
            item = val;
        } else {
            var object = val.object || $('<li />'),
                pos = this.dropDownItems.length;
            item = new DQ.UI.MenuItem(this, val, object);
        }
        this.dropDownItems.push(item);
    },
    addRange: function (menus) {
        var me = this;
        for (var nm in menus) {
            var val = menus[nm];
            val.name = nm;
            this.add(val);
        }
    },
    click: function (fn) {
        var me = this;
        if (!fn) {
            this.obj.click();
        }
    },
    dispose: function () {
        this.obj.unbind("click");
        if (this instanceof DQ.UI.ContextMenuStrip) {
            this.parent.obj.unbind('click', this._context$outerClick);
        }
    },
    _imageIndex: 0,
    imageHeight: 18,
    imageIndex: function (value) {
        ///<summary>
        /// ImageListのimageの位置を指定します。
        ///</summary>
        if (!arguments.length) {
            return this._imageIndex;
        }
        this._imageIndex = value;
        if (value == -1) {
            $('.image', this._imgObj).hide();
        } else {
            $('.image', this._imgObj).show().css('background-position', (value * -this.imageHeight) + "px 0px");
        }
    },
    text: function (data) {
        if (!arguments.length) {
            return this._text;
        }
        this._text = data;
        this.textObj && this.textObj.html(data);
    },
    hideDropDown: function () {
        this.cancelShow = false;
        this.selected = false;
        this._subMenu && this._subMenu.hide();
    },
    hasDropDownItems: function () {
        return me._subMenu != null;
    },
    onClick: null,
    onOpend: null,
    onOpening: null,
    show: function () {
        this.onOpening && this.onOpening(this);
        __dqsuper__(this, "show");
        this._resize(this.client);
        this.onOpend && this.onOpend(this);
    },
    showDropDown: function () {
        var d = this.parent.dropDownItems;
        this.parent.opendDropDown && this.parent.opendDropDown.hideDropDown();
        this.selected = true;
        this.parent.cancelTo = this;
        this.cancelShow = true;
        if (this._subMenu) {
            this._subMenu.show();
            this._resize($('ul', this._subMenu));
        }
        this.parent.opendDropDown = this;
    },
    select: function () {
        if (!this._enabled) {
            return;
        }
        __dqsuper__(this, "select");

        var pt = this.parent;
        var me = this;
        $('li div', me.parent.obj).removeClass('hover');

        this.parent.cancelTo && (this.parent.cancelTo.cancelShow = true);
        this.parent.cancelTo = me;
        me.anchor.addClass('hover');
        me.selected = true;
        me.cancelShow = false;
        if (me._subMenu) {
            var t = me.obj[0].offsetTop + me.parent.obj[0].offsetTop;
            var l = me.obj[0].offsetLeft + +me.obj[0].offsetWidth + me.parent.obj[0].offsetLeft;
            me._subMenu.css({ 'top': t, 'left': l });
        }
        setTimeout(function () {
            if (!me.cancelShow) {
                me.showDropDown();
            }
        }, 500);
    }
});

DQ.UI.MenuItem.count = 1;
DQ.UI.ContextMenuStrip = DQ.UI.MenuItem;

DQ.ShortcutKey = function (ctl, shift, alt, key) {
    if (typeof ctl == object) {
        this.ctl = ctl.ctl || false;
        this.shift = ctl.shift || false;
        this.alt = ctl.alt || false;
        this.key = 1;
    } else {
        this.ctl = ctl;
        this.shift = shift;
        this.alt = alt;
        this.key = key;
    }
}

DQ.ShortcutKey.prototype.toString = function () {
    var c = this.ctl ? "Ctrl+" : "";
    var s = this.shift ? "Shift+" : "";
    var a = this.alt ? "Alt+" : "";
    return [c, s, a, string.fromCharCode(this.key)].join('');
}

DQ.UI.StatusStrip = dqextend(DQ.Control, function (parent, options, tg) {
    if (!tg) {
        tg = $('<div>').addClass('dq-statusstrip');
    }
    this.base(parent, tg);

    this.items = new Array();

    options = options || {};
    this._create(options);

    this._dock = DQ.UI.DockStyle.Top;
}, {
    dqid: "DQ.UI.StatusStrip",
    _create: function (options) {
        var me = this,
            s;
        if (this.obj.children().length) {
            this.client = $('<div>').append(this.obj.children()).appendTo(this.obj);
        }
        else {
            var txt = this.obj.text();
            this.obj.text("");
            this.client = $('<div>').append(txt).appendTo(this.obj);
        }
        if (this.parent instanceof DQ.UI.Form) {
            this.parent._sth = this.obj.height();
        }
        $('>div', this.client).each(function () {
            var cls = $(this).attr('class'),
                id = $(this).attr('id');
            switch (cls) {
                case 'dq-statuslabel':
                    s = new DQ.UI.ToolStripStatusLabel(me, { name: id }, id);
                    me.append(s, true);
                    break;
                default:
                    //s = new DQ.Control(me, {}, id);
                    //this.append(s);
            }
        });
        //this.resize();
    },
    resize: function () {
        var style = this.parent.obj[0].currentStyle || document.defaultView.getComputedStyle(this.parent.obj[0], ''),
        bh = parseFloat(style.paddingBottom) + parseFloat(style.borderBottomWidth);
        bw = parseFloat(style.paddingLeft);

        this.obj.width(this.obj.parent().outerWidth({ margin: true }));
        //this.obj.css('left', bw);
        //this.obj.css('bottom', bh);
    },
    update: function () {
        //this.resize();
    },
    items: []
});

DQ.UI.ToolStripStatusLabel = dqextend(DQ.Control, function (parent, options, tg) {
    if (!tg) {
        tg = $('<div>').addClass("dq-statuslabel");
    }
    this.base(parent, tg);
}, {
    dqid: "DQ.ToolStripStatusLabel"
});

DQ.UI.ImageList = function (url) {
    if (url instanceof Array) {
        this.urls = url.clone();
    } else {
        this.sprite = url;
    }
}
DQ.UI.ImageList.prototype = {
    width: 16,
    height: 16,
    sprite: null,
    padding: 16,
    urls: []
}

DQ.UI.Splitter = dqextend(DQ.Control, function (parent, options, object) {

    object = object || $('<div>').addClass('dq-splitter');

    options = DQ.options(options, {
        dock: DQ.UI.DockStyle.Left,
        width: '8px',
        height: '8px',
        showGrip: false
    });

    this.base(parent, object);
    this.obj.addClass('dq-splitter');

    this._create(options, object);
}, {
    _create: function (options, object) {
        var DS = DQ.UI.DockStyle,
            me = this,
            pr = this.obj.prev(),
            sg = this.showGrip = options.showGrip,
            grip = sg ? $('<div>').addClass("dq-ui-vgrip").appendTo(this.obj) : this.obj;

        this._prev = this.parent.controls[this.parent.controls.length - 2];
        this.dock(options.dock);
        if (!object) {
            if (this._dock == DS.Left || this._dock == DS.Right) {
                this.width(options.width);
            } else {
                this.height(options.height);
            }
        }

        function _splitter$dragEnd(e) {
            var pr = me.obj.prev();
            if (me._dock == DS.Left || me._dock == DS.Right) {
                me._prev.width(pr.width() + (e.pageX - me.drag.start.x));
            } else {
                me._prev.height(pr.height() + (e.pageY - me.drag.start.y));
            }
            me._prev.update();

            $(e.data)
            .removeClass('dq-drag')
            .css({ left: 0, top: 0 });
            DQ.document
            .unbind("mousemove", _splitter$move);
            delete me.drag;

            var p = me.parent, after = false;
            for (var i = 0; i < p.controls.length; i++) {
                var v = p.controls[i];
                if (v == me) {
                    after = true;
                    continue;
                }
                if (!after) {
                    continue;
                }
                v.update && v.update();
            }
            me.onMouseUp && me.onMouseUp.apply(me, [e, lrx, lry]);
            return false;
        }
        function _splitter$move(e) {
            if (me._dock == DS.Left || me._dock == DS.Right) {
                $(e.data).css('left', me.drag.left + (e.pageX - me.drag.start.x));
            } else {
                $(e.data).css('top', me.drag.top + (e.pageY - me.drag.start.y));
            }
            return false;
        }

        this._grip = $('<div>').addClass("dq-grip")
        .appendTo(me.client)
        .css({ left: 0, top: 0, 'z-index': "999" })
        .height(me.height())
        .width(me.width())
        .mousedown(function _dragStart(e) {
            me.dragging = true;
            me.drag = {
                start: { x: e.pageX, y: e.pageY },
                left: parseInt($(this).css('left')), top: parseInt($(this).css('top'))
            };
            $(this).addClass('dq-drag');
            $(document).bind("mousemove", this, _splitter$move);
            $(document).one("mouseup", this, _splitter$dragEnd);
            me.onMouseDown && me.onMouseDown.apply(me, [e, me.drag.start]);
            return false;
        });
    },
    dqid: "DQ.UI.Splitter",
    _dock: DQ.UI.DockStyle.Left,
    dock: function (value) {
        if (!arguments.length) {
            return this._dock;
        }
        this._dock = value;

        var DS = DQ.UI.DockStyle;
        switch (value) {
            case DS.Left:
                this.obj.css({ 'position': 'static', 'float': 'left', 'left': '0px' });
                break;
            case DS.Right:
                this.obj.css({ 'position': 'static', 'float': 'right' });
                break;
            case DS.Top:
                this.obj.css({ 'position': 'static', 'float': 'left' });
                break;
            case DS.Bottom:
                this.obj.css({ 'position': 'absolute', 'float': 'left', 'bottom': '0px' });
                break;
            case DS.Fill:
                this.obj.css({ 'position': 'static', 'float': 'left' });
                break;
            case DS.None:
                this.obj.css({ 'position': 'absolute', 'float': 'none' });
                break;
        }

        var p = this.parent,
            style = this.obj[0].currentStyle || document.defaultView.getComputedStyle(this.obj[0], ''),
            mw = parseFloat(style.marginLeft) + parseFloat(style.marginRight),
            mh = parseFloat(style.marginLop) + parseFloat(style.marginBottom),
            h = (p.clientSize ? p.clientSize().height : p.height())
                - (isNaN(mh) ? 0 : mh)
                - parseFloat(style.borderTopWidth)
                - parseFloat(style.borderBottomWidth),
            w = (p.clientSize ? p.clientSize().width : p.width())
                - (isNaN(mw) ? 0 : mw)
                - parseFloat(style.borderLeftWidth)
                - parseFloat(style.borderRightWidth),
                l = 0;

        for (var i = 0; i < p.controls.length; i++) {
            var v = p.controls[i];
            if (v == this) {
                break;
            }
            if (v._dock) {
                if (v._dock == DS.Top || v._dock == DS.Bottom) {
                    h -= v.obj.outerHeight({ margin: true });
                }
                if (v._dock == DS.Left || v._dock == DS.Right) {
                    w -= v.obj.outerWidth({ margin: true });
                }
                if (v._dock == DS.Left) {
                    l += v.obj.outerWidth({ margin: true });
                }
            }
        }
        switch (value) {
            case DS.Left:
                this.obj.removeClass('dq-grip-row').addClass('dq-grip-col');
                this.height(h);
                break;
            case DS.Right:
                this.obj.removeClass('dq-grip-row').addClass('dq-grip-col');
                this.height(h);
                break;
            case DS.Top:
                this.obj.removeClass('dq-grip-col').addClass('dq-grip-row');
                this.obj.css({ width: w, height: 6 });
                this._width = w;
                break;
            case DS.Bottom:
                this.obj.removeClass('dq-grip-col').addClass('dq-grip-row');
                this.obj.css({ 'width': w, 'height': this._height });
                break;
            case DS.Fill:
                this.obj.css({ 'width': w, 'height': h });
                break;
            case DS.None:
                this.obj.css({ 'width': this._width, 'height': this._height });
                break;
        }

        this._grip && this._grip.width(this.obj.width()).height(this.obj.height());
    },
    update: function () {
        this.dock(this._dock);
        for (var i = 0; i < this.controls.length; i++) {
            this.controls[i].update && this.controls[i].update();
        }
    },
    showGrip: false
});

DQ.UI.resizable = function (control, options) {
    ///<summary>
    /// コントロールにリサイズ機能を付加します。
    ///</summary>

    //削除
    if (options == "destroy") {
        if (control._handle == null) {
            return;
        }
        for (var nm in control._handle) {
            if (control._handle.hasOwnProperty(nm)) {
                control._handle[nm].remove();
            }
        }

        delete control._handle;
        control._handle = null;
        return;
    }
    //作成
    var h = options.handles || ['w', 'e', 's', 'se'],
        me = control;

    if (!control._handle) {
        control._handle = [];
    }
    for (var i = 0; i < h.length; i++) {
        if (control._handle[h[i]]) {
            continue;
        }
        var handle = "dq-resize-" + $.trim(h[i]);
        control._handle[h[i]] =
            $('<div>')
            .addClass("dq-resize-handle")
            .addClass(handle)
            .attr('unselectable', 'on')
			.css('MozUserSelect', 'none')
            .appendTo(control.obj)
            .bind("mousedown", this, _resize$dragStart);
    }

    function _resize$dragStart(e) {
        var axis = this.className.match(/dq-resize-(w|se|e|s)/);

        me._drag = {
            axis: axis && axis[1] || 'se',
            start: { x: e.pageX, y: e.pageY },
            left: parseInt($(this).css('left')),
            right: parseInt($(this).css('right')),
            top: parseInt($(this).css('top')),
            bottom: parseInt($(this).css('bottom'))
        };
        DQ.document.bind("mousemove", this, _resize$move);
        DQ.document.one("mouseup", this, _resize$dragEnd);
        $(this).addClass('dq-drag');
        options.start && options.start.apply(me, [e, me._drag.start]);
        return false;
    }

    function _resize$move(e) {
        if (me._drag.axis == 'w' || me._drag.axis == 'sw') {
            $(e.data).css('left', me._drag.left - (me._drag.start.x - e.pageX));
        }
        if (me._drag.axis == 'e' || me._drag.axis == 'se') {
            $(e.data).css('right', me._drag.right - (e.pageX - me._drag.start.x));
        }
        if (me._drag.axis == 's' || me._drag.axis == 'se') {
            $(e.data).css('bottom', me._drag.bottom - (e.pageY - me._drag.start.y));
        }
        options.resize && options.resize.apply(me, [e, e.pageX - me._drag.start.x, e.pageY - me._drag.start.y]);
        return false;
    }

    function _resize$dragEnd(e) {
        if (me._drag.axis == 'w' || me._drag.axis == 'sw') {
            var left = parseInt(me.obj.css('left'));
            left -= (me._drag.start.x - e.pageX);
            me.obj.width(me.obj.width() - (e.pageX - me._drag.start.x)).css('left', left);
        }
        if (me._drag.axis == 'e' || me._drag.axis == 'se') {
            me.obj.width(me.obj.width() + (e.pageX - me._drag.start.x));
        }
        if (me._drag.axis == 's' || me._drag.axis == 'se') {
            me.obj.height(me.obj.height() + (e.pageY - me._drag.start.y));
        }
        $(e.data).removeClass('dq-drag').css({ 'left': '', 'bottom': '', 'right': '' });
        DQ.document
            .unbind("mousemove", _resize$move);

        options.stop && options.stop.apply(me, [e, e.pageX - me._drag.start.x, e.pageY - me._drag.start.y]);
        delete me._drag;
        return false;
    }
}

DQ.UI.GroupBox = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    ///     コントロールのコレクションをグループ化して管理します。
    /// </sumamry>
    /// <param name="parent" type="DQ.Control">
    ///     WindowのコンテナとなるDQ.Controlオブジェクトを指定
    /// </param>
    /// <param name="element" type="jQuery">
    ///     メニューに追加するアイテムを指定
    /// </param>
    /// <param name="options" type="object">
    ///     Panelのオプションを指定
    /// </param>
    object == null && typeof options == 'string' && (object = options);
    object = object || $('<fieldset />').addClass('dq-groupbox');
    var o = (typeof object == "string") ? $('#' + object) : object;

    if (o[0].nodeName.toUpperCase() == "FIELDSET") {
        this.client = $('> div', o);
        o.attr('tabIndex') == null && o.attr('tabIndex', 100);
        o.css('width', '');
        this.base(parent, object);
    } else {
        var p = $('<fieldset />').addClass('dq-groupbox').css({ 'z-index': this.zIndex });
        this.client = o.removeClass('dq-groupbox');
        p.append(this.client);
        this.base(parent, p);
    }

    options = options || {};
    this._create(options);

    if (options.visible != null) {
        this.visible(option.visible);
    } else {
        this.show();
    }

}, {
    _dock: DQ.UI.DockStyle.Top,
    dock: function (value) {
        __dqsuper__(this, "dock", arguments);
    },
    dqui: "DQ.UI.GroupBox",
    _create: function (options) {
        //外枠
        options.width && this.width(options.width);
        options.height && this.height(options.height);
        options.cls && panel.addClass(options.cls);
        options.top && panel.css('top', options.top);
        options.left && panel.css('left', options.left);
        options.groupText && this.groupText(options.groupText);
    },
    zIndex: 10,
    groupText: function (text) {
        var legend = $('legend', this.obj);
        legend = legend.length == 0 ? $('<legend />').appendTo(this.obj) : legend;
        if (text != null) {
            legend.text(text);
        }

        return legend.text();
    }
});

DQ.UI._ClipBoard_ = [];
DQ.UI.Clipboard = function () {
}
DQ.UI.Clipboard.prototype = {
    clear: function () {
        delete DQ.UI._ClipBoard_;
    },
    setText: function (text) {
        DQ.UI._ClipBoard_["String"] = text;
    },
    getText: function () {
        return DQ.UI._ClipBoard_["String"];
    },
    setDataObject: function (obj) {
        DQ.UI._ClipBoard_["Object"] = obj;
    },
    getDataObject: function (type) {
        return DQ.UI._ClipBoard_["Object"];
    }
}

DQ.UI.Button = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    /// Buttonコントロールを作成します。
    /// </summary>
    ///<param name="options" type="object">
    /// オプションを指定
    ///</param>
    /// <param name="object" type="string/jQuery">
    /// Buttonコントロールの外殻を指すオブジェクトを指定
    /// </param>

    object = object || $('<div>');
    var o = (typeof object == "string") ? $('#' + object) : object;
    options = DQ.options(options, {
        cls: "dq-button",
        click: null,
        className: null,
        custom: false,
        dialogResult: DQ.UI.DialogResult.None,
        name: "",
        tabIndex: 0,
        useSpan: false, /* obslute */
        layout: DQ.UI.LayoutStyle.Absolute,
        width: o.width(),
        height: "",
        top: parseInt(o.css('top')),
        left: parseInt(o.css('left'))
    });


    this.base(parent, object);
    this.obj.addClass(options.cls);

    var 
            t = options.top || 0,
            l = options.left || 0,
            caption = $.trim(options.text || (o.text() != "" ? o.text() : "Button"));

    this.name = options.name;
    this.tabIndex(options.tabIndex);
    this.obj.css({ 'top': t, 'left': l });
    var cl = this.client = $('<button>');
    this.onClick = options.click;

    if (options.custom) {
        o.addClass('dq-button-image');
        if (options.className) {
            cl.addClass(options.className);
        } else {
            cl.addClass('image');
        }
    }

    this.obj.text('');
    this.text(caption);
    cl.appendTo(this.obj);
    if (options.width) {
        this.size(options.width, options.h);
    }

    var me = this;
    cl.bind("click", this, function (e) {
        var me = e.data;
        if (me._enabled) {
            var fm = me.findForm();
            if (fm && fm.dialogResult) {
                fm.dialogResult = me.dialogResult;
            }
            me.onClick && me.onClick(me, e);
        }
        e.stopImmediatePropagation();
        return false;
    });
    if (options.custom) {
        this.obj.hover(
            function () {
                if (me._enabled) {
                    if (me.custom) {
                        me.client.addClass('hover');
                    } else {
                        me.obj.addClass('dq-button-hover');
                    }
                }
            }, function () {
                me.obj.removeClass('dq-button-hover');
                me.client.removeClass('hover');
            });
    }
    options.layout == DQ.UI.LayoutStyle.Inline && o.addClass('dq-button-inline');
    this.obj.attr('tabIndex', -1);

}, {
    dialogResult: DQ.UI.DialogResult.None,
    dqid: "DQ.UI.Button",
    dispose: function () {
        /// <summary>
        /// リソースを破棄します
        /// </summary>
        this.obj.unbind('keydown focus mouseenter mouseleave')
        this.client.unbind("click mousedown");

        this.obj.remove();
        this.obj = null;
    },
    enabled: function (value) {
        __dqsuper__(this, "enabled", arguments);
        if (!arguments.length) {
            return this._enabled;
        }
        if (!this._enabled) {
            $('button', this.obj).attr('disabled', 'disabled');
        } else {
            $('button', this.obj).removeAttr('disabled');
        }
    },
    focus: function () {
        this.client.focus();
    },
    layout: DQ.UI.LayoutStyle.Absolute,
    onClick: null,
    performClick: function () {
        this.client.click();
    }
});

DQ.UI.RadioButton = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    /// RadioButtonコントロールを作成します。
    /// </summary>
    ///<param name="options" type="object">
    /// オプションを指定
    ///</param>
    /// <param name="object" type="string/jQuery">
    /// RadioButtonコントロールの外殻を指すオブジェクトを指定
    /// </param>

    object = object || $('<div>').addClass('dq-radiobutton');
    options = DQ.options(options, {
        height: 'auto',
        width: 'auto',
        top: 0,
        left: 0,
        custom: false,
        checked: false,
        name: "",
        text: ""
    });

    var o = (typeof object == "string") ? $('#' + object) : object,
            h = options.height,
            w = options.width,
            t = options.top,
            l = options.left,
            tab = options.tabIndex || 0,
            caption = $.trim(options.text || (o.text() != "" ? o.text() : "RadioButton")),
            name = options.name,
            custom = this._custom = options.custom;

    this.base(parent, object);

    this.tabIndex(tab);
    this.size(w, h);
    this.obj.css({ 'top': t, 'left': l });
    var cl = this.client = $('<input type="radio" />').attr('id', '_' + o.attr('id'));
    name && cl.attr('name', name);

    this.obj.text('');
    if (custom) {
        this.inner = $('<span class="dq-inner"></span>').appendTo(this.obj).attr('tabIndex', tab);
        cl.hide().appendTo(this.obj);
        this.inner
            .bind('mousedown', this, function (e) {
                e.data.inner.addClass('down');
            })
            .bind('click', this, function (e) {
                var me = e.data;
                me.inner.removeClass('down').addClass('checked');
                //me.client.trigger('click');
                me.checked(true);
            });
    } else {
        cl.appendTo(this.obj).attr('tabIndex', tab);
    }
    cl.bind("click", this, function (e) {
        var me = e.data;
        $('input[name=' + me.client.attr('name') + ']', me.parent.obj).prev().removeClass('checked');
        me.onCheckedChanged && me.onCheckedChanged(me);
    });
    this.obj.append('<label for="' + cl.attr('id') + '"/>');
    this.text(caption);

    this.obj.attr('tabIndex', '');
    options.checked && this.checked(true);

}, {
    dqid: "DQ.UI.RadioButton",
    dispose: function () {
        /// <summary>
        /// リソースを破棄します
        /// </summary>
        if (this.inner) {
            this.inner.unbind("click mousedown");
        }
        this.client.unbind("click change");
        this.obj.unbind('mouseenter mouseleave')
        this.obj.remove();
        this.obj = null;
    },
    enabled: function (value) {
        __dqsuper__(this, "enabled", arguments);
        if (!arguments.length) {
            return this._enabled;
        }
        if (!this._enabled) {
            $('input', this.obj).attr('disabled', 'disabled');
        } else {
            $('input', this.obj).removeAttr('disabled');
        }
    },
    focus: function () {
        this.client.focus();
    },
    onClick: null,
    onCheckedChanged: null,
    checked: function (value) {
        if (!arguments.length) {
            return this.client[0].checked;
        }
        if (this.client[0].checked != value) {
            if (value) {
                $('input[name=' + this.client.attr('name') + ']', this.parent.obj).prev().removeClass('checked');
                this._custom && this.inner.addClass('checked');
            } else {
                this._custom && this.inner.removeClass('checked');
            }
            this.client[0].checked = value;
            this.onCheckedChanged && this.onCheckedChanged(this);
        }
    },
    performClick: function () {
        this.onClick && this.onClick(this);
    },
    text: function (data) {
        if (!arguments.length) {
            return this._text;
        }
        this._text = data;
        this.client.next().text(data);
    }
},
    {});

DQ.UI.CheckBox = dqextend(DQ.Control, function (parent, options, object) {
    /// <summary>
    /// CheckBoxコントロールを作成します。
    /// </summary>
    ///<param name="options" type="object">
    /// オプションを指定
    ///</param>
    /// <param name="object" type="string/jQuery">
    /// CheckBoxコントロールの外殻を指すオブジェクトを指定
    /// </param>

    object = object || $('<div>').addClass('dq-checkbox');
    options = DQ.options(options, {
        custom: true,
        name: "",
        tabIndex: 0
    });

    var o = (typeof object == "string") ? $('#' + object) : object,
            h = options.height || o.css('height') || 'auto',
            w = options.width || o.css('width') || 'auto',
            t = options.top || parseInt(o.css('top')) || 0,
            l = options.left || parseInt(o.css('left')) || 0,
            tab = options.tabIndex,
            caption = $.trim(options.text || (o.text() != "" ? o.text() : "CheckBox")),
            name = options.name,
            custom = options.custom;

    this.base(parent, object);

    this.tabIndex(tab);
    this.size(w, h);
    this.obj.css({ 'top': t, 'left': l });
    var cl = this.client = $('<input type="checkbox" />').attr('id', '_' + o.attr('id'));
    name && cl.attr('name', name);

    this.obj.text('');
    //カスタムボタンの場合のイベントハンドラ
    if (custom) {
        this.inner = $('<span class="dq-inner"></span>').appendTo(this.obj).attr('tabIndex', tab);
        cl.hide().appendTo(this.obj);
        this.inner
            .bind('mousedown', this, function (e) {
                e.data.checked() ? e.data.inner.addClass('checkeddown') : e.data.inner.addClass('down');
            })
            .bind('change', this, function (e) {
            })
            .bind('click', this, function (e) {
                var me = e.data;
                me.inner.removeClass('down checkeddown');
                me.checked(!me.checked());
            });
    } else {
        cl.appendTo(this.obj).attr('tabIndex', tab);
    }
    cl
        .bind('change', this, function (e) {
        })
        .bind("click", this, function (e) {
            var me = e.data;
            me.onCheckedChanged && me.onCheckedChanged(me);
        });
    this.obj.append('<label for="' + cl.attr('id') + '"/>');
    this.text(caption);

    this.obj.attr('tabIndex', -1);

}, {
    dqid: "DQ.UI.CheckBox",
    dispose: function () {
        /// <summary>
        /// リソースを破棄します
        /// </summary>
        if (this.inner) {
            this.inner.unbind("click mousedown");
        }
        this.client.unbind("click change");
        this.obj.unbind('mouseenter mouseleave');
        this.obj.remove();
        this.obj = null;
    },
    enabled: function (value) {
        __dqsuper__(this, "enabled", arguments);
        if (!arguments.length) {
            return this._enabled;
        }
        if (!this._enabled) {
            $('input', this.obj).attr('disabled', 'disabled');
        } else {
            $('input', this.obj).removeAttr('disabled');
        }
    },
    focus: function () {
        this.client.focus();
    },
    onClick: null,
    onCheckedChanged: null,
    checked: function (value) {
        if (!arguments.length) {
            return this.client[0].checked;
        }
        if (this.client[0].checked != value) {
            this.client[0].checked = value;
            this.inner && (value ? this.inner.addClass('checked') : this.inner.removeClass('checked'));
            this.onCheckedChanged && this.onCheckedChanged(this);
        };
    },
    performClick: function () {
        this.onClick && this.onClick(this);
    },
    text: function (data) {
        if (!arguments.length) {
            return this._text;
        }
        this._text = data;
        this.client.next().text(data);
    }
},
{});

DQ._ctrl_['dq-form'] = DQ.UI.Form;
DQ._ctrl_['dq-panel'] = DQ.UI.Panel;
DQ._ctrl_['dq-splitter'] = DQ.UI.Splitter;
DQ._ctrl_['dq-toolbar'] = DQ.UI.Toolbar;
DQ._ctrl_['dq-menustrip'] = DQ.UI.MenuStrip;
DQ._ctrl_['dq-statusstrip'] = DQ.UI.StatusStrip;
DQ._ctrl_['dq-groupbox'] = DQ.UI.GroupBox;
DQ._ctrl_['dq-button'] = DQ.UI.Button;
DQ._ctrl_['dq-checkbox'] = DQ.UI.CheckBox;
DQ._ctrl_['dq-radiobutton'] = DQ.UI.RadioButton;


DQ.afterLoad(function () {
    if (typeof debugConsole == "undefined") {
        var _context_open = function (e) {
            if (e && e.button == 2 && DQ._page.contextMenu) {
                var me = DQ._page.obj;
                DQ._page.contextMenu.obj.css({ 'top': e.pageY, 'left': e.pageX });
                DQ._page.contextMenu.show();
                e.stopImmediatePropagation();
            }
            return true;
        }
        document.oncontextmenu = _context_open;

        DQ.document.mouseup(_context_open);
        //DQ.document.mousedown(function (e) { DQ._page.visible && DQ._page.contextMenu.hide(); });
    }

    if (DQ.queryString["debug"] == "true") {
        debugConsole = new DQ.UI.Form(DQ.page(), {
            windowState: DQ.UI.FormWindowState.Normal,
            initializeComponent: function (sender) {
                sender.text("debug console");
                sender.debug = debug;
                sender.onClosing = function (sender) { sender.hide(); return true; }
            },
            onLoaded: function (sender) {
                //sender.windowState(DQ.UI.FormWindowState.Normal);
            }
        }, $('<div class="dq-form" style="width:480px; height:370px;" />'));
        debugConsole.show();
    }
    DQ.page().onLoad.apply(DQ._page);
});
